@@ -196,6 +196,11 @@ public class ClientPooledConnection implements PooledConnection {
         return this.statementCache != null;
     }
 
+    //
+    // This method in java.lang.Object was deprecated as of build 167
+    // of JDK 9. See DERBY-6932.
+    //
+    @SuppressWarnings("deprecation")
     protected void finalize() throws Throwable {
         if (logWriter_ != null) {
             logWriter_.traceEntry(this, "finalize");
