@@ -21,6 +21,7 @@
 package org.apache.derby.client.am;
 
 import java.math.BigDecimal;
+import java.math.BigInteger;
 import org.apache.derby.shared.common.reference.SQLState;
 import org.apache.derby.shared.common.i18n.MessageUtil;
 
@@ -145,10 +146,12 @@ public class Decimal {
      *
      * @throws IllegalArgumentException if the specified representation is not recognized.
      */
-    public static final java.math.BigDecimal getBigDecimal(byte[] buffer,
-                                                           int offset,
-                                                           int precision,
-                                                           int scale) throws java.io.UnsupportedEncodingException {
+    static BigDecimal getBigDecimal(
+            byte[] buffer,
+            int offset,
+            int precision,
+            int scale) {
+
         // The byte-length of a packed decimal with precision <code>p</code> is always <code>p/2 + 1</code>
         int length = precision / 2 + 1;
 
@@ -193,7 +196,7 @@ public class Decimal {
             magnitude[10] = (byte) (value[2] >>> 8);
             magnitude[11] = (byte) (value[2]);
 
-            return new java.math.BigDecimal(new java.math.BigInteger(signum, magnitude), scale);
+            return new BigDecimal(new BigInteger(signum, magnitude), scale);
         } else if (precision <= 31) {
             // get the value of last 9 digits (5 bytes).
             int lo = packedNybblesToInt(buffer, offset, (length - 5) * 2, 9);
@@ -226,10 +229,10 @@ public class Decimal {
             magnitude[14] = (byte) (value[3] >>> 8);
             magnitude[15] = (byte) (value[3]);
 
-            return new java.math.BigDecimal(new java.math.BigInteger(signum, magnitude), scale);
+            return new BigDecimal(new BigInteger(signum, magnitude), scale);
         } else {
             // throw an exception here if nibbles is greater than 31
-            throw new java.lang.IllegalArgumentException(
+            throw new IllegalArgumentException(
                 msgutil.getTextMessage(SQLState.DECIMAL_TOO_MANY_DIGITS));
         }
     }
@@ -239,10 +242,12 @@ public class Decimal {
      *
      * @throws IllegalArgumentException if the specified representation is not recognized.
      */
-    public static final double getDouble(byte[] buffer,
-                                         int offset,
-                                         int precision,
-                                         int scale) throws java.io.UnsupportedEncodingException {
+    static double getDouble(
+            byte[] buffer,
+            int offset,
+            int precision,
+            int scale) {
+
         // The byte-length of a packed decimal with precision <code>p</code> is always <code>p/2 + 1</code>
         int length = precision / 2 + 1;
 
@@ -291,7 +296,7 @@ public class Decimal {
                     hi * Math.pow(10, 27 - scale));
         } else {
             // throw an exception here if nibbles is greater than 31
-            throw new java.lang.IllegalArgumentException(
+            throw new IllegalArgumentException(
                 msgutil.getTextMessage(SQLState.DECIMAL_TOO_MANY_DIGITS));
         }
     }
@@ -302,13 +307,15 @@ public class Decimal {
      * @throws IllegalArgumentException if the specified representation is not recognized.
      * @throws ArithmeticException if value is too large for a long
      */
-    public static final long getLong(byte[] buffer,
-                                     int offset,
-                                     int precision,
-                                     int scale) throws java.io.UnsupportedEncodingException {
+    static long getLong(
+            byte[] buffer,
+            int offset,
+            int precision,
+            int scale) {
+
         if (precision > 31) {
             // throw an exception here if nibbles is greater than 31
-            throw new java.lang.IllegalArgumentException(
+            throw new IllegalArgumentException(
                 msgutil.getTextMessage(SQLState.DECIMAL_TOO_MANY_DIGITS));
         }
 
@@ -346,7 +353,7 @@ public class Decimal {
      */
     public static final int bigDecimalToPackedDecimalBytes(byte[] buffer,
                                                            int offset,
-                                                           java.math.BigDecimal b,
+                                                           BigDecimal b,
                                                            int declaredPrecision,
                                                            int declaredScale)
             throws SqlException {
@@ -365,7 +372,7 @@ public class Decimal {
         if (bigPrecision > 31) {
             throw new SqlException(null,
                 new ClientMessageId(SQLState.LANG_OUTSIDE_RANGE_FOR_DATATYPE),
-                "packed decimal", new SqlCode(-405));
+                new SqlCode(-405), "packed decimal");
         }
 
         int bigScale = b.scale();
@@ -376,7 +383,7 @@ public class Decimal {
             if (bigWholeIntegerLength > declaredWholeIntegerLength) {
                 throw new SqlException(null,
                     new ClientMessageId(SQLState.NUMERIC_OVERFLOW),
-                    b.toString(), "packed decimal", new SqlCode(-413));
+                    new SqlCode(-413), b.toString(), "packed decimal");
             }
         }
 
