@@ -22,19 +22,22 @@
 package org.apache.derby.client.am;
 
 import java.sql.DataTruncation;
+import java.sql.Types;
+import java.util.Locale;
 import org.apache.derby.client.net.Typdef;
 import org.apache.derby.shared.common.error.ExceptionSeverity;
 import org.apache.derby.shared.common.reference.SQLState;
+import org.apache.derby.shared.common.error.MessageUtils;
 
 public abstract class Sqlca {
 
     // Indexes into sqlErrd_
-    public  static  final   int HIGH_ORDER_ROW_COUNT = 0;
-    public  static  final   int LOW_ORDER_ROW_COUNT = 1;
-    public  static  final   int LOW_ORDER_UPDATE_COUNT = 2;
-    public  static  final   int HIGH_ORDER_UPDATE_COUNT = 3;
+    private  static  final   int HIGH_ORDER_ROW_COUNT = 0;
+    private  static  final   int LOW_ORDER_ROW_COUNT = 1;
+    private  static  final   int LOW_ORDER_UPDATE_COUNT = 2;
+    private  static  final   int HIGH_ORDER_UPDATE_COUNT = 3;
     public  static  final   int SQL_ERR_LENGTH = 6;
-    transient protected Connection connection_;
+    transient private ClientConnection connection_;
     SqlException exceptionThrownOnStoredProcInvocation_;
     boolean messageTextRetrievedContainsTokensOnly_ = true;
 
@@ -43,13 +46,13 @@ public abstract class Sqlca {
     /** A string representation of <code>sqlErrmcBytes_</code>. */
     private String sqlErrmc_;
     /** Array of errmc strings for each message in the chain. */
-    protected String[] sqlErrmcMessages_;
+    private String[] sqlErrmcMessages_;
     /** SQL states for all the messages in the exception chain. */
     private String[] sqlStates_;
     // contain an error token
-    protected String sqlErrp_;        // function name issuing error
+    private String sqlErrp_;        // function name issuing error
     protected int[] sqlErrd_;        // 6 diagnostic Information
-    protected char[] sqlWarn_;        // 11 warning Flags
+    private String sqlWarn_;        // 11 warning Flags
     protected String sqlState_;       // SQLSTATE
 
     // raw sqlca data fields before unicode conversion
@@ -57,18 +60,8 @@ public abstract class Sqlca {
     protected byte[] sqlErrpBytes_;
     protected byte[] sqlWarnBytes_;
     
-    protected int sqlErrmcCcsid_;
-    protected boolean containsSqlcax_ = true;
-    protected long rowsetRowCount_;
-
-    /**
-     * Character sequence that separates the different messages in the errmc.
-     * @see org.apache.derby.catalog.SystemProcedures#SQLERRMC_MESSAGE_DELIMITER
-     */
-    private static final String sqlErrmcDelimiter__ = "\u0014\u0014\u0014";
-
-    /** Token delimiter for SQLERRMC. */
-    private final static String SQLERRMC_TOKEN_DELIMITER = "\u0014";
+    private boolean containsSqlcax_ = true;
+    private long rowsetRowCount_;
 
     // JDK stack trace calls e.getMessage(), so we must set some state on the sqlca that says return tokens only.
     private boolean returnTokensOnlyInMessageText_ = false;
@@ -79,7 +72,7 @@ public abstract class Sqlca {
      * procedure to get the same message). */
     private String[] cachedMessages;
 
-    protected Sqlca(org.apache.derby.client.am.Connection connection) {
+    protected Sqlca(ClientConnection connection) {
         connection_ = connection;
         agent_ = connection_ != null ? connection_.agent_ : null;
     }
@@ -171,7 +164,7 @@ public abstract class Sqlca {
         int indx;
         for (indx = 0; indx < sqlErrmcMessages_.length - 1; indx++) {
             buffer.append(sqlErrmcMessages_[indx]);
-            buffer.append(sqlErrmcDelimiter__);
+            buffer.append(MessageUtils.SQLERRMC_MESSAGE_DELIMITER);
             // all but the first message should be preceded by the SQL state
             // and a colon (see DRDAConnThread.buildTokenizedSqlerrmc() on the
             // server)
@@ -206,18 +199,11 @@ public abstract class Sqlca {
             return null;
         }
 
-        try {
-            sqlErrp_ = bytes2String(sqlErrpBytes_,
-                    0,
-                    sqlErrpBytes_.length);
-            return sqlErrp_;
-        } catch (java.io.UnsupportedEncodingException e) {
-            // leave sqlErrp as null.
-            return null;
-        }
+        sqlErrp_ = bytes2String(sqlErrpBytes_, 0, sqlErrpBytes_.length);
+        return sqlErrp_;
     }
 
-    public int[] getSqlErrd() {
+    private int[] getSqlErrd() {
         if (sqlErrd_ != null) {
             return sqlErrd_;
         }
@@ -226,22 +212,21 @@ public abstract class Sqlca {
         return sqlErrd_;
     }
 
-    synchronized public char[] getSqlWarn() {
-        if (sqlWarn_ != null) {
-            return sqlWarn_;
-        }
+    String formatSqlErrd() {
+        return Utils.getStringFromInts(getSqlErrd());
+    }
 
-        try {
-            if (sqlWarnBytes_ == null) {
-                sqlWarn_ = new char[]{' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '}; // 11 blank.
+    private final static String elevenBlanks = "           ";
+
+    synchronized public String getSqlWarn() {
+        if (sqlWarn_ == null) {
+            if (sqlWarnBytes_ != null) {
+                sqlWarn_ = bytes2String(sqlWarnBytes_, 0, sqlWarnBytes_.length);
             } else {
-                sqlWarn_ = bytes2String(sqlWarnBytes_, 0, sqlWarnBytes_.length).toCharArray();
+                sqlWarn_ = elevenBlanks;
             }
-            return sqlWarn_;
-        } catch (java.io.UnsupportedEncodingException e) {
-            sqlWarn_ = new char[]{' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '}; // 11 blank.
-            return sqlWarn_;
         }
+        return sqlWarn_;
     }
 
     synchronized public String getSqlState() {
@@ -262,6 +247,13 @@ public abstract class Sqlca {
         return getSqlState();
     }
 
+    public Object [] getArgs(int messageNumber) {
+        if (sqlErrmcMessages_ != null)
+	    return MessageUtils.getArgs(getSqlState(messageNumber),
+                                        sqlErrmcMessages_[messageNumber] );
+        return null;
+    }
+
     // Gets the formatted message, can throw an exception.
     private String getMessage(int messageNumber) throws SqlException {
         // should this be traced to see if we are calling a stored proc?
@@ -273,11 +265,14 @@ public abstract class Sqlca {
             return getUnformattedMessage(messageNumber);
         }
 
-        CallableStatement cs = null;
+        ClientCallableStatement cs = null;
         synchronized (connection_) {
             try {
                 cs = connection_.prepareMessageProc("call SYSIBM.SQLCAMESSAGE(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
-
+                // Cannot let this statement commit the transaction. Otherwise, 
+                // calling getWarnings while navigating a ResultSet will 
+                // release and invalidate locators used by the cursor.
+                cs.isAutoCommittableStatement_ = false;
                 String errmc = null;
                 String sqlState = null;
 
@@ -302,19 +297,19 @@ public abstract class Sqlca {
                 cs.setIntX(9, getSqlErrd()[4]);
                 cs.setIntX(10, getSqlErrd()[5]);
                 // SQLWarn: SQL warning flags.
-                cs.setStringX(11, new String(getSqlWarn()));
+                cs.setStringX(11, getSqlWarn());
                 // SQLState: standard SQL state.
                 cs.setStringX(12, sqlState);
                 // MessageFileName: Not used by our driver, so set to null.
                 cs.setStringX(13, null);
                 // Locale: language preference requested for the return error message.
-                cs.setStringX(14, java.util.Locale.getDefault().toString());
+                cs.setStringX(14, Locale.getDefault().toString());
                 // server could return a locale different from what we requested
-                cs.registerOutParameterX(14, java.sql.Types.VARCHAR);
+                cs.registerOutParameterX(14, Types.VARCHAR);
                 // Message: error message returned from SQLCAMessage stored procedure.
-                cs.registerOutParameterX(15, java.sql.Types.LONGVARCHAR);
+                cs.registerOutParameterX(15, Types.LONGVARCHAR);
                 // RCode: return code from SQLCAMessage stored procedure.
-                cs.registerOutParameterX(16, java.sql.Types.INTEGER);
+                cs.registerOutParameterX(16, Types.INTEGER);
                 cs.executeX();
 
                 if (cs.getIntX(16) == 0) {
@@ -407,15 +402,6 @@ public abstract class Sqlca {
         }
     }
 
-    public boolean includesSqlCode(int[] codes) {
-        for (int i = 0; i < codes.length; i++) {
-            if (codes[i] == getSqlCode()) {
-                return true;
-            }
-        }
-        return false;
-    }
-
     /**
      * Get a {@code java.sql.DataTruncation} warning based on the information
      * in this SQLCA.
@@ -425,7 +411,7 @@ public abstract class Sqlca {
     DataTruncation getDataTruncation() {
         // The network server has serialized all the parameters needed by
         // the constructor in the SQLERRMC field.
-        String[] tokens = getSqlErrmc().split(SQLERRMC_TOKEN_DELIMITER);
+        String[] tokens = getSqlErrmc().split(MessageUtils.SQLERRMC_TOKEN_DELIMITER);
         return new DataTruncation(
                 Integer.parseInt(tokens[0]),                // index
                 Boolean.valueOf(tokens[1]).booleanValue(),  // parameter
@@ -448,30 +434,25 @@ public abstract class Sqlca {
             return;
         }
 
-        try {
-            // tokenize and convert tokenBytes
-            String fullString = bytes2String(tokenBytes, 0, length);
-            String[] tokens = fullString.split("\\u0014{3}");
-            String[] states = new String[tokens.length];
-            states[0] = getSqlState();
-            for (int i = 1; i < tokens.length; i++) {
-                // All but the first message are preceded by the SQL state
-                // (five characters) and a colon. Extract the SQL state and
-                // clean up the token. See
-                // DRDAConnThread.buildTokenizedSqlerrmc() for more details.
-                int colonpos = tokens[i].indexOf(":");
-                states[i] = tokens[i].substring(0, colonpos);
-                tokens[i] = tokens[i].substring(colonpos + 1);
-            }
-            sqlStates_ = states;
-            sqlErrmcMessages_ = tokens;
-        } catch (java.io.UnsupportedEncodingException e) {
-            /* do nothing, the arrays continue to be null */
+        // tokenize and convert tokenBytes
+        String fullString = bytes2String(tokenBytes, 0, length);
+        String[] tokens = fullString.split("\\u0014{3}");
+        String[] states = new String[tokens.length];
+        states[0] = getSqlState();
+        for (int i = 1; i < tokens.length; i++) {
+            // All but the first message are preceded by the SQL state
+            // (five characters) and a colon. Extract the SQL state and
+            // clean up the token. See
+            // DRDAConnThread.buildTokenizedSqlerrmc() for more details.
+            int colonpos = tokens[i].indexOf(":");
+            states[i] = tokens[i].substring(0, colonpos);
+            tokens[i] = tokens[i].substring(colonpos + 1);
         }
+        sqlStates_ = states;
+        sqlErrmcMessages_ = tokens;
     }
 
-    protected String bytes2String(byte[] bytes, int offset, int length)
-            throws java.io.UnsupportedEncodingException {
+    protected String bytes2String(byte[] bytes, int offset, int length) {
         // Network server uses utf8 encoding
         return new String(bytes, offset, length, Typdef.UTF8ENCODING);
     }
@@ -480,14 +461,15 @@ public abstract class Sqlca {
         if (sqlErrd_ == null) {
             return 0L;
         }
-        long    result = sqlErrd_[ LOW_ORDER_UPDATE_COUNT ];
+        long    result = getSqlErrd()[ LOW_ORDER_UPDATE_COUNT ];
         result &= 0xFFFFFFFFL;
-        result |= ((long) sqlErrd_[ HIGH_ORDER_UPDATE_COUNT ] << 32);
+        result |= ((long) getSqlErrd()[ HIGH_ORDER_UPDATE_COUNT ] << 32);
         return result;
     }
 
-    public long getRowCount() throws org.apache.derby.client.am.DisconnectException {
-        return ((long) sqlErrd_[ HIGH_ORDER_ROW_COUNT ] << 32) + sqlErrd_[ LOW_ORDER_ROW_COUNT ];
+    public long getRowCount() throws DisconnectException {
+        return ((long) getSqlErrd()[ HIGH_ORDER_ROW_COUNT ] << 32) +
+                getSqlErrd()[ LOW_ORDER_ROW_COUNT ];
     }
 
     public void setContainsSqlcax(boolean containsSqlcax) {
@@ -498,14 +480,13 @@ public abstract class Sqlca {
         return containsSqlcax_;
     }
 
-    public void resetRowsetSqlca(org.apache.derby.client.am.Connection connection,
+    public void resetRowsetSqlca(ClientConnection connection,
                                  int sqlCode,
-                                 String sqlState,
-                                 byte[] sqlErrpBytes) {
+                                 String sqlState) {
         connection_ = connection;
         sqlCode_ = sqlCode;
         sqlState_ = sqlState;
-        sqlErrpBytes_ = sqlErrpBytes;
+        sqlErrpBytes_ = null;
     }
 
     public void setRowsetRowCount(long rowCount) {
