@@ -26,16 +26,17 @@ import java.net.Inet6Address;
 import java.net.InetAddress;
 import java.net.URL;
 import java.security.AccessController;
-import java.security.PrivilegedExceptionAction;
+import java.security.CodeSource;
+import java.security.PrivilegedAction;
+import java.security.ProtectionDomain;
 import java.util.Properties;
 import org.apache.derby.iapi.reference.Property;
-import org.apache.derby.iapi.services.info.JVMInfo;
 import org.apache.derby.iapi.services.property.PropertyUtil;
 import org.apache.derby.impl.drda.NetworkServerControlImpl;
 
 /** 
     NetworkServerControl provides the ability to start a Network Server or 
-    connect to a running Network Server to shutdown, configure or retreive 
+    connect to a running Network Server to shutdown, configure or retrieve 
     diagnostic information.  With the exception of ping, these commands 
     can  only be performed from the  machine on which the server is running.  
     Commands can be performed from  the command line with the following 
@@ -43,22 +44,24 @@ import org.apache.derby.impl.drda.NetworkServerControlImpl;
 
     <P>
     <UL>
-    <LI>start [-h &lt;host>] [-p &lt;portnumber>] [-ssl &lt;sslmode&gt;]:  This starts the network
-    server on the port/host specified or on localhost, port 1527 if no
+    <LI>start [-h &lt;host&gt;] [-p &lt;portnumber&gt;] [-ssl &lt;sslmode&gt;]:  This starts the Network
+    Server on the port/host specified or on localhost, port 1527 if no
     host/port is specified and no properties are set to override the 
-    defaults. By default Network Server will only listen for 
+    defaults. By default a security manager with a default security policy will 
+    be installed. The default security policy file is called server.policy. 
+    By default the Network Server will only listen for 
     connections from the machine on which it is running. 
-    Use -h 0.0.0.0 to listen on all interfaces or -h &lt;hostname> to listen 
+    Use -h 0.0.0.0 to listen on all interfaces or -h &lt;hostname&gt; to listen 
     on a specific interface on a  multiple IP machine. 
     For documentation on &lt;sslmode&gt;, consult the Server and Administration Guide.</LI>
 
-    <LI>shutdown [-h &lt;host&gt;][-p &lt;portnumber&gt;] [-ssl &lt;sslmode&gt;] [-user &lt;username&gt;] [-password &lt;password&gt;]: This shutdowns the network server with given user credentials on the host and port specified or on the local host and port 1527(default) if no host or port is specified.  </LI> 
+    <LI>shutdown [-h &lt;host&gt;][-p &lt;portnumber&gt;] [-ssl &lt;sslmode&gt;] [-user &lt;username&gt;] [-password &lt;password&gt;]: This shutdowns the Network Server with given user credentials on the host and port specified or on the local host and port 1527(default) if no host or port is specified.  </LI> 
 
-    <LI>ping [-h &lt;host>] [-p &lt;portnumber>] [-ssl &lt;sslmode&gt;]
+    <LI>ping [-h &lt;host&gt;] [-p &lt;portnumber&gt;] [-ssl &lt;sslmode&gt;]
     This will test whether the Network Server is up.
     </LI>
 
-    <LI>sysinfo [-h &lt;host>] [-p &lt;portnumber>] [-ssl &lt;sslmode&gt;]:  This prints 
+    <LI>sysinfo [-h &lt;host&gt;] [-p &lt;portnumber&gt;] [-ssl &lt;sslmode&gt;]:  This prints 
     classpath and version information about the Network Server, 
     the JVM and the Derby engine. 
     </LI>
@@ -68,28 +71,28 @@ import org.apache.derby.impl.drda.NetworkServerControlImpl;
     prepared statements, and memory usage for the running Network Server.
     </LI>
 
-    <LI>logconnections {on | off} [-h &lt;host>] [-p &lt;portnumber>] [-ssl &lt;sslmode&gt;]:  
+    <LI>logconnections {on | off} [-h &lt;host&gt;] [-p &lt;portnumber&gt;] [-ssl &lt;sslmode&gt;]:  
     This turns logging of connections on or off.  
     Connections are logged to derby.log. 
     Default is off.</LI>
 
-    <LI>maxthreads &lt;max> [-h &lt;host>][-p &lt;portnumber>] [-ssl &lt;sslmode&gt;]:  
+    <LI>maxthreads &lt;max&gt; [-h &lt;host&gt;][-p &lt;portnumber&gt;] [-ssl &lt;sslmode&gt;]:  
     This sets the maximum number of threads that can be used for connections. 
     Default 0 (unlimitted).
     </LI>
 
-    <LI>timeslice &lt;milliseconds> [-h &lt;host>][-p &lt;portnumber>] [-ssl &lt;sslmode&gt;]: 
+    <LI>timeslice &lt;milliseconds&gt; [-h &lt;host&gt;][-p &lt;portnumber&gt;] [-ssl &lt;sslmode&gt;]: 
     This sets the time each session can have using a connection thread 
     before yielding to a waiting session. Default is 0 (no yeild).
     
     </LI>
 
-    <LI>trace {on | off} [-s &lt;session id>] [-h &lt;host>] [-p &lt;portnumber>]  [-ssl &lt;sslmode&gt;]: 
+    <LI>trace {on | off} [-s &lt;session id&gt;] [-h &lt;host&gt;] [-p &lt;portnumber&gt;]  [-ssl &lt;sslmode&gt;]: 
     This turns drda tracing on or off for the specified session or if no 
     session is  specified for all sessions. Default is off</LI>
 
 
-    <LI>tracedirectory &lt;tracedirectory> [-h &lt;host>] [-p &lt;portnumber>]  [-ssl &lt;sslmode&gt;]: 
+    <LI>tracedirectory &lt;tracedirectory&gt; [-h &lt;host&gt;] [-p &lt;portnumber&gt;]  [-ssl &lt;sslmode&gt;]: 
     This changes where new trace files will be placed. 
     For sessions with tracing already turned on,  
     trace files remain in the previous location. 
@@ -105,13 +108,13 @@ import org.apache.derby.impl.drda.NetworkServerControlImpl;
     The following is a list of properties that can be set for 
     NetworkServerControl:
 
-    <UL><LI>derby.drda.portNumber=&lt;port number>: This property 
+    <UL><LI>derby.drda.portNumber=&lt;port number&gt;: This property 
     indicates which port should be used for the Network Server. </LI>
 
-    <LI>derby.drda.host=&lt;host name  or ip address >: This property 
+    <LI>derby.drda.host=&lt;host name  or ip address &gt;: This property 
     indicates the ip address to which NetworkServerControl should connect. </LI>
 
-    <LI>derby.drda.traceDirectory=&lt;trace directory>: This property 
+    <LI>derby.drda.traceDirectory=&lt;trace directory&gt;: This property 
     indicates where to put trace files. </LI>
 
     <LI>derby.drda.traceAll=true:  This property turns on tracing for
@@ -120,23 +123,23 @@ import org.apache.derby.impl.drda.NetworkServerControlImpl;
     <LI>derby.drda.logConnections=true:  This property turns on logging
     of connections. Default is connections are not logged.</LI>
 
-    <LI>derby.drda.minThreads=&lt;value>: If this property
-    is set, the &lt;value> number of threads will be created when the Network Server is
+    <LI>derby.drda.minThreads=&lt;value&gt;: If this property
+    is set, the &lt;value&gt; number of threads will be created when the Network Server is
     booted. </LI>
 
-    <LI>derby.drda.maxThreads=&lt;value>: If this property
-    is set, the &lt;value> is the maximum number of connection threads that will be 
+    <LI>derby.drda.maxThreads=&lt;value&gt;: If this property
+    is set, the &lt;value&gt; is the maximum number of connection threads that will be 
     created.  If a session starts when there are no connection threads available
     and the maximum number of threads has been reached, it will wait until a 
     conection thread becomes available. </LI>
 
-    <LI>derby.drda.timeSlice=&lt;milliseconds>: If this property
+    <LI>derby.drda.timeSlice=&lt;milliseconds&gt;: If this property
     is set, the connection threads will not check for waiting sessions until the
-    current session has been working for &lt;milliseconds>.  
+    current session has been working for &lt;milliseconds&gt;.  
     A value of 0 causes the thread to work on the current session until the 
     session exits. If this property is not set, the default value is 0. </LI>
 
-    <LI>derby.drda.sslMode=&lt;sslmode&gt: This property sets the SSL
+    <LI>derby.drda.sslMode=&lt;sslmode&gt; This property sets the SSL
     mode of the server.
     
 </LI>
@@ -157,8 +160,8 @@ import org.apache.derby.impl.drda.NetworkServerControlImpl;
     </PRE>
     </P>
 
-    <P>This is an example of starting and then shutting down the network 
-       server on port 1621 on machine myhost   
+    <P>This is an example of starting and then shutting down the Network 
+       Server on port 1621 on machine myhost   
     <PRE>
     java org.apache.derby.drda.NetworkServerControl  start -h myhost -p 1621
     java org.apache.derby.drda.NetworkServerControl  shutdown -h myhost -p 1621
@@ -203,7 +206,7 @@ public class NetworkServerControl{
      * @param address     The IP address of the Network Server host.
      *                     address cannot be null.
      *
-     * @param portNumber  port number server is to used. If <= 0,
+     * @param portNumber  port number server is to used. If &lt;= 0,
      *                    default port number is used
      *
      * @param userName    The user name for actions requiring authorization.
@@ -255,7 +258,7 @@ public class NetworkServerControl{
      * @param address     The IP address of the Network Server host.
      *                     address cannot be null.
 
-     * @param portNumber  port number server is to used. If <= 0,
+     * @param portNumber  port number server is to used. If &lt;= 0,
      *                    default port number is used
      *                       
      * @throws             Exception on error
@@ -316,17 +319,15 @@ public class NetworkServerControl{
             int     command = server.parseArgs( args );
 
             // Java 7 and above: file permission restriction
-            if (command == NetworkServerControlImpl.COMMAND_START &&
-                    JVMInfo.JDK_ID >= JVMInfo.J2SE_17) {
+            if (command == NetworkServerControlImpl.COMMAND_START) {
                 try {
-                    AccessController.doPrivileged(
-                        new PrivilegedExceptionAction<Void>() {
-                            public Void run() throws Exception {
-                                System.setProperty(
-                                    Property.SERVER_STARTED_FROM_CMD_LINE,
-                                    "true");
-                                return null;
-                            }});
+                    AccessController.doPrivileged((PrivilegedAction<Void>) () ->
+                    {
+                        System.setProperty(
+                                Property.SERVER_STARTED_FROM_CMD_LINE,
+                                "true");
+                        return null;
+                    });
                 } catch (Exception e) {
                     server.consoleExceptionPrintTrace(e);
                     System.exit(1);
@@ -377,9 +378,10 @@ public class NetworkServerControl{
      ************************************************************************
      **/
 
-    /** Start a Network Server
-     *  This method will launch a separate thread and start Network Server.
+    /** Start a Network Server.
+     *  This method will launch a separate thread and start a Network Server.
      *  This method  may return before the server is ready to accept connections.
+     *  This will also install a security manager with a default security policy.
      *  Use the ping method to verify that the server has started.
      *
      * <P>
@@ -417,7 +419,7 @@ public class NetworkServerControl{
     }
 
     /**
-     * Check if Network Server is started
+     * Check if the Network Server is started.
      * Excecutes and returns without error if the server has started
      *
      * @exception Exception throws an exception if an error occurs
@@ -521,7 +523,7 @@ public class NetworkServerControl{
      * should also be set so that clients will yield appropriately.
      *
      * @param max       maximum number of connection threads.
-     *                  If <= 0, connection threads will be created when 
+     *                  If &lt;= 0, connection threads will be created when 
      *                  there are no free connection threads.
      *
      * @exception Exception throws an exception if an error occurs
@@ -549,10 +551,10 @@ public class NetworkServerControl{
 
     /**
      * Set Network Server connection time slice parameter.  
-     * This should be set and is only relevant if setMaxThreads > 0.
+     * This should be set and is only relevant if setMaxThreads &gt; 0.
      *
      * @param timeslice number of milliseconds given to each session before yielding to
-     *                      another session, if <=0, never yield.
+     *                      another session, if &lt;=0, never yield.
      *
      * @exception Exception throws an exception if an error occurs
      * @see #setMaxThreads
@@ -598,7 +600,7 @@ public class NetworkServerControl{
           
     protected void setClientLocale(String locale)
     {
-        serverImpl.clientLocale = locale;
+        serverImpl.setClientLocale( locale );
     }
 
     /**
@@ -699,12 +701,12 @@ public class NetworkServerControl{
 
         System.setSecurityManager( securityManager );
 
-        //
         // Report success.
-        //
-        String  successMessage = server.localizeMessage( "DRDA_SecurityInstalled.I", null );
-        
-        server.consoleMessage( successMessage, true );
+        if (securityManager.equals(System.getSecurityManager())) {
+            String successMessage = server.localizeMessage(
+                    "DRDA_SecurityInstalled.I", null);
+            server.consoleMessage(successMessage, true);
+        }
     }
 
     /**
@@ -791,8 +793,21 @@ public class NetworkServerControl{
     private static  String  getCodeSourcePrefix( NetworkServerControlImpl server )
         throws Exception
     {
-        String  derbyNetURL = NetworkServerControl.class.getProtectionDomain().getCodeSource().getLocation().toExternalForm();
-        int         idx = derbyNetURL.indexOf( DERBYNET_JAR );
+        // Note: This method is expected to run only when no security manager
+        //       has been installed, hence no use of privileged blocks.
+        ProtectionDomain pd = NetworkServerControl.class.getProtectionDomain();
+        CodeSource cs = pd.getCodeSource();
+        if (cs == null) {
+            return null;
+        }
+        URL url = cs.getLocation();
+        if (url == null) {
+            return null;
+        }
+        // Replace in "file://some", but not in "file:///some".
+        String extForm = url.toExternalForm().replaceFirst(
+                "^file://([^/].*)", "file:////$1");
+        int idx = extForm.indexOf(DERBYNET_JAR);
 
         //
         // If the customer isn't running against jar files, our Basic policy
@@ -809,7 +824,7 @@ public class NetworkServerControl{
         //
         // Otherwise, we have the directory prefix for our url.
         //
-        String  directoryPrefix = derbyNetURL.substring( 0, idx );
+        String directoryPrefix = extForm.substring(0, idx);
 
         return directoryPrefix;
     }
