@@ -31,21 +31,29 @@ import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Timestamp;
+import java.util.ArrayList;
 import java.util.NoSuchElementException;
 import java.util.Random;
 import java.util.StringTokenizer;
 
+import org.apache.derby.catalog.AliasInfo;
 import org.apache.derby.iapi.db.Factory;
 import org.apache.derby.iapi.db.PropertyInfo;
 import org.apache.derby.iapi.error.PublicAPI;
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.Property;
 import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.security.Securable;
+import org.apache.derby.iapi.security.SecurityUtil;
 import org.apache.derby.iapi.services.cache.CacheManager;
 import org.apache.derby.iapi.services.i18n.MessageService;
+import org.apache.derby.iapi.services.monitor.ModuleFactory;
+import org.apache.derby.iapi.services.monitor.Monitor;
 import org.apache.derby.iapi.services.property.PropertyUtil;
+import org.apache.derby.iapi.sql.conn.Authorizer;
 import org.apache.derby.iapi.sql.conn.ConnectionUtil;
 import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
+import org.apache.derby.iapi.sql.Activation;
 import org.apache.derby.iapi.util.IdUtil;
 import org.apache.derby.iapi.util.StringUtil;
 import org.apache.derby.impl.jdbc.EmbedDatabaseMetaData;
@@ -62,14 +70,16 @@ import org.apache.derby.impl.sql.catalog.XPLAINStatementTimingsDescriptor;
 import org.apache.derby.impl.sql.execute.JarUtil;
 import org.apache.derby.jdbc.InternalDriver;
 import org.apache.derby.iapi.store.access.TransactionController;
-import org.apache.derby.iapi.sql.dictionary.CatalogRowFactory;
-import org.apache.derby.iapi.sql.dictionary.SystemColumn;
+import org.apache.derby.iapi.sql.dictionary.AliasDescriptor;
 import org.apache.derby.iapi.sql.dictionary.DataDictionary;
 import org.apache.derby.iapi.sql.dictionary.DataDescriptorGenerator;
 import org.apache.derby.iapi.sql.dictionary.PasswordHasher;
 import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
+import org.apache.derby.iapi.sql.dictionary.StatementPermission;
+import org.apache.derby.iapi.sql.dictionary.StatementRoutinePermission;
 import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
 import org.apache.derby.iapi.sql.dictionary.UserDescriptor;
+import org.apache.derby.shared.common.error.MessageUtils;
 
 /**
 	Some system built-in procedures, and help routines.  Now used for network server.
@@ -89,14 +99,6 @@ public class SystemProcedures  {
 	private final static String DRIVER_TYPE_OPTION = "DATATYPE";
 	private final static String ODBC_DRIVER_OPTION = "'ODBC'";
 
-    // This token delimiter value is used to separate the tokens for multiple 
-    // error messages.  This is used in DRDAConnThread
-    /**
-     * <code>SQLERRMC_MESSAGE_DELIMITER</code> When message argument tokes are sent,
-     * this value separates the tokens for mulitiple error messages 
-     */
-    public static final String SQLERRMC_MESSAGE_DELIMITER = new String(new char[] {(char)20,(char)20,(char)20});
-
 	/**
 	  Method used by Derby Network Server to get localized message (original call
 	  from jcc.
@@ -132,16 +134,16 @@ public class SystemProcedures  {
 		// translated and append to make the final result.
 		for (int index=0; ; numMessages++)
 		{
-			if (sqlerrmc.indexOf(SQLERRMC_MESSAGE_DELIMITER, index) == -1)
+			if (sqlerrmc.indexOf(MessageUtils.SQLERRMC_MESSAGE_DELIMITER, index) == -1)
 				break;
-			index = sqlerrmc.indexOf(SQLERRMC_MESSAGE_DELIMITER, index) + 
-                        SQLERRMC_MESSAGE_DELIMITER.length();
+			index = sqlerrmc.indexOf(MessageUtils.SQLERRMC_MESSAGE_DELIMITER, index) + 
+                        MessageUtils.SQLERRMC_MESSAGE_DELIMITER.length();
 		}
 
 		// Putting it here instead of prepareCall it directly is because inter-jar reference tool
 		// cannot detect/resolve this otherwise
 		if (numMessages == 1)
-			MessageService.getLocalizedMessage(sqlcode, errmcLen, sqlerrmc, sqlerrp, errd0, errd1,
+			MessageUtils.getLocalizedMessage(sqlcode, errmcLen, sqlerrmc, sqlerrp, errd0, errd1,
 											errd2, errd3, errd4, errd5, warn, sqlState, file,
 											localeStr, msg, rc);
 		else
@@ -151,7 +153,7 @@ public class SystemProcedures  {
 			String[] errMsg = new String[2];
 			for (int i=0; i<numMessages; i++)
 			{
-				endIdx = sqlerrmc.indexOf(SQLERRMC_MESSAGE_DELIMITER, startIdx);
+				endIdx = sqlerrmc.indexOf(MessageUtils.SQLERRMC_MESSAGE_DELIMITER, startIdx);
 				if (i == numMessages-1)				// last error message
 					sqlError = sqlerrmc.substring(startIdx);
 				else sqlError = sqlerrmc.substring(startIdx, endIdx);
@@ -164,7 +166,7 @@ public class SystemProcedures  {
 					msg[0] += " SQLSTATE: " + sqlState + ": ";
 				}
 
-				MessageService.getLocalizedMessage(sqlcode, (short)sqlError.length(), sqlError,
+				MessageUtils.getLocalizedMessage(sqlcode, (short)sqlError.length(), sqlError,
 											sqlerrp, errd0, errd1, errd2, errd3, errd4, errd5,
 											warn, sqlState, file, localeStr, errMsg, rc);
 
@@ -174,7 +176,7 @@ public class SystemProcedures  {
 						msg[0] = errMsg[0];
 					else msg[0] += errMsg[0];	// append the new message
 				}
-				startIdx = endIdx + SQLERRMC_MESSAGE_DELIMITER.length();
+				startIdx = endIdx + MessageUtils.SQLERRMC_MESSAGE_DELIMITER.length();
 			}
 		}
 	}
@@ -679,16 +681,38 @@ public class SystemProcedures  {
      * @param key       The property key.
      * @param value     The new value, if null the property is deleted.
      *
-	 * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
     public static void SYSCS_SET_DATABASE_PROPERTY(
     String  key,
     String  value)
         throws SQLException
     {
-        PropertyInfo.setDatabaseProperty(key, value);
+        setDatabaseProperty( key, value, Securable.SET_DATABASE_PROPERTY );
     }
 
+    private static void setDatabaseProperty( String key, String value, Securable authorizationCheck )
+        throws SQLException
+	{
+		LanguageConnectionContext lcc = ConnectionUtil.getCurrentLCC();
+
+		try {
+            if ( authorizationCheck != null )
+            {
+                SecurityUtil.authorize( authorizationCheck );
+            }
+            
+            Authorizer a = lcc.getAuthorizer();
+            a.authorize((Activation) null, Authorizer.PROPERTY_WRITE_OP);
+
+            // Get the current transaction controller
+            TransactionController tc = lcc.getTransactionExecute();
+
+            tc.setProperty(key, value, false);
+		} catch (StandardException se) {throw PublicAPI.wrapStandardException( se ); }
+	}
+
+
     /**
      * Get the value of a property of the database in current connection.
      * <p>
@@ -696,15 +720,32 @@ public class SystemProcedures  {
      *
      * @param key       The property key.
      *
-	 * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
     public static String SYSCS_GET_DATABASE_PROPERTY(
     String  key)
         throws SQLException
+    {
+        return getProperty( key, Securable.GET_DATABASE_PROPERTY );
+    }
+
+    /**
+     * This method exists so that we can get a property value without performing
+     * authorization checks.
+     */
+    private static  String  getProperty( String key, Securable authorizationCheck )
+        throws SQLException
     {
         LanguageConnectionContext lcc = ConnectionUtil.getCurrentLCC();
 
         try {
+            // make sure that application code doesn't bypass security checks
+            // by calling this public entry point
+            if ( authorizationCheck != null )
+            {
+                SecurityUtil.authorize( authorizationCheck );
+            }
+            
             return PropertyUtil.getDatabaseProperty(lcc.getTransactionExecute(), key);
         } catch (StandardException se) {
             throw PublicAPI.wrapStandardException(se);
@@ -881,7 +922,7 @@ public class SystemProcedures  {
      * @param sequential    if non-zero then rebuild indexes sequentially,
      *                      if 0 then rebuild all indexes in parallel.
      *
-	 * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
     public static void SYSCS_COMPRESS_TABLE(
     String  schemaname,
@@ -909,7 +950,7 @@ public class SystemProcedures  {
      * Call internal routine to freeze the database so that a backup
      * can be made.
      *
-	 * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
     public static void SYSCS_FREEZE_DATABASE()
 		throws SQLException
@@ -924,7 +965,7 @@ public class SystemProcedures  {
      * by calling SYSCS_FREEZE_DATABASE().
      * can be made.
      *
-	 * @exception  SQLException  if a database error occurs
+     * @exception  SQLException if a database error occurs
      **/
     public static void SYSCS_UNFREEZE_DATABASE()
 		throws SQLException
@@ -1011,7 +1052,6 @@ public class SystemProcedures  {
     short     deleteOnlineArchivedLogFiles)
 		throws SQLException
     {
-
         Factory.getDatabaseOfConnection().backupAndEnableLogArchiveMode(
                 backupDir, 
                 (deleteOnlineArchivedLogFiles != 0),
@@ -1047,7 +1087,6 @@ public class SystemProcedures  {
     short     deleteOnlineArchivedLogFiles)
 		throws SQLException
     {
-
         Factory.getDatabaseOfConnection().backupAndEnableLogArchiveMode(
                 backupDir,
                 (deleteOnlineArchivedLogFiles != 0),
@@ -1062,9 +1101,8 @@ public class SystemProcedures  {
 	 * @param deleteOnlineArchivedLogFiles  If <tt>non-zero</tt> deletes all the
 	 *        online archived log files that exist before this call immediately.
      *
-	 * @exception SQLException if a database error occurs
+     * @exception SQLException if a database error occurs
 	 */
-
     public static void SYSCS_DISABLE_LOG_ARCHIVE_MODE(
     short     deleteOnlineArchivedLogFiles)
 		throws SQLException
@@ -1431,7 +1469,7 @@ public class SystemProcedures  {
 	 * IN TABLENAME    VARCHAR(128),  IN FILENAME VARCHAR(32672) , 
 	 * IN COLUMNDELIMITER CHAR(1),  IN CHARACTERDELIMITER CHAR(1) ,  
 	 * IN CODESET VARCHAR(128))
-	 * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
 	public static void SYSCS_EXPORT_TABLE(
 	String  schemaName,
@@ -1442,6 +1480,11 @@ public class SystemProcedures  {
 	String  codeset)
         throws SQLException
     {
+        //
+        // In case this routine is called directly by the application,
+        // authorization is performed by Export.exportTable().
+        //
+
 		Connection conn = getDefaultConn();
 		Export.exportTable(conn, schemaName , tableName , fileName ,
 							  columnDelimiter , characterDelimiter, codeset);
@@ -1460,7 +1503,7 @@ public class SystemProcedures  {
      * IN TABLENAME    VARCHAR(128),  IN FILENAME VARCHAR(32672) , 
      * IN COLUMNDELIMITER CHAR(1),  IN CHARACTERDELIMITER CHAR(1) ,  
      * IN CODESET VARCHAR(128), IN LOBSFILENAME VARCHAR(32672))
-     * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
     public static void SYSCS_EXPORT_TABLE_LOBS_TO_EXTFILE(
     String  schemaName,
@@ -1492,7 +1535,7 @@ public class SystemProcedures  {
 	 * IN COLUMNDELIMITER CHAR(1),  IN CHARACTERDELIMITER CHAR(1) ,  
 	 * IN CODESET VARCHAR(128))
 	 *
-	 * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
 	public static void SYSCS_EXPORT_QUERY(
     String  selectStatement,
@@ -1523,7 +1566,7 @@ public class SystemProcedures  {
      * IN COLUMNDELIMITER CHAR(1),  IN CHARACTERDELIMITER CHAR(1) ,  
      * IN CODESET VARCHAR(128), IN LOBSFILENAME VARCHAR(32672))
      *
-     * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
     public static void SYSCS_EXPORT_QUERY_LOBS_TO_EXTFILE(
     String  selectStatement,
@@ -1555,7 +1598,7 @@ public class SystemProcedures  {
 	 * IN TABLENAME    VARCHAR(128),  IN FILENAME VARCHAR(32672) , 
 	 * IN COLUMNDELIMITER CHAR(1),  IN CHARACTERDELIMITER CHAR(1) ,  
 	 * IN CODESET VARCHAR(128), IN  REPLACE SMALLINT)
-	 * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
 	public static void SYSCS_IMPORT_TABLE(
 	String  schemaName,
@@ -1580,6 +1623,46 @@ public class SystemProcedures  {
 		conn.commit();
 	}
 	
+
+
+
+
+
+/*import  data from a given file to a table skipping header lines.
+     * <p>
+     * Will be called by system procedure as
+	 * SYSCS_IMPORT_TABLE_BULK(IN SCHEMANAME  VARCHAR(128), 
+	 * IN TABLENAME    VARCHAR(128),  IN FILENAME VARCHAR(32672) , 
+	 * IN COLUMNDELIMITER CHAR(1),  IN CHARACTERDELIMITER CHAR(1) ,  
+	 * IN CODESET VARCHAR(128), IN  REPLACE SMALLINT
+	 * IN SKIP SMALLINT)
+     * @exception SQLException if a database error occurs
+     **/
+	public static void SYSCS_IMPORT_TABLE_BULK(
+	String  schemaName,
+    	String  tableName,
+	String  fileName,
+	String  columnDelimiter,
+	String  characterDelimiter,
+	String  codeset,
+	short   replace,
+	short skip)
+        throws SQLException
+    {
+		Connection conn = getDefaultConn();
+		try{
+			Import.importTable(conn, schemaName , tableName , fileName ,
+                               columnDelimiter , characterDelimiter, codeset, 
+                               replace, false, skip);
+		}catch(SQLException se)
+		{
+			rollBackAndThrowSQLException(conn, se);
+		}
+		//import finished successfull, commit it.
+		conn.commit();
+     }
+
+
     /**
      * issue a rollback when SQLException se occurs. If SQLException ouccurs when rollback,
      * the new SQLException will be added into the chain of se. 
@@ -1605,7 +1688,7 @@ public class SystemProcedures  {
      * IN TABLENAME    VARCHAR(128),  IN FILENAME VARCHAR(32672) , 
      * IN COLUMNDELIMITER CHAR(1),  IN CHARACTERDELIMITER CHAR(1) ,  
      * IN CODESET VARCHAR(128), IN  REPLACE SMALLINT)
-     * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
     public static void SYSCS_IMPORT_TABLE_LOBS_FROM_EXTFILE(
     String  schemaName,
@@ -1645,7 +1728,7 @@ public class SystemProcedures  {
 	 *                    IN CHARACTERDELIMITER CHAR(1), IN CODESET VARCHAR(128), 
 	 *                    IN REPLACE SMALLINT)
 	 *
-	 * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
 	public static void SYSCS_IMPORT_DATA(
     String  schemaName,
@@ -1675,6 +1758,56 @@ public class SystemProcedures  {
 	}
 
 
+
+/**
+      * Import data from a given file into the specified table columns skipping header lines from the 
+	 * specified columns in the file.
+     * <p>
+     * Will be called as 
+	 * SYSCS_IMPORT_DATA_BULK (IN SCHEMANAME VARCHAR(128), IN TABLENAME VARCHAR(128),
+	 *                    IN INSERTCOLUMNLIST VARCHAR(32672), IN COLUMNINDEXES VARCHAR(32672),
+	 *                    IN FILENAME VARCHAR(32672), IN COLUMNDELIMITER CHAR(1), 
+	 *                    IN CHARACTERDELIMITER CHAR(1), IN CODESET VARCHAR(128), 
+	 *                    IN REPLACE SMALLINT, IN SKIP SMALLINT)
+	 *
+     * @exception SQLException if a database error occurs
+     **/
+	public static void SYSCS_IMPORT_DATA_BULK(
+    String  schemaName,
+    String  tableName,
+	String  insertColumnList,
+	String  columnIndexes,
+	String  fileName,
+	String  columnDelimiter,
+	String  characterDelimiter,
+	String  codeset,
+	short   replace,
+	short skip)
+        throws SQLException
+    {
+		Connection conn = getDefaultConn();
+		try{
+			Import.importData(conn, schemaName , tableName ,
+								  insertColumnList, columnIndexes, fileName,
+								  columnDelimiter, characterDelimiter, 
+								  codeset, replace, false, skip);
+		}catch(SQLException se)
+		{
+		    rollBackAndThrowSQLException(conn, se);
+		}
+
+		//import finished successfull, commit it.
+		conn.commit();
+	}
+
+
+
+
+
+
+
+
+
     /**
      * Import data from a given file into the specified table columns 
      * from the  specified columns in the file. Data for large object 
@@ -1691,7 +1824,7 @@ public class SystemProcedures  {
      *               IN CHARACTERDELIMITER CHAR(1), IN CODESET VARCHAR(128), 
      *               IN REPLACE SMALLINT)
      *
-     * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
     public static void SYSCS_IMPORT_DATA_LOBS_FROM_EXTFILE(
     String  schemaName,
@@ -1728,7 +1861,7 @@ public class SystemProcedures  {
 	 * SYSCS_BULK_INSERT (IN SCHEMANAME VARCHAR(128), IN TABLENAME VARCHAR(128), 
 	 *                    IN VTINAME VARCHAR(32672), IN VTIARG VARCHAR(32672))
 	 *
-	 * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
 	public static void SYSCS_BULK_INSERT(
     String  schemaName,
@@ -1738,6 +1871,13 @@ public class SystemProcedures  {
 	)
         throws SQLException
     {
+        try {
+            // make sure that application code doesn't bypass security checks
+            // by calling this public entry point
+            SecurityUtil.authorize( Securable.BULK_INSERT );
+        }
+        catch (StandardException se) { throw PublicAPI.wrapStandardException( se ); }
+
 		Connection conn = getDefaultConn();
 
         // Use default schema if schemaName is null. This isn't consistent
@@ -1772,17 +1912,26 @@ public class SystemProcedures  {
     public static void SYSCS_RELOAD_SECURITY_POLICY()
         throws SQLException
     {
+        try {
+            // make sure that application code doesn't bypass security checks
+            // by calling this public entry point
+            SecurityUtil.authorize( Securable.RELOAD_SECURITY_POLICY );
+        }
+        catch (StandardException se) { throw PublicAPI.wrapStandardException( se ); }
+
         // If no security manager installed then there
         // is no policy to refresh. Calling Policy.getPolicy().refresh()
         // without a SecurityManager seems to lock in a policy with
         // no permissions thus ignoring the system property java.security.policy
         // when later installing a SecurityManager.
         if (System.getSecurityManager() == null)
+        {
             return;
+        }
         
         try {
             AccessController.doPrivileged(
-                    new PrivilegedAction() {
+                    new PrivilegedAction<Object>() {
                         public Object run() {
                             Policy.getPolicy().refresh();
                             return null;
@@ -1803,24 +1952,6 @@ public class SystemProcedures  {
 		return StrictMath.PI;
 	}
 	
-	/**
-	 * Constant for natural log(10).
-	 */
-	private static final double LOG10 = StrictMath.log(10.0d);
-	
-	/**
-	 * Base 10 log function. SYSFUN.LOG10
-	 * Calculated by
-	 * <code>
-	 * log(value) / log(10)
-	 * </code>
-	 * where log is the natural log.
-	 */
-	public static double LOG10(double value)
-	{
-		return StrictMath.log(value) / LOG10;
-	}
-
 	/**
 	 * Cotangent function. SYSFUN.COT
 	 * @see <a href="http://mathworld.wolfram.com/HyperbolicFunctions.html">HyperbolicFunctions</a>
@@ -1831,37 +1962,6 @@ public class SystemProcedures  {
 		return 1.0 / StrictMath.tan(value);
 	}
 
-	/**
-	 * Hyperbolic Cosine function. SYSFUN.COSH
-	 * @see <a href="http://mathworld.wolfram.com/HyperbolicFunctions.html">HyperbolicFunctions</a>
-	 * @return 1/2 (e^x + e^-x)
-	 */
-	public static double COSH(double value)
-	{
-		return (StrictMath.exp(value) + StrictMath.exp(-value)) / 2.0;
-	}
-
-	/**
-	 * Hyperbolic Sine function. SYSFUN.SINH
-	 * @see <a href="http://mathworld.wolfram.com/HyperbolicFunctions.html">HyperbolicFunctions</a>
-	 * @return 1/2 (e^x - e^-x)
-	 */
-	public static double SINH(double value)
-	{
-		return (StrictMath.exp(value) - StrictMath.exp(-value)) / 2.0;
-	}
-
-	/**
-	 * Hyperbolic Tangent function. SYSFUN.TANH
-	 * @see <a href="http://mathworld.wolfram.com/HyperbolicFunctions.html">HyperbolicFunctions</a>
-	 * @return (e^x - e^-x) / (e^x + e^-x)
-	 */
-	public static double TANH(double value)
-	{
-		return (StrictMath.exp(value) - StrictMath.exp(-value)) /
-			(StrictMath.exp(value) + StrictMath.exp(-value));
-	}
-
 	/**
 	 * Method to return the sign of the given value.
 	 * SYSFUN.SIGN().
@@ -1914,8 +2014,10 @@ public class SystemProcedures  {
                 addListProperty = null;
             }
             else
+            {
                 throw StandardException.newException(SQLState.UU_UNKNOWN_PERMISSION,
                         connectionPermission);
+            }
 
             // Always remove from both lists to avoid any repeated
             // user on list errors.
@@ -1926,9 +2028,13 @@ public class SystemProcedures  {
             
             
             if (addListProperty != null) {
-                String addList = SYSCS_GET_DATABASE_PROPERTY(addListProperty);
-                SYSCS_SET_DATABASE_PROPERTY(addListProperty,
-                    IdUtil.appendNormalToList(userName, addList));
+                String addList = getProperty( addListProperty, Securable.SET_USER_ACCESS );
+                setDatabaseProperty
+                    (
+                     addListProperty,
+                     IdUtil.appendNormalToList(userName, addList),
+                     null
+                     );
             }
             
         } catch (StandardException se) {
@@ -1944,11 +2050,10 @@ public class SystemProcedures  {
             String listProperty, String userName)
         throws SQLException, StandardException
     {
-        String removeList = SYSCS_GET_DATABASE_PROPERTY(listProperty);
+        String removeList = getProperty( listProperty, Securable.SET_USER_ACCESS );
         if (removeList != null)
         {
-            SYSCS_SET_DATABASE_PROPERTY(listProperty,
-                    IdUtil.deleteId(userName, removeList));
+            setDatabaseProperty( listProperty, IdUtil.deleteId( userName, removeList ), null );
         }
     }
     
@@ -1965,25 +2070,35 @@ public class SystemProcedures  {
         try {
             
             if (userName == null)
+            {
                 throw StandardException.newException(SQLState.AUTH_INVALID_USER_NAME,
                         userName);
+            }
            
             String fullUserList =
-                SYSCS_GET_DATABASE_PROPERTY(Property.FULL_ACCESS_USERS_PROPERTY);
+                getProperty( Property.FULL_ACCESS_USERS_PROPERTY, Securable.GET_USER_ACCESS );
             if (IdUtil.idOnList(userName, fullUserList))
+            {
                 return Property.FULL_ACCESS;
+            }
             
             String readOnlyUserList =
-                SYSCS_GET_DATABASE_PROPERTY(Property.READ_ONLY_ACCESS_USERS_PROPERTY);
+                getProperty( Property.READ_ONLY_ACCESS_USERS_PROPERTY, Securable.GET_USER_ACCESS );
             if (IdUtil.idOnList(userName, readOnlyUserList))
+            {
                 return Property.READ_ONLY_ACCESS;
+            }
             
             String defaultAccess = 
-                SYSCS_GET_DATABASE_PROPERTY(Property.DEFAULT_CONNECTION_MODE_PROPERTY);
+                getProperty( Property.DEFAULT_CONNECTION_MODE_PROPERTY, Securable.GET_USER_ACCESS );
             if (defaultAccess != null)
+            {
                 defaultAccess = StringUtil.SQLToUpperCase(defaultAccess);
+            }
             else
+            {
                 defaultAccess = Property.FULL_ACCESS; // is the default.
+            }
             
             return defaultAccess;
             
@@ -2003,6 +2118,10 @@ public class SystemProcedures  {
         
         DataDictionary dd = lcc.getDataDictionary();
         try {
+            // make sure that application code doesn't bypass security checks
+            // by calling this public entry point
+            SecurityUtil.authorize( Securable.INVALIDATE_STORED_STATEMENTS );
+
         	dd.invalidateAllSPSPlans(lcc);
         } catch (StandardException se) {
             throw PublicAPI.wrapStandardException(se);
@@ -2018,25 +2137,42 @@ public class SystemProcedures  {
     public static void SYSCS_EMPTY_STATEMENT_CACHE()
        throws SQLException
     {
-       LanguageConnectionContext lcc = ConnectionUtil.getCurrentLCC();
+        try {
+            // make sure that application code doesn't bypass security checks
+            // by calling this public entry point
+            SecurityUtil.authorize( Securable.EMPTY_STATEMENT_CACHE );
+        }
+        catch (StandardException se) { throw PublicAPI.wrapStandardException( se ); }
+
+        LanguageConnectionContext lcc = ConnectionUtil.getCurrentLCC();
        
-       CacheManager statementCache =
+        CacheManager statementCache =
            lcc.getLanguageConnectionFactory().getStatementCache();
        
-       if (statementCache != null)
-           statementCache.ageOut();
+        if (statementCache != null)
+        {
+            statementCache.ageOut();
+        }
     }
   
 	 /**
      * this procedure switches between the different xplain modes 
-     * @param mode either 0 for explain only, or 1 for explain & execute (default)
-     * @throws SQLException, StandardException
+     * @param mode either 0 for explain only, or 1 for explain and execute (default)
+     * @throws SQLException
      */
     public static void SYSCS_SET_XPLAIN_MODE(int mode)
-                throws SQLException, StandardException
-            {
+        throws SQLException, StandardException
+    {
+        try {
+            // make sure that application code doesn't bypass security checks
+            // by calling this public entry point
+            SecurityUtil.authorize( Securable.SET_XPLAIN_MODE );
+        }
+        catch (StandardException se) { throw PublicAPI.wrapStandardException( se ); }
+
 		ConnectionUtil.getCurrentLCC().setXplainOnlyMode(mode != 0 ? true : false);
-            }
+    }
+    
     /**
      * This procedure returns the current status of the xplain mode.
      *
@@ -2044,13 +2180,20 @@ public class SystemProcedures  {
      * are being XPLAIN'd only, not executed.
      *
      * @return 0 if XPLAIN mode is off, non-zero if on.
-     * @throws SQLException, StandardException
+     * @throws SQLException
      */
     public static int SYSCS_GET_XPLAIN_MODE()
-                throws SQLException, StandardException
-           {
-                return ConnectionUtil.getCurrentLCC().getXplainOnlyMode()?1:0;
-           }
+        throws SQLException, StandardException
+    {
+        try {
+            // make sure that application code doesn't bypass security checks
+            // by calling this public entry point
+            SecurityUtil.authorize( Securable.GET_XPLAIN_MODE );
+        }
+        catch (StandardException se) { throw PublicAPI.wrapStandardException( se ); }
+
+        return ConnectionUtil.getCurrentLCC().getXplainOnlyMode()?1:0;
+    }
     
     /**
      * This procedure sets the current xplain schema.
@@ -2058,11 +2201,18 @@ public class SystemProcedures  {
      * textual stream printout. If it is set, statisitcs information is
      * stored in that schema in user tables.
      * @param schemaName May be an empty string.
-     * @throws SQLException, StandardException
+     * @throws SQLException
      */
     public static void SYSCS_SET_XPLAIN_SCHEMA(String schemaName)
                 throws SQLException, StandardException
     {
+        try {
+            // make sure that application code doesn't bypass security checks
+            // by calling this public entry point
+            SecurityUtil.authorize( Securable.SET_XPLAIN_SCHEMA );
+        }
+        catch (StandardException se) { throw PublicAPI.wrapStandardException( se ); }
+
         LanguageConnectionContext lcc       = ConnectionUtil.getCurrentLCC();
         TransactionController     tc        = lcc.getTransactionExecute();
 
@@ -2152,11 +2302,18 @@ public class SystemProcedures  {
     /**
      * This procedure returns the current set XPLAIN_SCHEMA
      * @return schema name, may be blank if no schema currently set.
-     * @throws SQLException, StandardException
+     * @throws SQLException
      */
     public static String SYSCS_GET_XPLAIN_SCHEMA()
                 throws SQLException, StandardException
     {
+        try {
+            // make sure that application code doesn't bypass security checks
+            // by calling this public entry point
+            SecurityUtil.authorize( Securable.GET_XPLAIN_SCHEMA );
+        }
+        catch (StandardException se) { throw PublicAPI.wrapStandardException( se ); }
+
         String sd = ConnectionUtil.getCurrentLCC().getXplainSchema();
         if (sd == null)
             return "";
@@ -2181,6 +2338,10 @@ public class SystemProcedures  {
         // the first credentials must be those of the DBO and only the DBO
         // can add them
         try {
+            // make sure that application code doesn't bypass security checks
+            // by calling this public entry point
+            SecurityUtil.authorize( Securable.CREATE_USER );
+
             DataDictionary dd = lcc.getDataDictionary();
             String  dbo = dd.getAuthorizationDatabaseOwner();
 
@@ -2216,6 +2377,11 @@ public class SystemProcedures  {
          )
         throws SQLException
     {
+        //
+        // Application code can call this method, and that is a security hole until
+        // we prevent the application from getting its hands on a transaction controller.
+        // DERBY-6648 may help prevent that.
+        //
         try {
             LanguageConnectionContext lcc = ConnectionUtil.getCurrentLCC();
             DataDictionary dd = lcc.getDataDictionary();
@@ -2282,6 +2448,13 @@ public class SystemProcedures  {
          )
         throws SQLException
     {
+        try {
+            // make sure that application code doesn't bypass security checks
+            // by calling this public entry point
+            SecurityUtil.authorize( Securable.RESET_PASSWORD );
+        }
+        catch (StandardException se) { throw PublicAPI.wrapStandardException( se ); }
+
         resetAuthorizationIDPassword( normalizeUserName( userName ), password );
     }
 
@@ -2346,6 +2519,10 @@ public class SystemProcedures  {
         userName = normalizeUserName( userName );
             
         try {
+            // make sure that application code doesn't bypass security checks
+            // by calling this public entry point
+            SecurityUtil.authorize( Securable.DROP_USER );
+            
             LanguageConnectionContext lcc = ConnectionUtil.getCurrentLCC();
             DataDictionary dd = lcc.getDataDictionary();
             String  dbo = dd.getAuthorizationDatabaseOwner();
@@ -2397,14 +2574,29 @@ public class SystemProcedures  {
             return IdUtil.getUserAuthorizationId( userName );
         } catch (StandardException se) { throw PublicAPI.wrapStandardException(se); }
     }
-  
+
+    /**
+     * Return the database name
+     * @return database name
+     * @throws SQLException
+     */
+    public static String SYSCS_GET_DATABASE_NAME()
+        throws SQLException
+    {
+        //DERBY-6725(Add a system function which returns the name of the database.)
+        LanguageConnectionContext lcc = ConnectionUtil.getCurrentLCC();
+        try {
+            return( getMonitor().getCanonicalServiceName( lcc.getDbname() ) );
+        } catch (StandardException se) { throw PublicAPI.wrapStandardException(se); }
+    }
+
     /**
      * Peek at the current value of a sequence generator without advancing it.
      *
      * @param schemaName    The name of the schema holding the sequence.
      * @param sequenceName    The name of the sequence in that schema.
      *
-	 * @exception  SQLException  if a database error occurs
+     * @exception SQLException if a database error occurs
      **/
     public static Long SYSCS_PEEK_AT_SEQUENCE( String schemaName, String sequenceName )
         throws SQLException
@@ -2414,4 +2606,37 @@ public class SystemProcedures  {
         } catch (StandardException se) { throw PublicAPI.wrapStandardException(se); }
     }
 
+    /**
+     * Peek at the current value of an identity generator without advancing it.
+     *
+     * @param schemaName    The name of the schema holding the table.
+     * @param tableName    The name of the table in that schema.
+     *
+     * @exception SQLException if a database error occurs
+     **/
+    public static Long SYSCS_PEEK_AT_IDENTITY( String schemaName, String tableName )
+        throws SQLException
+    {
+        try {
+            return ConnectionUtil.getCurrentLCC().getDataDictionary().peekAtIdentity( schemaName, tableName );
+        } catch (StandardException se) { throw PublicAPI.wrapStandardException(se); }
+    }
+
+    /**
+     * Helper routine which looks up the monitor.
+     */
+    private static  ModuleFactory  getMonitor()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ModuleFactory>()
+             {
+                 public ModuleFactory run()
+                 {
+                     return Monitor.getMonitor();
+                 }
+             }
+             );
+    }
+
 }
