@@ -52,7 +52,7 @@ public interface Property {
 
 	/**
 		By convention properties that must not be stored any persistent form of
-		service properties start with this prefix.
+		service properties start with this prefix. These properties are not documented.
 	*/
 	public static final String PROPERTY_RUNTIME_PREFIX = "derby.__rt.";
 
@@ -95,6 +95,16 @@ public interface Property {
      */
 	
 	String LOG_BOOT_TRACE = "derby.stream.error.logBootTrace";
+    
+        /**
+		derby.stream.error.style=<b>The error stream error style.</b>
+        <b>rollingFile<b> is the only file currently supported.
+		Takes precendence over derby.stream.error.file.
+		Takes precendence over derby.stream.error.method.
+		Takes precendence over derby.stream.error.field
+	*/	
+	String ERRORLOG_STYLE_PROPERTY = "derby.stream.error.style";
+
         /**
 		derby.stream.error.file=<b>absolute or relative error log filename</b>
 		Takes precendence over derby.stream.error.method.
@@ -113,11 +123,71 @@ public interface Property {
 
         /**
 		derby.stream.error.field=
-			<className>.<fieldName> returning an OutputStream or Writer object>
+			className.fieldName returning an OutputStream or Writer object
 	*/
 	
 	String ERRORLOG_FIELD_PROPERTY = "derby.stream.error.field";
 
+        /**
+		derby.stream.error.rollingfile.pattern=<b>the pattern</b>
+        A pattern consists of a string that includes the following special
+        components that will be replaced at runtime:
+        <UL>
+        <LI>    "/"    the local pathname separator 
+        <LI>     "%t"   the system temporary directory
+        <LI>     "%h"   the value of the "user.home" system property
+        <LI>     "%d"   the value of the "derby.system.home" system property
+        <LI>     "%g"   the generation number to distinguish rotated logs
+        <LI>     "%u"   a unique number to resolve conflicts
+        <LI>     "%%"   translates to a single percent sign "%"
+        </UL>
+        If no "%g" field has been specified and the file count is greater
+        than one, then the generation number will be added to the end of
+        the generated filename, after a dot.
+        <P> 
+        Thus for example a pattern of "%t/java%g.log" with a count of 2
+        would typically cause files to be written on Solaris to 
+        /var/tmp/java0.log and /var/tmp/java1.log whereas on Windows 95 they
+        would be typically written to C:\TEMP\java0.log and C:\TEMP\java1.log
+        <P> 
+        Generation numbers follow the sequence 0, 1, 2, etc.
+        <P>
+        Normally the "%u" unique field is set to 0.  However, if the <tt>FileHandler</tt>
+        tries to open the filename and finds the file is currently in use by
+        another process it will increment the unique number field and try
+        again.  This will be repeated until <tt>FileHandler</tt> finds a file name that
+        is  not currently in use. If there is a conflict and no "%u" field has
+        been specified, it will be added at the end of the filename after a dot.
+        (This will be after any automatically added generation number.)
+        <P>
+        Thus if three processes were all trying to output to fred%u.%g.txt then 
+        they  might end up using fred0.0.txt, fred1.0.txt, fred2.0.txt as
+        the first file in their rotating sequences.
+        <P>
+        Note that the use of unique ids to avoid conflicts is only guaranteed
+        to work reliably when using a local disk file system.
+        <P>
+        The default pattern is "%d/derby-%g.log"
+	*/	
+	String ERRORLOG_ROLLINGFILE_PATTERN_PROPERTY = "derby.stream.error.rollingFile.pattern";
+
+        /**
+		derby.stream.error.rollingfile.limit=<b>the rolling file size limit</b>
+        <P>
+        The default limit is 1024000
+	*/
+	
+	String ERRORLOG_ROLLINGFILE_LIMIT_PROPERTY = "derby.stream.error.rollingFile.limit";
+
+        /**
+		derby.stream.error.rollingfile.count=<b>the rolling file count</b>
+        <P>
+        The default count is 10
+	*/
+	
+	String ERRORLOG_ROLLINGFILE_COUNT_PROPERTY = "derby.stream.error.rollingFile.count";
+
+    
 	/** 
 	derby.infolog.append={true,false}
 	<BR>
@@ -326,6 +396,8 @@ public interface Property {
 
 	/**
 		internal use only, passes the database classpathinto the class manager
+
+        Undocumented.
 	*/
 	String BOOT_DB_CLASSPATH = PROPERTY_RUNTIME_PREFIX + "database.classpath";
 
@@ -368,9 +440,15 @@ public interface Property {
 	*/
 
     /**
+     * <p>
      * Creation of an access factory should be done with no logging.
 	 * This is a run-time property that should not make it to disk
 	 * in the service.properties file.
+     * </p>
+     *
+     * <p>
+     * Undocumented.
+     * </p>
      **/
 	public static final String CREATE_WITH_NO_LOG =
 		PROPERTY_RUNTIME_PREFIX + "storage.createWithNoLog";
@@ -498,7 +576,11 @@ public interface Property {
      * When set, the store system will use sync() call on the log at 
      * commit instead of doing  a write sync on all writes to  the log;
 	 * even if the write sync mode (rws) is supported in the JVM. 
-     * <p>
+     * </p>
+     *
+	 * <p>
+	 * Undocumented.
+     * </p>
      *
      **/
 	public static final String FILESYNC_TRANSACTION_LOG = 
@@ -618,10 +700,14 @@ public interface Property {
 	String	LANG_PERMISSIONS_CACHE_SIZE = "derby.language.permissionsCacheSize";
 	int		LANG_PERMISSIONS_CACHE_SIZE_DEFAULT = 64;
 	/**
+     * <p>
 	 * The size of the stored prepared statment descriptor cache 
 	 * used by the data dictionary.  Database.  Static.
-	 * <p>
-	 * Externally visible.
+	 * </p>
+     *
+     * <p>
+	 * Externally visible but undocumented.
+	 * </p>
 	 */
 	String	LANG_SPS_CACHE_SIZE = "derby.language.spsCacheSize";
 	int		LANG_SPS_CACHE_SIZE_DEFAULT =32;
@@ -630,10 +716,10 @@ public interface Property {
 	 * The size of the sequence generator cache 
 	 * used by the data dictionary.  Database.  Static.
 	 * <p>
-	 * Externally visible.
+	 * Externally visible and documented.
 	 */
 	String	LANG_SEQGEN_CACHE_SIZE = "derby.language.sequenceGeneratorCacheSize";
-	int		LANG_SEQGEN_CACHE_SIZE_DEFAULT =32;
+	int		LANG_SEQGEN_CACHE_SIZE_DEFAULT = 1000;
 
 	/**
 	 * Name of the implementation of SequencePreallocator which is used
@@ -652,7 +738,7 @@ public interface Property {
 	  be executed before checking whether its plan is stale.  Database.
 	  Dynamic.
 	  <P>
-	  Externally visible.
+	  Externally visible but undocumented.
 	 */
 	String LANGUAGE_STALE_PLAN_CHECK_INTERVAL =
 								"derby.language.stalePlanCheckInterval";
@@ -690,47 +776,91 @@ public interface Property {
     String STORAGE_AUTO_INDEX_STATS_TRACING = "derby.storage.indexStats.trace";
 
     /**
+     * <p>
      * Specifies the lower threshold for the number of rows in a table before
      * creating statistics for the associated indexes.
+     * </p>
+     *
      * <p>
      * <em>NOTE:</em> This is a debug property which will be removed or renamed.
+     * </p>
+     *
+     * <p>
+     * Undocumented.
+     * </p>
      */
     String STORAGE_AUTO_INDEX_STATS_DEBUG_CREATE_THRESHOLD =
             "derby.storage.indexStats.debug.createThreshold";
     int STORAGE_AUTO_INDEX_STATS_DEBUG_CREATE_THRESHOLD_DEFAULT = 100;
 
     /**
+     * <p>
      * Specifies the lower threshold for the absolute difference between the
      * row estimate for the table and the row estimate for the index before
      * creating statistics for the associated indexes.
+     * </p>
+     *
      * <p>
      * <em>NOTE:</em> This is a debug property which will be removed or renamed.
+     * </p>
+     *
+     * <p>
+     * Undocumented.
+     * </p>
      */
     String STORAGE_AUTO_INDEX_STATS_DEBUG_ABSDIFF_THRESHOLD =
             "derby.storage.indexStats.debug.absdiffThreshold";
     int STORAGE_AUTO_INDEX_STATS_DEBUG_ABSDIFF_THRESHOLD_DEFAULT = 1000;
 
     /**
+     * <p>
      * Specifies the lower threshold for the logarithmical (natural logarithm e)
      * difference between the row estimate for the table and the row estimate
      * for the index before creating statistics for the associated indexes.
+     * </p>
+     *
      * <p>
      * <em>NOTE:</em> This is a debug property which will be removed or renamed.
+     * </p>
+     *
+     * <p>
+     * Undocumented.
+     * </p>
      */
     String STORAGE_AUTO_INDEX_STATS_DEBUG_LNDIFF_THRESHOLD =
             "derby.storage.indexStats.debug.lndiffThreshold";
     double STORAGE_AUTO_INDEX_STATS_DEBUG_LNDIFF_THRESHOLD_DEFAULT = 1.0;
 
     /**
+     * <p>
      * Specifies the size of the work unit queue in the index statistics update
      * daemon.
+     * </p>
+     *
+     * <p>
+     * <em>NOTE:</em> This is a debug property which will be removed or renamed.
+     * </p>
+     *
+     * <p>
+     * Undocumented.
+     * </p>
      */
     String STORAGE_AUTO_INDEX_STATS_DEBUG_QUEUE_SIZE =
             "derby.storage.indexStats.debug.queueSize";
     int STORAGE_AUTO_INDEX_STATS_DEBUG_QUEUE_SIZE_DEFAULT = 20;
 
     /**
+     * <p>
      * Specifies whether to revert to 10.8 behavior and keep disposable stats.
+     * </p>
+     *
+     * <p>
+     * <em>NOTE:</em> This is a debug property which will be removed or renamed.
+     * </p>
+     *
+     * <p>
+     * Undocumented.
+     * </p>
      */
     String STORAGE_AUTO_INDEX_STATS_DEBUG_KEEP_DISPOSABLE_STATS =
             "derby.storage.indexStats.debug.keepDisposableStats";
@@ -799,6 +929,8 @@ public interface Property {
 								"derby.authentication.provider";
 
 	// This is the user property used by Derby and LDAP schemes
+    //
+    // Should not be documented any longer.
 	public static final String USER_PROPERTY_PREFIX = "derby.user.";
 
 	// These are the different built-in providers Derby supports
@@ -840,8 +972,10 @@ public interface Property {
     
 
     /**
+     * <p>
      * Property that specifies the name of the hash algorithm to use with
      * the configurable hash authentication scheme.
+     * </p>
      */
     public static final String AUTHENTICATION_BUILTIN_ALGORITHM =
             "derby.authentication.builtin.algorithm";
@@ -862,9 +996,11 @@ public interface Property {
             "SHA-1";
 
     /**
+     * <p>
      * Property that specifies the number of bytes with random salt to use
      * when hashing credentials using the configurable hash authentication
      * scheme.
+     * </p>
      */
     public static final String AUTHENTICATION_BUILTIN_SALT_LENGTH =
             "derby.authentication.builtin.saltLength";
@@ -875,8 +1011,10 @@ public interface Property {
     public static final int AUTHENTICATION_BUILTIN_SALT_LENGTH_DEFAULT = 16;
 
     /**
+     * <p>
      * Property that specifies the number of times to apply the hash
      * function in the configurable hash authentication scheme.
+     * </p>
      */
     public static final String AUTHENTICATION_BUILTIN_ITERATIONS =
             "derby.authentication.builtin.iterations";
@@ -892,21 +1030,22 @@ public interface Property {
 
 	/**
 		Property name for specifying log switch interval
+
+        Undocumented.
 	 */
 	public static final String LOG_SWITCH_INTERVAL = "derby.storage.logSwitchInterval";
 
 	/**
 		Property name for specifying checkpoint interval
+
+        Undocumented.
 	 */
 	public static final String CHECKPOINT_INTERVAL = "derby.storage.checkpointInterval";
 
-	/**
-		Property name for specifying log archival location
-	 */
-	public static final String LOG_ARCHIVAL_DIRECTORY = "derby.storage.logArchive";
-
 	/**
 		Property name for specifying log Buffer Size
+
+        Undocumented.
 	 */
 	public static final String LOG_BUFFER_SIZE = "derby.storage.logBufferSize";
 	
@@ -933,9 +1072,15 @@ public interface Property {
 	*/
 	
 	/**
+     * <p>
 	 * Allow database upgrade during alpha/beta time. Only intended
 	 * to be used to allow Derby developers to test their upgrade code.
 	 * Only supported as a system/application (derby.properties) property.
+     * </p>
+     *
+     * <p>
+     * Undocumented.
+     * </p>
 	 */
 	String ALPHA_BETA_ALLOW_UPGRADE = "derby.database.allowPreReleaseUpgrade";
 	    
@@ -967,31 +1112,57 @@ public interface Property {
 	public static final String HTTP_DB_FILE_NAME =   "db2j.http.file.name";
 
     /**
+     * <p>
      * derby.drda.startNetworkServer
-     *<BR>
+     * </p>
+     *
+     *<p>
      * If true then we will attempt to start a DRDA network server when Derby 
      * boots, turning the current JVM into a server.
-     *<BR>
+     *</p>
+     *
+     * <p>
      * Default: false
+     * </p>
      */
     public static final String START_DRDA = "derby.drda.startNetworkServer";
 
     /**
+     * <p>
      * derby.drda.logConnections
-     *<BR>
+     *</p>
+     *
+     * <p>
      * Indicates whether to log connections and disconnections.
-     *<BR>
+     *</p>
+     *
+     * <p>
      * Default: false
+     * </p>
      */
 	public final static String DRDA_PROP_LOGCONNECTIONS = "derby.drda.logConnections";
+
     /**
+     * <p>
      * derby.drda.traceAll
-     *<BR>
+     *</p>
+     *
+     * <p>
      * Turns tracing on for all sessions.
-     *<BR>
+     *</p>
      * Default: false
+     *
+     * <p>
+     * Default: false
+     * </p>
      */
 	public final static String DRDA_PROP_TRACEALL = "derby.drda.traceAll";
+
+    /**
+     * <p>
+     * derby.drda.trace
+     *</p>
+     */
 	public final static String DRDA_PROP_TRACE = "derby.drda.trace";
 
     /**
@@ -1075,6 +1246,7 @@ public interface Property {
 	public static final String SERVICE_PROTOCOL = "derby.serviceProtocol";
 	public static final String SERVICE_LOCALE = "derby.serviceLocale";
 
+    // Undocumented
 	public static final String COLLATION = "derby.database.collation";
 	// These are the six possible values for collation type if the collation
 	// derivation is not NONE. If collation derivation is NONE, then collation
@@ -1146,16 +1318,20 @@ public interface Property {
 
 
     /**
+     * <p>
      * derby.storage.useDefaultFilePermissions = {false,true}
-     * <p/>
+     * </p>
+     * <p>
      * When set to true, the store system will not limit file permissions of
      * files created by Derby to owner, but rely on the current OS default.  On
-     * Unix, this is determined by {@code umask(1)}. Only relevant for JVM >=
+     * Unix, this is determined by {@code umask(1)}. Only relevant for JVM &gt;=
      * 6.
-     * <p/>
+     * </p>
+     * <p>
      * The default value is {@code true} on embedded, but {@code false} on the
      * Network server if started from command line, otherwise it is true for
      * the server, too (i.e. started via API).
+     * </p>
      */
     String STORAGE_USE_DEFAULT_FILE_PERMISSIONS =
         "derby.storage.useDefaultFilePermissions";
