@@ -24,12 +24,12 @@ package org.apache.derby.iapi.services.context;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.HashSet;
-import java.util.Iterator;
+import java.util.Stack;
 
-import org.apache.derby.iapi.error.ShutdownException;
-import org.apache.derby.iapi.services.info.JVMInfo;
+import org.apache.derby.shared.common.error.ShutdownException;
+import org.apache.derby.iapi.security.SecurityUtil;
 import org.apache.derby.iapi.services.monitor.Monitor;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.services.stream.HeaderPrintWriter;
 
 /**
@@ -63,14 +63,14 @@ public final class ContextService //OLD extends Hashtable
             and this is enforced by synchronization outside the ContextManager.
             E.g for JDBC connections, synchronization at the JDBC level.
 
-		<LI> java.util.Stack containing ContextManagers - the current
+        <LI> ContextManagerStack containing ContextManagers - the current
         thread is actively using multiple different ContextManagers,
         with nesting. All ContextManagers in the stack will have
         activeThread set to the current thread, and their activeCount
-        set to -1. This is beacause nesting is soley represented by
+        set to -1. This is because nesting is solely represented by
         the stack, with the current context manager on top of the stack.
         This supports multiple levels of nesting across two stacks, e.g.
-        C1->C2->C2->C1->C2.
+        C1-&gt;C2-&gt;C2-&gt;C1-&gt;C2.
 		</UL>
 
 		This thread local is used to find the current context manager. Basically it provides
@@ -95,7 +95,7 @@ public final class ContextService //OLD extends Hashtable
 		for the lifetime of the request. In this case this variable will contain a  WeakReference.
 		</UL>
         <BR>
-        Single thread for Connection exection.
+        Single thread for Connection execution.
         <pre>
         threadContextList.get() == cm
         // while in JDBC engine code
@@ -158,7 +158,7 @@ public final class ContextService //OLD extends Hashtable
         cm3.activeCount = -1; // nesting in stack
         </pre>   
 	*/
-	private ThreadLocal threadContextList = new ThreadLocal();
+	private ThreadLocal<Object> threadContextList = new ThreadLocal<Object>();
 
     /**
      * Collection of all ContextManagers that are open
@@ -169,7 +169,7 @@ public final class ContextService //OLD extends Hashtable
      * @see #newContextManager()
      * @see SystemContext#cleanupOnError(Throwable)
      */
-	private HashSet allContexts;
+	private HashSet<ContextManager> allContexts;
 
     /**
      * Create a new ContextService for a Derby system.
@@ -183,14 +183,18 @@ public final class ContextService //OLD extends Hashtable
 
 		ContextService.factory = this;
 
-		allContexts = new HashSet();
+		allContexts = new HashSet<ContextManager>();
 
 	}
 
 	/**
 		So it can be given to us and taken away...
 	 */
-	public static void stop() {
+	public static void stop()
+    {
+        // Verify that we have permission to execute this method.
+        SecurityUtil.checkDerbyInternalsPrivilege();
+        
 		// For some unknown reason, the ContextManager and
 		// ContextService objects will not be garbage collected
 		// without the next two lines.
@@ -204,7 +208,11 @@ public final class ContextService //OLD extends Hashtable
         }
 	}
 
-	public static ContextService getFactory() {
+	public static ContextService getFactory()
+    {
+        // Verify that we have permission to execute this method.
+        SecurityUtil.checkDerbyInternalsPrivilege();
+        
 		ContextService csf = factory;
 
 		if (csf == null)
@@ -217,8 +225,11 @@ public final class ContextService //OLD extends Hashtable
 
 		@return The requested context, null if it doesn't exist.
 	*/
-	public static Context getContext(String contextId) {
-
+	public static Context getContext(String contextId)
+    {
+        // Verify that we have permission to execute this method.
+        SecurityUtil.checkDerbyInternalsPrivilege();
+        
 		ContextManager cm = getFactory().getCurrentContextManager();
 
         if( cm == null)
@@ -236,7 +247,11 @@ public final class ContextService //OLD extends Hashtable
 
 		@return The requested context, null if it doesn't exist.
 	*/
-	public static Context getContextOrNull(String contextId) {
+	public static Context getContextOrNull(String contextId)
+    {
+        // Verify that we have permission to execute this method.
+        SecurityUtil.checkDerbyInternalsPrivilege();
+        
 		ContextService csf = factory;
 
 		if (csf == null)
@@ -263,7 +278,7 @@ public final class ContextService //OLD extends Hashtable
 	 */
 	public ContextManager getCurrentContextManager() {
 
-		ThreadLocal tcl = threadContextList;
+		ThreadLocal<Object> tcl = threadContextList;
 		if (tcl == null) {
 			// The context service is already stopped.
 			return null;
@@ -284,9 +299,7 @@ public final class ContextService //OLD extends Hashtable
 		if (list == null)
 			return null;
 
-		java.util.Stack stack = (java.util.Stack) list;
-		return (ContextManager) (stack.peek());
-
+        return ((ContextManagerStack) list).peek();
 	}
 
     /**
@@ -295,7 +308,7 @@ public final class ContextService //OLD extends Hashtable
      * see that method for details.
      */
 	public void resetCurrentContextManager(ContextManager cm) {
-		ThreadLocal tcl = threadContextList;
+		ThreadLocal<Object> tcl = threadContextList;
 
 		if (tcl == null) {
 			// The context service is already stopped.
@@ -346,17 +359,17 @@ public final class ContextService //OLD extends Hashtable
 			return;
 		}
 
-		java.util.Stack stack = (java.util.Stack) tcl.get();
+        ContextManagerStack stack = (ContextManagerStack) tcl.get();
 
-		Object oldCM = stack.pop();
+        // Remove the context manager at the top of the stack.
+        stack.pop();
 
-		ContextManager nextCM = (ContextManager) stack.peek();
+        ContextManager nextCM = stack.peek();
 
 		boolean seenMultipleCM = false;
 		boolean seenCM = false;
-		for (int i = 0; i < stack.size(); i++) {
+        for (ContextManager stackCM : stack) {
 
-			Object stackCM = stack.elementAt(i);
 			if (stackCM != nextCM)
 				seenMultipleCM = true;
 
@@ -392,7 +405,7 @@ public final class ContextService //OLD extends Hashtable
     */
 	private boolean addToThreadList(Thread me, ContextManager associateCM) {
 
-		ThreadLocal tcl = threadContextList;
+		ThreadLocal<Object> tcl = threadContextList;
 
 		if (tcl == null) {
 			// The context service is already stopped.
@@ -412,7 +425,7 @@ public final class ContextService //OLD extends Hashtable
 			return true;
 		}
 
- 		java.util.Stack stack;
+        ContextManagerStack stack;
 		if (list instanceof ContextManager) {
             
             // Could be two situations:
@@ -432,7 +445,7 @@ public final class ContextService //OLD extends Hashtable
             
             // Nested, need to create a Stack of ContextManagers,
             // the top of the stack will be the active one.
-			stack = new java.util.Stack();
+            stack = new ContextManagerStack();
 			tcl.set(stack);
             
             // The stack represents the true nesting
@@ -449,7 +462,7 @@ public final class ContextService //OLD extends Hashtable
 		{
             // existing stack, nesting represented
             // by stack entries, not activeCount.
-			stack = (java.util.Stack) list;
+            stack = (ContextManagerStack) list;
 		}
 
 		stack.push(associateCM);
@@ -545,9 +558,7 @@ public final class ContextService //OLD extends Hashtable
 		Thread me = Thread.currentThread();
 
 		synchronized (this) {
-			for (Iterator i = allContexts.iterator(); i.hasNext(); ) {
-
-				ContextManager cm = (ContextManager) i.next();
+            for (ContextManager cm : allContexts) {
 
 				Thread active = cm.activeThread;
 
@@ -560,38 +571,17 @@ public final class ContextService //OLD extends Hashtable
                 final Thread fActive = active;
 				if (cm.setInterrupted(c))
                 {
-                    // DERBY-6352; in some cases a SecurityException is seen
-                    // demanding an explicit granting of modifyThread
-                    // permission, which should not be needed for Derby, as we
-                    // should not have any system threads.
                     try {
                         AccessController.doPrivileged(
-                                new PrivilegedAction() {
-                                    public Object run()  {
+                                new PrivilegedAction<Void>() {
+                                    public Void run()  {
                                         fActive.interrupt();
                                         return null;
                                     }
                                 });
                     } catch (java.security.AccessControlException ace) {
-                        // if sane, ASSERT and stop. The Assert will
-                        // cause info on all current threads to be printed to
-                        // the console, and we're also adding details about
-                        // the thread causing the security exception.
-                        // if insane, rethrow, and if an IBM JVM, do a jvmdump
-                        if (SanityManager.DEBUG)
-                        {
-                            SanityManager.THROWASSERT("unexpectedly needing " +
-                                    "an extra permission, for thread: " +
-                                    fActive.getName() + " with state: "+
-                                    fActive.getState());
-                            ace.printStackTrace();
-                        }
-                        else {
-                            if (JVMInfo.isIBMJVM()) {
-                                JVMInfo.javaDump();
-                            }
-                            throw ace;
-                        }
+                        // DERBY-6352; if we see an exception here, just
+                        // swallow it, leaving the thread to finish
                     }
                 }
 			}
@@ -607,4 +597,14 @@ public final class ContextService //OLD extends Hashtable
         if (allContexts != null)
             allContexts.remove( cm);
     }
+
+    /** Specialized stack class that contains context managers. */
+    private static class ContextManagerStack extends Stack<ContextManager> {
+        // The class is empty. Its primary purpose is to allow type-safe casts
+        // from Object, which are needed because the stacks live in a
+        // ThreadLocal<Object> rather than ThreadLocal<Stack<ContextManager>>.
+        // Casts from Object to Stack<ContextManager> will cause an unchecked
+        // conversion warning, whereas casts from Object to ContextManagerStack
+        // won't.
+    }
 }
