@@ -21,36 +21,34 @@
 
 package org.apache.derby.iapi.sql.conn;
 
+import java.util.HashMap;
+import java.util.Map;
+import org.apache.derby.catalog.UUID;
+import org.apache.derby.iapi.db.Database;
+import org.apache.derby.iapi.db.TriggerExecutionContext;
+import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.services.context.Context;
 import org.apache.derby.iapi.services.io.FormatableBitSet;
-import org.apache.derby.iapi.db.Database;
-import org.apache.derby.iapi.error.StandardException;
-import org.apache.derby.iapi.sql.compile.CompilerContext;
-import org.apache.derby.iapi.sql.dictionary.DataDictionary;
-import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
-import org.apache.derby.iapi.sql.dictionary.RoleGrantDescriptor;
-import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
-import org.apache.derby.iapi.sql.compile.OptimizerFactory;
-import org.apache.derby.iapi.types.DataValueFactory;
-
-import org.apache.derby.iapi.sql.compile.ASTVisitor;
-import org.apache.derby.iapi.sql.depend.Provider;
-import org.apache.derby.iapi.sql.execute.ConstantAction;
-import org.apache.derby.iapi.sql.execute.CursorActivation;
-import org.apache.derby.iapi.sql.execute.ExecutionContext;
-import org.apache.derby.iapi.sql.execute.ExecutionStmtValidator;
 import org.apache.derby.iapi.sql.Activation;
 import org.apache.derby.iapi.sql.LanguageFactory;
-import org.apache.derby.iapi.sql.PreparedStatement;
 import org.apache.derby.iapi.sql.ParameterValueSet;
-
-import org.apache.derby.iapi.store.access.TransactionController;
-import org.apache.derby.iapi.db.TriggerExecutionContext;
+import org.apache.derby.iapi.sql.PreparedStatement;
+import org.apache.derby.iapi.sql.compile.ASTVisitor;
+import org.apache.derby.iapi.sql.compile.CompilerContext;
+import org.apache.derby.iapi.sql.compile.OptTrace;
+import org.apache.derby.iapi.sql.compile.OptimizerFactory;
+import org.apache.derby.iapi.sql.depend.Provider;
+import org.apache.derby.iapi.sql.dictionary.ConstraintDescriptor;
+import org.apache.derby.iapi.sql.dictionary.DataDictionary;
+import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
+import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
+import org.apache.derby.iapi.sql.execute.ConstantAction;
+import org.apache.derby.iapi.sql.execute.CursorActivation;
+import org.apache.derby.iapi.sql.execute.ExecutionStmtValidator;
 import org.apache.derby.iapi.sql.execute.RunTimeStatistics;
-import org.apache.derby.catalog.UUID;
-
-import java.util.Map;
-import java.util.AbstractMap;
+import org.apache.derby.iapi.store.access.TransactionController;
+import org.apache.derby.iapi.types.DataValueFactory;
+import org.apache.derby.impl.sql.execute.DeferredConstraintsMemory;
 
 /**
  * LanguageConnectionContext keeps the result sets,
@@ -583,8 +581,6 @@ public interface LanguageConnectionContext extends Context {
 	 * Pop a CompilerContext off the context stack.
 	 *
 	 * @param compilerContext  The compiler context.
-	 *
-	 * @exception StandardException thrown on failure
 	 */
 	public void popCompilerContext(CompilerContext compilerContext);
 
@@ -790,7 +786,7 @@ public interface LanguageConnectionContext extends Context {
     /**
 	  *	Reports how many statement levels deep we are.
 	  *
-	  *	@return	a statement level >= OUTERMOST_STATEMENT
+	  *	@return	a statement level &gt;= OUTERMOST_STATEMENT
 	  */
 	public	int		getStatementDepth();
 
@@ -839,7 +835,7 @@ public interface LanguageConnectionContext extends Context {
 	 * Get the prepare isolation level.
 	 * If the isolation level has been explicitly set with a SQL statement or
 	 * embedded call to setTransactionIsolation, this will return
-	 * ExecutionContext.UNSPECIFIED_ISOLATION_LEVEL 
+	 * TransactionControl.UNSPECIFIED_ISOLATION_LEVEL 
 	 * SET ISOLATION always takes priority.
 	 * 
 	 */
@@ -907,57 +903,24 @@ public interface LanguageConnectionContext extends Context {
 	    throws StandardException;
 
 	/**
-	 * Control whether or not optimizer trace is on.
+	 * Install an optimizer tracer (to enable tracing) or uninstall the current optimizer tracer
+     * (to disable tracing).
 	 *
-	 * @param onOrOff    Whether to turn optimizer trace on (true) or off (false).
-	 *
-	 * @return Whether or not the call was successful.  (false will be returned when optimizer tracing is not supported.)
+	 * @param tracer    Null if tracing is being turned off, otherwise an optimizer tracer
 	 */
-	public boolean setOptimizerTrace(boolean onOrOff);
+	public void setOptimizerTracer( OptTrace tracer );
+
+	/**
+	 * Get the optimizer tracer (could be null if we aren't tracing the optimizer).
+	 */
+	public OptTrace getOptimizerTracer();
 
 	/** 
 	 * Get whether or not optimizer trace is on.
 	 *
 	 * @return Whether or not optimizer trace is on.
 	 */
-	public boolean getOptimizerTrace();
-
-	/**
-	 * Control whether or not optimizer trace is generated in html.
-	 *
-	 * @param onOrOff    Whether or not optimizer trace will be in html (true) or not (false).
-	 *
-	 * @return Whether or not the call was successful.  (false will be returned when optimizer tracing is not supported.)
-	 */
-	public boolean setOptimizerTraceHtml(boolean onOrOff);
-
-	/** 
-	 * Get whether or not optimizer trace html is on.
-	 *
-	 * @return Whether or not optimizer trace html is on.
-	 */
-	public boolean getOptimizerTraceHtml();
-
-	/**
-	 * Get the optimizer trace output for the last optimized query as a String.  If optimizer trace
-	 * html is on, then the String will contain the html tags.
-	 *
-	 * @return The optimizer trace output for the last optimized query as a String.
-	 *    Null will be returned if optimizer trace output is off or not supported 
-	 *    or no trace output was found or an exception occurred.
-	 */
-	public String getOptimizerTraceOutput();
-
-	/**
-	 * Set the optimizer trace output to the specified String.
-	 * (Done at the beginning of each statement.)
-	 */
-	public void setOptimizerTraceOutput(String startingText);
-
-	/**
-	 * Append the latest output to the optimizer trace output.
-	 */
-	public void appendOptimizerTraceOutput(String output);
+	public boolean optimizerTracingIsOn();
 
     /**
 	  *	Reports whether there is any outstanding work in the transaction.
@@ -989,8 +952,6 @@ public interface LanguageConnectionContext extends Context {
 	 * to an existing table.
 	 * 
 	 * @param flag 	the value for autoincrementUpdate (TRUE or FALSE)
-	 * @see org.apache.derby.impl.sql.execute.AlterTableConstantAction#updateNewAutoincrementColumn
-	 *
 	 */
 	public void setAutoincrementUpdate(boolean flag);
 
@@ -1005,7 +966,7 @@ public interface LanguageConnectionContext extends Context {
 	 * Copy a map of autoincrement key value pairs into the cache of
 	 * ai values stored in the language connection context.
 	 */
-	public void copyHashtableToAIHT(Map from);
+	public void copyHashtableToAIHT(Map<String,Long> from);
 	
 	/**
 	 * returns the <b>next</b> value to be inserted into an autoincrement col.
@@ -1037,7 +998,6 @@ public interface LanguageConnectionContext extends Context {
 	 *
 	 * @see LanguageConnectionContext#lastAutoincrementValue
 	 * @see org.apache.derby.impl.sql.conn.GenericLanguageConnectionContext#lastAutoincrementValue
-	 * @see org.apache.derby.iapi.db.ConnectionInfo#lastAutoincrementValue
 	 */
 	public void autoincrementFlushCache(UUID tableUUID)
 		throws StandardException;
@@ -1090,7 +1050,7 @@ public interface LanguageConnectionContext extends Context {
 	public String getDbname();
 
 	/**
-	 * Check if in SQL standard mode, with support for Grant & Revoke
+	 * Check if in SQL standard mode, with support for Grant and Revoke
 	 *
 	 * @return True if SQL standard permissions are being used
 	 */
@@ -1153,32 +1113,43 @@ public interface LanguageConnectionContext extends Context {
 	 * section 4.27.3, since this gives rise to a nested connection.
 	 * <p>
 	 * Called from generated code, see
-	 * {@code StaticMethodCallNode#generateSetupNestedSessionContext}.
+     * {@link org.apache.derby.impl.sql.compile.StaticMethodCallNode#generatePushNestedSessionContext}.
 	 * <p>
 	 * The new SQL session context is also set in the current statement
 	 * context (of the invocation).
 	 *
-	 * @see org.apache.derby.impl.sql.compile.StaticMethodCallNode#generateSetupNestedSessionContext
+     * @see org.apache.derby.impl.sql.compile.StaticMethodCallNode#generatePushNestedSessionContext
 	 * @see StatementContext#getSQLSessionContext
 	 * @see #setupSubStatementSessionContext
 	 *
 	 * @param a activation of the statement which performs the call.
      * @param definersRights if the method should run with definer's rights
      * @param definer authorization id of the definer
+     * @throws StandardException standard error policy
 	 */
-    public void setupNestedSessionContext(Activation a,
-                                          boolean definersRights,
-                                          String definer)
+    public void pushNestedSessionContext(Activation a,
+                                        boolean definersRights,
+                                        String definer)
             throws StandardException;
 
+    /**
+     * If returning from a routine that can execute SQL, perform any
+     * actions needed when popping the SQL session context.
+     *
+     * @param a activation
+     * @throws StandardException standard error policy
+     */
+    public void popNestedSessionContext(Activation a) throws StandardException;
+
 	/**
 	 * Get the value of top level session context of the top level connection.
+     * @return the requested session context
 	 */
 	public SQLSessionContext getTopLevelSQLSessionContext();
 
 	/**
 	 * Used when a statement as part of its operation executes an other
-	 * statement. In contrast to setupNestedSessionContext, the activation (for
+     * statement. In contrast to pushNestedSessionContext, the activation (for
 	 * the substatement) just inherits the current session context from the
 	 * parent statements activation, it does <b>not</b> push a new copy on the
 	 * stack of session contexts.
@@ -1195,7 +1166,7 @@ public interface LanguageConnectionContext extends Context {
 	 *         and {@code deleteRow}.
 	 *     <li>During trigger body execution.
 	 * </ul>
-	 * @see #setupNestedSessionContext
+     * @see #pushNestedSessionContext
 	 */
     public void setupSubStatementSessionContext(Activation a)
             throws StandardException;
@@ -1222,7 +1193,7 @@ public interface LanguageConnectionContext extends Context {
 	 * @see org.apache.derby.impl.sql.compile.QueryTreeNode#treePrint(int)
 	 * @return the map
 	 */
-    public Map getPrintedObjectsMap();
+    public Map<Object,Object> getPrintedObjectsMap();
 	
     /**
      * sets the XplainOnlyMode.
@@ -1303,4 +1274,70 @@ public interface LanguageConnectionContext extends Context {
      */
     public void setReferencedColumnMap(TableDescriptor td,
                                        FormatableBitSet map);
+
+    /**
+     * Set the constraint mode for this constraint to {@code deferred}.
+     * If {@code deferred} is {@code false}, to immediate checking,
+     * if {@code true} to deferred checking.
+     *
+     * @param a         Activation
+     * @param cd        The descriptor of the constraint
+     * @param deferred  The new constraint mode
+     * @throws StandardException
+     */
+    public void setConstraintDeferred(Activation a,
+                                    ConstraintDescriptor cd,
+                                    boolean deferred) throws StandardException;
+
+    /**
+     * Determines if a check or foreign key constraint has deferred
+     * mode.
+     *
+     * @param sc           The SQL session context for which we are asking the status
+     * @param constraintId The constraint id we are inquiring about.
+     * @return             {@code true} if the constraint is deferred
+     * @throws StandardException
+     *                     Standard error policy
+     */
+    public boolean isEffectivelyDeferred(SQLSessionContext sc, UUID constraintId)
+            throws StandardException;
+
+    /**
+     * Set the constraint mode of all deferrable constraints to the value of
+     * {@code deferred}. If the value is {@code false}, this method might
+     * throw with a constraint violation error, i.e. if some constraint
+     * has deferred mode before this call and had seen violations.
+     *
+     * @param a         The activation
+     * @param deferred  The value which holds the constraint mode
+     * @throws          StandardException Standard error policy
+     */
+    public void setDeferredAll(Activation a, boolean deferred)
+            throws StandardException;
+
+    /**
+     * Get the set of disk backed hash tables containing any index
+     * rows saved for deferred unique/PK constraints in this
+     * transaction, keyed by the conglomerate id, or rows saved
+     * containing row locations violating rows for deferred check
+     * constraints.
+     * @return the set
+     */
+    HashMap<UUID, DeferredConstraintsMemory.ValidationInfo>
+        getDeferredHashTables();
+
+    /**
+     * Check that deferred constraints are valid, if not roll back the
+     * transaction.
+     *
+     * @throws StandardException
+     */
+    public void checkIntegrity() throws StandardException;
+
+    /**
+     * Get the SQL session context of the given activation.
+     * @param activation The activation
+     * @return           The SQL session object
+     */
+    public SQLSessionContext getCurrentSQLSessionContext(Activation activation);
 }
