@@ -289,8 +289,8 @@ public interface RawStoreFactory extends Corruptable {
         if this property is set, it will attempt to allocate
         CONTAINER_INITIAL_PAGES, but with no guarentee.
         CONTAIENR_INITIAL_PAGES legally ranges from 1 to
-        MAX_CONTAINER_INITIAL_PAGES.  Values < 1 will
-        be set to 1 and values > MAX_CONTAINER_INITIAL_PAGES will be set to
+        MAX_CONTAINER_INITIAL_PAGES.  Values &lt; 1 will
+        be set to 1 and values &gt; MAX_CONTAINER_INITIAL_PAGES will be set to
         MAX_CONTAINER_INITIAL_PAGES
 
         This property should only be set in the PROPERTIES list in a CREATE
@@ -429,6 +429,25 @@ public interface RawStoreFactory extends Corruptable {
     */
     public LockFactory getLockFactory();
 
+    /**
+        Register a handler class for insert undo events.
+        <P>
+        Register a class to be called when an undo of an insert 
+        is executed.  When an undo of an event is executed by
+        the raw store UndoHandler.insertUndoNotify() will be
+        called, allowing upper level callers to execute code
+        as necessary.  The initial need is for the access layer
+        to be able to queue post commit reclaim space in the
+        case of inserts which are aborted (including the normal
+        case of inserts failed for duplicate key violations)
+        (see DERBY-4057)
+
+        @exception StandardException Standard Derby error policy
+
+    */
+    public void setUndoInsertEventHandler(
+        UndoHandler undo_handle) throws StandardException;
+
 
     /**
         Create a user transaction, almost all work within the raw store is
@@ -660,6 +679,7 @@ public interface RawStoreFactory extends Corruptable {
         context is popped off the stack.
         </UL>
 
+        @param parentTransaction parent transaction
         @param compatibilitySpace compatibility space to use for locks.
         @param contextMgr is the context manager to use.  An exception will be
         thrown if context is not the current context.
@@ -674,6 +694,7 @@ public interface RawStoreFactory extends Corruptable {
     */
 
     public Transaction startNestedReadOnlyUserTransaction(
+    Transaction parentTransaction,
     CompatibilitySpace compatibilitySpace,
     ContextManager contextMgr,
     String         transName)
@@ -720,6 +741,7 @@ public interface RawStoreFactory extends Corruptable {
         context is popped off the stack.
         </UL>
 
+        @param parentTransaction parent transaction
         @param contextMgr is the context manager to use.  An exception will be
         thrown if context is not the current context.
         @param transName is the name of the transaction. This name will be 
@@ -737,6 +759,7 @@ public interface RawStoreFactory extends Corruptable {
     */
 
     public Transaction startNestedUpdateUserTransaction(
+    Transaction parentTransaction,
     ContextManager contextMgr,
     String         transName,
     boolean        flush_log_on_xact_end)
