@@ -21,7 +21,7 @@
 
 package org.apache.derby.iapi.types;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.services.io.Storable;
 import org.apache.derby.iapi.services.io.StoredFormatIds;
@@ -35,10 +35,17 @@ import org.apache.derby.iapi.util.StringUtil;
 import java.io.ObjectOutput;
 import java.io.ObjectInput;
 import java.io.IOException;
+import java.math.BigDecimal;
 
 import java.sql.ResultSet;
 import java.sql.PreparedStatement;
 import java.sql.SQLException;
+import org.apache.derby.catalog.UUID;
+import org.apache.derby.iapi.sql.Activation;
+import org.apache.derby.iapi.sql.Row;
+import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
+import org.apache.derby.iapi.sql.execute.ExecPreparedStatement;
+import org.apache.derby.impl.sql.execute.DMLWriteResultSet;
 
 /**
  * SQLBoolean satisfies the DataValueDescriptor
@@ -155,7 +162,7 @@ public final class SQLBoolean
 		if (isNull())
 			return null;
 		else
-			return new Boolean(value);
+			return value;
 	}
 
 	public int	getLength()
@@ -482,7 +489,7 @@ public final class SQLBoolean
 
 	}
 
-	public void setBigDecimal(Number bigDecimal) throws StandardException
+	public void setBigDecimal(BigDecimal bigDecimal) throws StandardException
 	{
 		if (SanityManager.DEBUG)
 			SanityManager.ASSERT( ! immutable,
@@ -494,9 +501,7 @@ public final class SQLBoolean
 		}
 		else
 		{
-			DataValueDescriptor tempDecimal = NumberDataType.ZERO_DECIMAL.getNewNull();
-			tempDecimal.setBigDecimal(bigDecimal);
-			value = NumberDataType.ZERO_DECIMAL.compare(tempDecimal) != 0;
+            value = BigDecimal.ZERO.compareTo(bigDecimal) != 0;
 			isnull = false;
 		}
 
@@ -598,11 +603,11 @@ public final class SQLBoolean
 	}
 
 	/**
-	 * The <> operator as called from the language module, as opposed to
+	 * The &lt;&gt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <>
-	 * @param right			The value on the right side of the <>
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the two parameters are
 	 *			not equal
@@ -620,11 +625,11 @@ public final class SQLBoolean
 	}
 
 	/**
-	 * The < operator as called from the language module, as opposed to
+	 * The &lt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <
-	 * @param right			The value on the right side of the <
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the left operand is
 	 *			less than the right operand
@@ -648,11 +653,11 @@ public final class SQLBoolean
 	}
 
 	/**
-	 * The > operator as called from the language module, as opposed to
+	 * The &gt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the >
-	 * @param right			The value on the right side of the >
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the left operand is
 	 *			greater than the right operand
@@ -676,11 +681,11 @@ public final class SQLBoolean
 	}
 
 	/**
-	 * The <= operator as called from the language module, as opposed to
+	 * The &lt;= operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <=
-	 * @param right			The value on the right side of the <=
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the left operand is
 	 *			less than or equal to the right operand
@@ -704,11 +709,11 @@ public final class SQLBoolean
 	}
 
 	/**
-	 * The >= operator as called from the language module, as opposed to
+	 * The &gt;= operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the >=
-	 * @param right			The value on the right side of the >=
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the left operand is
 	 *			greater than or equal to the right operand
@@ -867,6 +872,36 @@ public final class SQLBoolean
 		return this;
 	}
 
+    public BooleanDataValue throwExceptionIfImmediateAndFalse(
+                                    String sqlState,
+                                    String tableName,
+                                    String constraintName,
+                                    Activation a,
+                                    int savedUUIDIdx)
+                            throws StandardException
+    {
+        if ( !isNull() && (value == false) ) {
+            final ExecPreparedStatement ps = a.getPreparedStatement();
+            final UUID constrId = (UUID)ps.getSavedObject(savedUUIDIdx);
+            final LanguageConnectionContext lcc =
+                a.getLanguageConnectionContext();
+            final boolean isDeferred = lcc.isEffectivelyDeferred(
+                   lcc.getCurrentSQLSessionContext(a), constrId);
+
+            if (!isDeferred) {
+                throw StandardException.newException(
+                        sqlState, tableName, constraintName);
+            } else {
+                // Just return the false value and validate later,
+                // cf NoRowsResultSetImpl#evaluateCheckConstraints.
+                // and InsertResultSet#evaluateCheckConstraints
+                DMLWriteResultSet rs =  (DMLWriteResultSet)a.getResultSet();
+                rs.rememberConstraint(constrId);
+            }
+        }
+
+        return this;
+    }
 	/*
 	 * DataValueDescriptor interface
 	 */
