@@ -21,23 +21,18 @@
 
 package org.apache.derby.iapi.types;
 
-import org.apache.derby.iapi.reference.SQLState;
-
-import org.apache.derby.iapi.services.io.StoredFormatIds;
-import org.apache.derby.iapi.services.io.Storable;
-
-import org.apache.derby.iapi.error.StandardException;
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
-import org.apache.derby.iapi.services.cache.ClassSize;
-
-import java.io.ObjectOutput;
-import java.io.ObjectInput;
 import java.io.IOException;
-
-import java.sql.ResultSet;
+import java.io.ObjectInput;
+import java.io.ObjectOutput;
 import java.sql.PreparedStatement;
+import java.sql.ResultSet;
 import java.sql.SQLException;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.cache.ClassSize;
+import org.apache.derby.iapi.services.io.Storable;
+import org.apache.derby.iapi.services.io.StoredFormatIds;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 /**
  * SQLLongint satisfies the DataValueDescriptor
@@ -137,7 +132,7 @@ public final class SQLLongint
 		if (isNull())
 			return null;
 		else
-			return new Long(value);
+			return value;
 	}
 
 	public int	getLength()
@@ -148,7 +143,7 @@ public final class SQLLongint
 	// this is for DataType's error generator
 	public String getTypeName()
 	{
-		return TypeId.LONGINT_NAME;
+        return TypeId.BIGINT_NAME;
 	}
 
 	/*
@@ -460,11 +455,11 @@ public final class SQLLongint
 	}
 
 	/**
-	 * The <> operator as called from the language module, as opposed to
+	 * The &lt;&gt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <>
-	 * @param right			The value on the right side of the <>
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the two parameters
 	 *			are not equal
@@ -482,11 +477,11 @@ public final class SQLLongint
 	}
 
 	/**
-	 * The < operator as called from the language module, as opposed to
+	 * The &lt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <
-	 * @param right			The value on the right side of the <
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is less
 	 *			than the second operand
@@ -504,11 +499,11 @@ public final class SQLLongint
 	}
 
 	/**
-	 * The > operator as called from the language module, as opposed to
+	 * The &gt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the >
-	 * @param right			The value on the right side of the >
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is greater
 	 *			than the second operand
@@ -526,11 +521,11 @@ public final class SQLLongint
 	}
 
 	/**
-	 * The <= operator as called from the language module, as opposed to
+	 * The &lt;= operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <=
-	 * @param right			The value on the right side of the <=
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is less
 	 *			than or equal to the second operand
@@ -548,11 +543,11 @@ public final class SQLLongint
 	}
 
 	/**
-	 * The >= operator as called from the language module, as opposed to
+	 * The &gt;= operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the >=
-	 * @param right			The value on the right side of the >=
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is greater
 	 *			than or equal to the second operand
