@@ -27,7 +27,7 @@ import org.apache.derby.iapi.services.io.StoredFormatIds;
 import org.apache.derby.iapi.services.io.Storable;
 
 import org.apache.derby.iapi.error.StandardException;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.services.cache.ClassSize;
 
@@ -158,7 +158,7 @@ public final class SQLSmallint
 		if (isNull())
 			return null;
 		else
-			return new Integer(value);
+          return (int) value;
 	}
 
 	// this is for DataType's error generator
@@ -477,11 +477,11 @@ public final class SQLSmallint
 	}
 
 	/**
-	 * The <> operator as called from the language module, as opposed to
+	 * The &lt;&gt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <>
-	 * @param right			The value on the right side of the <>
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the two parameters
 	 *			are not equal
@@ -499,11 +499,11 @@ public final class SQLSmallint
 	}
 
 	/**
-	 * The < operator as called from the language module, as opposed to
+	 * The &lt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <
-	 * @param right			The value on the right side of the <
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *						is not.
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is less
@@ -522,11 +522,11 @@ public final class SQLSmallint
 	}
 
 	/**
-	 * The > operator as called from the language module, as opposed to
+	 * The &gt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the >
-	 * @param right			The value on the right side of the >
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is greater
 	 *			than the second operand
@@ -544,11 +544,11 @@ public final class SQLSmallint
 	}
 
 	/**
-	 * The <= operator as called from the language module, as opposed to
+	 * The &lt;= operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <=
-	 * @param right			The value on the right side of the <=
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is less
 	 *			than or equal to the second operand
@@ -566,11 +566,11 @@ public final class SQLSmallint
 	}
 
 	/**
-	 * The >= operator as called from the language module, as opposed to
+	 * The &gt;= operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the >=
-	 * @param right			The value on the right side of the >=
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is greater
 	 *			than or equal to the second operand
