@@ -27,13 +27,11 @@ import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.db.DatabaseContext;
 
 import org.apache.derby.iapi.services.io.StoredFormatIds;
-import org.apache.derby.iapi.services.context.ContextService;
  
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.services.i18n.LocaleFinder;
 import org.apache.derby.iapi.services.cache.ClassSize;
 import org.apache.derby.iapi.util.StringUtil;
-import org.apache.derby.iapi.util.ReuseFactory;
 
 import java.sql.Date;
 import java.sql.Time;
@@ -382,10 +380,28 @@ public final class SQLTimestamp extends DataType
 	/** no-arg constructor required by Formattable */
 	public SQLTimestamp() { }
 
+    /**
+     * Create a new SQLTimestamp instance that represents the specified
+     * Timestamp in the time zone of the given Calendar.
+     * @param value the Timestamp value to be represented by this instance
+     * @param cal the time zone of the calendar is used to construct the
+     * database timestamp value
+     * @throws StandardException if an error occurs
+     */
+    public SQLTimestamp(Timestamp value, Calendar cal) throws StandardException
+    {
+        setValue(value, cal);
+    }
 
+    /**
+     * Create a new SQLTimestamp instance that represents the specified
+     * Timestamp in the local time zone.
+     * @param value the Timestamp value to be represented by this instance
+     * @throws StandardException if an error occurs
+     */
 	public SQLTimestamp(Timestamp value) throws StandardException
 	{
-		setValue(value, (Calendar) null);
+        this(value, (Calendar) null);
 	}
 
 	SQLTimestamp(int encodedDate, int encodedTime, int nanos) {
@@ -637,7 +653,7 @@ public final class SQLTimestamp extends DataType
 
 		if (theValue != null)
 		{
-            DatabaseContext databaseContext = (DatabaseContext) ContextService.getContext(DatabaseContext.CONTEXT_ID);
+            DatabaseContext databaseContext = (DatabaseContext) DataValueFactoryImpl.getContext(DatabaseContext.CONTEXT_ID);
             parseTimestamp( theValue,
                             false,
                             (databaseContext == null) ? null : databaseContext.getDatabase(),
@@ -864,7 +880,7 @@ public final class SQLTimestamp extends DataType
 		computeEncodedDate sets the date in a Calendar object
 		and then uses the SQLDate function to compute an encoded date
 		The encoded date is
-			year << 16 + month << 8 + date
+			year &lt;&lt; 16 + month &lt;&lt; 8 + date
 		@param value	the value to convert
 		@return 		the encodedDate
 
@@ -880,7 +896,7 @@ public final class SQLTimestamp extends DataType
 	/**
 		computeEncodedTime extracts the hour, minute and seconds from
 		a java.util.Date value and encodes them as
-			hour << 16 + minute << 8 + second
+			hour &lt;&lt; 16 + minute &lt;&lt; 8 + second
 		using the SQLTime function for encoding the data
 		@param value	the value to convert
 		@return 		the encodedTime
@@ -1038,8 +1054,7 @@ public final class SQLTimestamp extends DataType
             break;
 
         default:
-            throw StandardException.newException( SQLState.LANG_INVALID_FUNCTION_ARGUMENT,
-                                                  ReuseFactory.getInteger( intervalType),
+            throw StandardException.newException(SQLState.LANG_INVALID_FUNCTION_ARGUMENT, intervalType,
                                                   "TIMESTAMPADD");
         }
         return tsResult;
@@ -1220,8 +1235,7 @@ public final class SQLTimestamp extends DataType
             break;
 
         default:
-            throw StandardException.newException( SQLState.LANG_INVALID_FUNCTION_ARGUMENT,
-                                                  ReuseFactory.getInteger( intervalType),
+            throw StandardException.newException(SQLState.LANG_INVALID_FUNCTION_ARGUMENT, intervalType,
                                                   "TIMESTAMPDIFF");
         }
         resultHolder.setValue(ldiff);
