@@ -22,64 +22,51 @@
 
 package org.apache.derby.impl.sql.compile;
 
-import org.apache.derby.catalog.UUID;
-
-import org.apache.derby.iapi.sql.conn.LanguageConnectionFactory;
-
-import org.apache.derby.iapi.sql.depend.ProviderList;
-import org.apache.derby.iapi.sql.compile.CompilerContext;
-import org.apache.derby.iapi.sql.compile.NodeFactory;
-import org.apache.derby.iapi.sql.compile.Parser;
-
-import org.apache.derby.iapi.sql.conn.Authorizer;
-import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
-
-import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
-import org.apache.derby.iapi.sql.dictionary.ColumnDescriptor;
-import org.apache.derby.iapi.sql.dictionary.SequenceDescriptor;
-import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
-import org.apache.derby.iapi.sql.dictionary.AliasDescriptor;
-import org.apache.derby.iapi.sql.dictionary.PermDescriptor;
-import org.apache.derby.iapi.sql.dictionary.PrivilegedSQLObject;
-import org.apache.derby.iapi.sql.dictionary.StatementGenericPermission;
-import org.apache.derby.iapi.sql.dictionary.StatementTablePermission;
-import org.apache.derby.iapi.sql.dictionary.StatementSchemaPermission;
-import org.apache.derby.iapi.sql.dictionary.StatementColumnPermission;
-import org.apache.derby.iapi.sql.dictionary.StatementRoutinePermission;
-import org.apache.derby.iapi.sql.dictionary.StatementRolePermission;
-
-
-import org.apache.derby.iapi.sql.compile.TypeCompilerFactory;
-
-import org.apache.derby.iapi.sql.depend.Dependent;
-import org.apache.derby.iapi.sql.depend.Provider;
-import org.apache.derby.iapi.sql.depend.DependencyManager;
-import org.apache.derby.iapi.error.ExceptionSeverity;
-import org.apache.derby.iapi.sql.execute.ExecutionContext;
-
-import org.apache.derby.iapi.types.DataTypeDescriptor;
-
-import org.apache.derby.iapi.store.access.StoreCostController;
-import org.apache.derby.iapi.store.access.SortCostController;
-
-import org.apache.derby.iapi.services.context.ContextManager;
-import org.apache.derby.iapi.services.loader.ClassFactory;
-import org.apache.derby.iapi.services.compiler.JavaFactory;
-import org.apache.derby.iapi.services.io.FormatableBitSet;
-
-import org.apache.derby.iapi.error.StandardException;
-
-
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
-import org.apache.derby.iapi.services.context.ContextImpl;
-import org.apache.derby.iapi.util.ReuseFactory;
-
 import java.sql.SQLWarning;
+import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
-import java.util.ArrayList;
+import org.apache.derby.catalog.UUID;
+import org.apache.derby.iapi.error.ExceptionSeverity;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.services.compiler.JavaFactory;
+import org.apache.derby.iapi.services.context.ContextImpl;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.iapi.services.io.FormatableBitSet;
+import org.apache.derby.iapi.services.loader.ClassFactory;
+import org.apache.derby.shared.common.sanity.SanityManager;
+import org.apache.derby.iapi.sql.compile.CompilerContext;
+import org.apache.derby.iapi.sql.compile.OptimizerFactory;
+import org.apache.derby.iapi.sql.compile.Parser;
+import org.apache.derby.iapi.sql.compile.Visitable;
+import org.apache.derby.iapi.sql.compile.VisitableFilter;
+import org.apache.derby.iapi.sql.compile.TypeCompilerFactory;
+import org.apache.derby.iapi.sql.conn.Authorizer;
+import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
+import org.apache.derby.iapi.sql.conn.LanguageConnectionFactory;
+import org.apache.derby.iapi.sql.depend.DependencyManager;
+import org.apache.derby.iapi.sql.depend.Dependent;
+import org.apache.derby.iapi.sql.depend.Provider;
+import org.apache.derby.iapi.sql.depend.ProviderList;
+import org.apache.derby.iapi.sql.dictionary.AliasDescriptor;
+import org.apache.derby.iapi.sql.dictionary.ColumnDescriptor;
+import org.apache.derby.iapi.sql.dictionary.PermDescriptor;
+import org.apache.derby.iapi.sql.dictionary.PrivilegedSQLObject;
+import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
+import org.apache.derby.iapi.sql.dictionary.SequenceDescriptor;
+import org.apache.derby.iapi.sql.dictionary.StatementColumnPermission;
+import org.apache.derby.iapi.sql.dictionary.StatementGenericPermission;
+import org.apache.derby.iapi.sql.dictionary.StatementPermission;
+import org.apache.derby.iapi.sql.dictionary.StatementRolePermission;
+import org.apache.derby.iapi.sql.dictionary.StatementRoutinePermission;
+import org.apache.derby.iapi.sql.dictionary.StatementSchemaPermission;
+import org.apache.derby.iapi.sql.dictionary.StatementTablePermission;
+import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
+import org.apache.derby.iapi.store.access.SortCostController;
+import org.apache.derby.iapi.store.access.StoreCostController;
+import org.apache.derby.iapi.transaction.TransactionControl;
+import org.apache.derby.iapi.types.DataTypeDescriptor;
 
 /**
  *
@@ -90,6 +77,8 @@ import java.util.ArrayList;
 public class CompilerContextImpl extends ContextImpl
 	implements CompilerContext {
 
+    private static  final   int SCOPE_CELL = 0;
+    
 	//
 	// Context interface       
 	//
@@ -146,7 +135,7 @@ public class CompilerContextImpl extends ContextImpl
 		compilationSchema = null;
 		parameterList = null;
 		parameterDescriptors = null;
-		scanIsolationLevel = ExecutionContext.UNSPECIFIED_ISOLATION_LEVEL;
+		scanIsolationLevel = TransactionControl.UNSPECIFIED_ISOLATION_LEVEL;
 		warnings = null;
 		savedObjects = null;
 		reliability = CompilerContext.SQL_LEGAL;
@@ -154,6 +143,9 @@ public class CompilerContextImpl extends ContextImpl
 		initRequiredPriv();
 		defaultSchemaStack = null;
         referencedSequences = null;
+        privilegeCheckFilters =  null;
+        namedScopes = null;
+        skippingTypePrivileges = false;
 	}
 
 	//
@@ -168,12 +160,12 @@ public class CompilerContextImpl extends ContextImpl
 	}
 
 	/**
-	  *	Get the NodeFactory for this context
+      * Get the OptimizerFactory for this context
 	  *
-	  *	@return	The NodeFactory for this context.
+      * @return The OptimizerFactory for this context.
 	  */
-	public	NodeFactory	getNodeFactory()
-	{	return lcf.getNodeFactory(); }
+    public  OptimizerFactory    getOptimizerFactory()
+    {   return lcf.getOptimizerFactory(); }
 
 
 	public int getNextColumnNumber()
@@ -330,7 +322,7 @@ public class CompilerContextImpl extends ContextImpl
 
 	public int addSavedObject(Object obj) {
         if (savedObjects == null) {
-            savedObjects = new ArrayList();
+            savedObjects = new ArrayList<Object>();
         }
 
 		savedObjects.add(obj);
@@ -346,17 +338,12 @@ public class CompilerContextImpl extends ContextImpl
 	}
 
 	/** @see CompilerContext#setSavedObjects */
-	public void setSavedObjects(Object[] objs) 
+    public void setSavedObjects(List<Object> objs)
 	{
-		if (objs == null)
-		{
-			return;
-		}
-
-		for (int i = 0; i < objs.length; i++)
-		{
-			addSavedObject(objs[i]);
-		}		
+        Iterator<Object> it = objs.iterator();
+        while (it.hasNext()) {
+            addSavedObject(it.next());
+        }
 	}
 
 	/** @see CompilerContext#setCursorInfo */
@@ -441,11 +428,11 @@ public class CompilerContextImpl extends ContextImpl
 	public StoreCostController getStoreCostController(long conglomerateNumber)
 			throws StandardException
 	{
-        Long conglomNum = ReuseFactory.getLong(conglomerateNumber);
+        Long conglomNum = conglomerateNumber;
 
         // Try to find the given conglomerate number among the already
         // opened conglomerates.
-        StoreCostController retval = (StoreCostController)
+        StoreCostController retval =
                 storeCostControllers.get(conglomNum);
 
         if (retval == null) {
@@ -463,10 +450,10 @@ public class CompilerContextImpl extends ContextImpl
 	 */
 	private void closeStoreCostControllers()
 	{
-        Iterator it = storeCostControllers.values().iterator();
+        Iterator<StoreCostController> it = storeCostControllers.values().iterator();
         while (it.hasNext())
 		{
-            StoreCostController scc = (StoreCostController) it.next();
+            StoreCostController scc = it.next();
 			try {
 				scc.close();
 			} catch (StandardException se) {
@@ -544,7 +531,7 @@ public class CompilerContextImpl extends ContextImpl
 	public void pushCompilationSchema(SchemaDescriptor sd)
 	{
 		if (defaultSchemaStack == null) {
-			defaultSchemaStack = new ArrayList(2);
+			defaultSchemaStack = new ArrayList<SchemaDescriptor>(2);
 		}
 
 		defaultSchemaStack.add(defaultSchemaStack.size(),
@@ -558,7 +545,7 @@ public class CompilerContextImpl extends ContextImpl
 	public void popCompilationSchema()
 	{
 		SchemaDescriptor sd =
-			(SchemaDescriptor)defaultSchemaStack.remove(
+			defaultSchemaStack.remove(
 				defaultSchemaStack.size() - 1);
 		setCompilationSchema(sd);
 	}
@@ -566,7 +553,7 @@ public class CompilerContextImpl extends ContextImpl
 	/**
 	 * @see CompilerContext#setParameterList
 	 */
-	public void setParameterList(List parameterList)
+    public void setParameterList(List<ParameterNode> parameterList)
 	{
 		this.parameterList = parameterList;
 
@@ -582,7 +569,7 @@ public class CompilerContextImpl extends ContextImpl
 	/**
 	 * @see CompilerContext#getParameterList
 	 */
-	public List getParameterList()
+    public List<ParameterNode> getParameterList()
 	{
 		return parameterList;
 	}
@@ -604,9 +591,9 @@ public class CompilerContextImpl extends ContextImpl
 	}
 
 	/**
-	 * @see CompilerContext#getParameterTypes
+     * Get an array of type descriptors for all the ? parameters.
 	 */
-	public DataTypeDescriptor[] getParameterTypes()
+    DataTypeDescriptor[] getParameterTypes()
 	{
 		return parameterDescriptors;
 	}
@@ -653,16 +640,16 @@ public class CompilerContextImpl extends ContextImpl
 		return warnings;
 	}
 
-	/////////////////////////////////////////////////////////////////////////////////////
-	//
-	// class interface
-	//
-	// this constructor is called with the parser
-	// to be saved when the context
-	// is created (when the first statement comes in, likely).
-	//
-	/////////////////////////////////////////////////////////////////////////////////////
+    /*
+     * class interface
+     */
 
+    /**
+     * This constructor creates and saves the parser using the language
+     * connection factory when the context is created (when the first statement
+     * comes in, likely).
+     */
+    @SuppressWarnings("LeakingThisInConstructor")
 	public CompilerContextImpl(ContextManager cm,
 			LanguageConnectionContext lcc,
 		TypeCompilerFactory typeCompilerFactory )
@@ -690,16 +677,15 @@ public class CompilerContextImpl extends ContextImpl
 		requiredRoutinePrivileges = null;
 		requiredUsagePrivileges = null;
 		requiredRolePrivileges = null;
-		LanguageConnectionContext lcc = (LanguageConnectionContext)
-		getContextManager().getContext(LanguageConnectionContext.CONTEXT_ID);
-		if( lcc.usesSqlAuthorization())
+
+        if( lcc.usesSqlAuthorization())
 		{
-			requiredColumnPrivileges = new HashMap();
-			requiredTablePrivileges = new HashMap();
-			requiredSchemaPrivileges = new HashMap();
-			requiredRoutinePrivileges = new HashMap();
-			requiredUsagePrivileges = new HashMap();
-			requiredRolePrivileges = new HashMap();
+			requiredColumnPrivileges = new HashMap<StatementTablePermission,StatementColumnPermission>();
+			requiredTablePrivileges = new HashMap<StatementTablePermission,StatementTablePermission>();
+			requiredSchemaPrivileges = new HashMap<StatementSchemaPermission,StatementSchemaPermission>();
+			requiredRoutinePrivileges = new HashMap<UUID,Integer>();
+			requiredUsagePrivileges = new HashMap<UUID,String>();
+			requiredRolePrivileges = new HashMap<StatementRolePermission,StatementRolePermission>();
 		}
 	} // end of initRequiredPriv
 
@@ -713,13 +699,13 @@ public class CompilerContextImpl extends ContextImpl
 	 */
 	public void pushCurrentPrivType( int privType)
 	{
-		privTypeStack.add(ReuseFactory.getInteger(currPrivType));
+		privTypeStack.add(currPrivType);
 		currPrivType = privType;
 	}
 
 	public void popCurrentPrivType( )
 	{
-        Integer top = (Integer) privTypeStack.remove(privTypeStack.size() - 1);
+        Integer top = privTypeStack.remove(privTypeStack.size() - 1);
         currPrivType = top.intValue();
 	}
 
@@ -737,7 +723,10 @@ public class CompilerContextImpl extends ContextImpl
 			|| currPrivType == Authorizer.TRIGGER_PRIV // Table privilege only
 			|| currPrivType == Authorizer.EXECUTE_PRIV
 			|| column == null)
+        {
 			return;
+        }
+
 		/*
 		* Note that to look up the privileges for this column,
 		* we need to know what table the column is in. However,
@@ -791,7 +780,7 @@ public class CompilerContextImpl extends ContextImpl
 		
 		StatementTablePermission key = new StatementTablePermission( tableUUID, currPrivType);
 		StatementColumnPermission tableColumnPrivileges
-		  = (StatementColumnPermission) requiredColumnPrivileges.get( key);
+		  = requiredColumnPrivileges.get( key);
 		if( tableColumnPrivileges == null)
 		{
 			tableColumnPrivileges = new StatementColumnPermission( tableUUID,
@@ -848,7 +837,7 @@ public class CompilerContextImpl extends ContextImpl
 			return;
 
  		if (requiredRoutinePrivileges.get(routine.getUUID()) == null)
- 			requiredRoutinePrivileges.put(routine.getUUID(), ReuseFactory.getInteger(1));
+			requiredRoutinePrivileges.put(routine.getUUID(), 1);
 	}
 
 	/**
@@ -902,7 +891,7 @@ public class CompilerContextImpl extends ContextImpl
 	/**
 	 * @return The list of required privileges.
 	 */
-	public List getRequiredPermissionsList()
+	public List<StatementPermission> getRequiredPermissionsList()
 	{
 		int size = 0;
 		if( requiredRoutinePrivileges != null)
@@ -918,49 +907,52 @@ public class CompilerContextImpl extends ContextImpl
 		if( requiredRolePrivileges != null)
         { size += requiredRolePrivileges.size(); }
 		
-		ArrayList list = new ArrayList( size);
+		ArrayList<StatementPermission> list = new ArrayList<StatementPermission>( size);
 		if( requiredRoutinePrivileges != null)
 		{
-			for( Iterator itr = requiredRoutinePrivileges.keySet().iterator(); itr.hasNext();)
+			for( Iterator<UUID> itr = requiredRoutinePrivileges.keySet().iterator(); itr.hasNext();)
 			{
-				UUID routineUUID = (UUID) itr.next();
+				UUID routineUUID = itr.next();
 				
 				list.add( new StatementRoutinePermission( routineUUID));
 			}
 		}
 		if( requiredUsagePrivileges != null)
 		{
-			for( Iterator itr = requiredUsagePrivileges.keySet().iterator(); itr.hasNext();)
+			for( Iterator<UUID> itr = requiredUsagePrivileges.keySet().iterator(); itr.hasNext();)
 			{
-				UUID objectID = (UUID) itr.next();
+				UUID objectID = itr.next();
 				
-				list.add( new StatementGenericPermission( objectID, (String) requiredUsagePrivileges.get( objectID ), PermDescriptor.USAGE_PRIV ) );
+                list.add(new StatementGenericPermission(
+                        objectID,
+                        requiredUsagePrivileges.get(objectID),
+                        PermDescriptor.USAGE_PRIV));
 			}
 		}
 		if( requiredTablePrivileges != null)
 		{
-			for( Iterator itr = requiredTablePrivileges.values().iterator(); itr.hasNext();)
+			for( Iterator<StatementTablePermission> itr = requiredTablePrivileges.values().iterator(); itr.hasNext();)
 			{
 				list.add( itr.next());
 			}
 		}
 		if( requiredSchemaPrivileges != null)
 		{
-			for( Iterator itr = requiredSchemaPrivileges.values().iterator(); itr.hasNext();)
+			for( Iterator<StatementSchemaPermission> itr = requiredSchemaPrivileges.values().iterator(); itr.hasNext();)
 			{
 				list.add( itr.next());
 			}
 		}
 		if( requiredColumnPrivileges != null)
 		{
-			for( Iterator itr = requiredColumnPrivileges.values().iterator(); itr.hasNext();)
+			for( Iterator<StatementColumnPermission> itr = requiredColumnPrivileges.values().iterator(); itr.hasNext();)
 			{
 				list.add( itr.next());
 			}
 		}
 		if( requiredRolePrivileges != null)
 		{
-			for( Iterator itr = requiredRolePrivileges.values().iterator();
+			for( Iterator<StatementRolePermission> itr = requiredRolePrivileges.values().iterator();
 				 itr.hasNext();)
 			{
 				list.add( itr.next());
@@ -971,7 +963,7 @@ public class CompilerContextImpl extends ContextImpl
 
 	public void addReferencedSequence( SequenceDescriptor sd )
     {
-        if ( referencedSequences == null ) { referencedSequences = new HashMap(); }
+        if ( referencedSequences == null ) { referencedSequences = new HashMap<UUID,SequenceDescriptor>(); }
 
         referencedSequences.put( sd.getUUID(), sd );
     }
@@ -986,8 +978,88 @@ public class CompilerContextImpl extends ContextImpl
         return referencedSequences.containsKey( sd.getUUID() );
     }
 
+    public  void    addPrivilegeFilter( VisitableFilter vf )
+    {
+        if ( privilegeCheckFilters == null ) { privilegeCheckFilters = new ArrayList<VisitableFilter>(); }
+        
+        privilegeCheckFilters.add( vf );
+    }
+
+    public  void    removePrivilegeFilter( VisitableFilter vf )
+    {
+        if ( (vf != null) && (privilegeCheckFilters != null) )
+        {
+            privilegeCheckFilters.remove( vf );
+        }
+    }
+
+    public  boolean passesPrivilegeFilters( Visitable visitable )
+        throws StandardException
+    {
+        // if there are no filters, then all QueryTreeNodes pass.
+        if ( privilegeCheckFilters == null ) { return true; }
+
+        for ( VisitableFilter filter : privilegeCheckFilters )
+        {
+            if ( !filter.accept( visitable ) ) { return false; }
+        }
+
+        return true;
+    }
+    
+    public  void    beginScope( String scopeName )
+    {
+        if ( namedScopes == null ) { namedScopes = new HashMap<String,int[]>(); }
+        
+        int[]   scopeDepth = namedScopes.get( scopeName );
+        if ( scopeDepth == null )
+        {
+            scopeDepth = new int[ SCOPE_CELL + 1 ];
+            namedScopes.put( scopeName, scopeDepth );
+        }
+
+        scopeDepth[ SCOPE_CELL ]++;
+    }
+    
+    public  void    endScope( String scopeName )
+    {
+        if ( namedScopes == null ) { return; }
+
+        int[]   scopeDepth = namedScopes.get( scopeName );
+        if ( scopeDepth == null )   { return; }
+
+        scopeDepth[ SCOPE_CELL ]--;
+
+        if ( scopeDepth[ SCOPE_CELL ] <= 0 ) { namedScopes.remove( scopeName ); }
+    }
+
+    public  int     scopeDepth( String scopeName )
+    {
+        if ( namedScopes == null ) { return 0; }
+
+        int[]   scopeDepth = namedScopes.get( scopeName );
+        if ( scopeDepth == null )   { return 0; }
+        else { return scopeDepth[ SCOPE_CELL ]; }
+    }
+
+    /** Set whether we should skip adding USAGE privileges for user-defined types */
+    public  boolean    skipTypePrivileges( boolean skip )
+    {
+        boolean oldValue = skippingTypePrivileges;
+        
+        skippingTypePrivileges = skip;
+
+        return oldValue;
+    }
+
+    /** Return whether we are skipping USAGE privileges for user-defined types */
+    public  boolean skippingTypePrivileges()
+    {
+        return skippingTypePrivileges;
+    }
+
 	/*
-	** Context state must be reset in restContext()
+	** Context state must be reset in resetContext()
 	*/
 
 	private final Parser 		parser;
@@ -1006,15 +1078,18 @@ public class CompilerContextImpl extends ContextImpl
 	private int					scanIsolationLevel;
 	private int					nextEquivalenceClass = -1;
 	private long				nextClassName;
-	private List				savedObjects;
+	private List<Object>				savedObjects;
 	private String				classPrefix;
 	private SchemaDescriptor	compilationSchema;
+    private ArrayList<VisitableFilter> privilegeCheckFilters;
+    private HashMap<String,int[]> namedScopes;
+    private boolean             skippingTypePrivileges;
 
 	/**
 	 * Saved execution time default schema, if we need to change it
 	 * temporarily.
 	 */
-	private ArrayList        	defaultSchemaStack;
+	private ArrayList<SchemaDescriptor>        	defaultSchemaStack;
 
 	private ProviderList		currentAPL;
 	private boolean returnParameterFlag;
@@ -1023,11 +1098,11 @@ public class CompilerContextImpl extends ContextImpl
      * Map that contains all store cost controllers opened in this compiler
      * context. Conglomerate id (long) is the key.
      */
-    private final HashMap storeCostControllers = new HashMap();
+    private final HashMap<Long,StoreCostController> storeCostControllers = new HashMap<Long,StoreCostController>();
 
 	private SortCostController	sortCostController;
 
-	private List parameterList;
+    private List<ParameterNode> parameterList;
 
 	/* Type descriptors for the ? parameters */
 	private DataTypeDescriptor[]	parameterDescriptors;
@@ -1036,13 +1111,14 @@ public class CompilerContextImpl extends ContextImpl
 
 	private SQLWarning warnings;
 
-	private final ArrayList privTypeStack = new ArrayList();
+	private final ArrayList<Integer> privTypeStack = new ArrayList<Integer>();
 	private int currPrivType = Authorizer.NULL_PRIV;
-	private HashMap requiredColumnPrivileges;
-	private HashMap requiredTablePrivileges;
-	private HashMap requiredSchemaPrivileges;
-	private HashMap requiredRoutinePrivileges;
-	private HashMap requiredUsagePrivileges;
-	private HashMap requiredRolePrivileges;
-    private HashMap referencedSequences;
+	private HashMap<StatementTablePermission,StatementColumnPermission> requiredColumnPrivileges;
+	private HashMap<StatementTablePermission,StatementTablePermission> requiredTablePrivileges;
+	private HashMap<StatementSchemaPermission,StatementSchemaPermission> requiredSchemaPrivileges;
+	private HashMap<UUID,Integer> requiredRoutinePrivileges;
+	private HashMap<UUID,String> requiredUsagePrivileges;
+	private HashMap<StatementRolePermission,StatementRolePermission> requiredRolePrivileges;
+    private HashMap<UUID,SequenceDescriptor> referencedSequences;
+    
 } // end of class CompilerContextImpl
