@@ -21,36 +21,23 @@
 
 package	org.apache.derby.impl.sql.compile;
 
+import java.sql.Types;
+import java.util.BitSet;
+import java.util.Properties;
+import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.ClassName;
-
-import org.apache.derby.iapi.services.sanity.SanityManager;
 import org.apache.derby.iapi.services.classfile.VMOpcode;
 import org.apache.derby.iapi.services.compiler.MethodBuilder;
 import org.apache.derby.iapi.services.context.ContextManager;
-
-import org.apache.derby.iapi.error.StandardException;
-
-import org.apache.derby.iapi.sql.compile.NodeFactory;
+import org.apache.derby.shared.common.sanity.SanityManager;
+import org.apache.derby.iapi.sql.compile.CostEstimate;
 import org.apache.derby.iapi.sql.compile.Optimizable;
-import org.apache.derby.iapi.sql.compile.OptimizablePredicate;
 import org.apache.derby.iapi.sql.compile.OptimizablePredicateList;
 import org.apache.derby.iapi.sql.compile.Optimizer;
-import org.apache.derby.iapi.sql.compile.CostEstimate;
 import org.apache.derby.iapi.sql.compile.RowOrdering;
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
-
 import org.apache.derby.iapi.sql.dictionary.ConglomerateDescriptor;
-
-import org.apache.derby.iapi.reference.SQLState;
-
-import org.apache.derby.iapi.types.DataTypeDescriptor;
-
+import org.apache.derby.iapi.types.TypeId;
 import org.apache.derby.iapi.util.JBitSet;
-import org.apache.derby.iapi.util.ReuseFactory;
-
-import java.sql.Types;
-
-import java.util.BitSet;
 
 /**
  * A IntersectOrExceptNode represents an INTERSECT or EXCEPT DML statement.
@@ -89,25 +76,26 @@ public class IntersectOrExceptNode extends SetOperatorNode
     private boolean[] intermediateOrderByNullsLow; // TRUE means NULL values should be ordered lower than non-NULL values
 
 	/**
-	 * Initializer for a SetOperatorNode.
+     * Constructor for a SetOperatorNode.
 	 *
+     * @param opType            The operator type: one of {@link #EXCEPT_OP} or
+     *                          {@link #INTERSECT_OP}.
 	 * @param leftResult		The ResultSetNode on the left side of this union
 	 * @param rightResult		The ResultSetNode on the right side of this union
-	 * @param all				Whether or not this is an ALL.
+     * @param all               {@code true} if this is an ALL set operation.
 	 * @param tableProperties	Properties list associated with the table
-	 *
+     * @param cm                The context manager
 	 * @exception StandardException		Thrown on error
 	 */
+    IntersectOrExceptNode(int            opType,
+                          ResultSetNode  leftResult,
+                          ResultSetNode  rightResult,
+                          boolean        all,
+                          Properties     tableProperties,
+                          ContextManager cm) throws StandardException {
 
-	public void init( Object opType,
-                      Object leftResult,
-                      Object rightResult,
-                      Object all,
-                      Object tableProperties)
-        throws StandardException
-	{
-        super.init( leftResult, rightResult, all, tableProperties);
-        this.opType = ((Integer) opType).intValue();
+        super(leftResult, rightResult, all, tableProperties, cm);
+        this.opType = opType;
     }
 
     private int getOpType()
@@ -128,7 +116,8 @@ public class IntersectOrExceptNode extends SetOperatorNode
 	 * @exception StandardException		Thrown on error
 	 */
 
-	public ResultSetNode preprocess(int numTables,
+    @Override
+    ResultSetNode preprocess(int numTables,
 									GroupByList gbl,
 									FromList fromList)
 								throws StandardException
@@ -146,22 +135,27 @@ public class IntersectOrExceptNode extends SetOperatorNode
         intermediateOrderByColumns = new int[ getResultColumns().size()];
         intermediateOrderByDirection = new int[ intermediateOrderByColumns.length];
         intermediateOrderByNullsLow = new boolean[ intermediateOrderByColumns.length];
-        /* If there is an order by on the result of the intersect then use that because we know that doing so
-         * will avoid a sort.  If the output of the intersect/except is small relative to its inputs then in some
-         * cases it would be better to sort the inputs on a different sequence of columns, but it is hard to analyze
-         * the input query expressions to see if a sort can be avoided.
+
+        /* If there is an order by on the result of the intersect then use
+         * that because we know that doing so will avoid a sort.  If the
+         * output of the intersect/except is small relative to its inputs then
+         * in some cases it would be better to sort the inputs on a different
+         * sequence of columns, but it is hard to analyze the input query
+         * expressions to see if a sort can be avoided.
          */
-        if( orderByLists[0] != null)
+        final OrderByList obl = qec.getOrderByList(0);
+
+        if( obl != null)
         {
             BitSet colsOrdered = new BitSet( intermediateOrderByColumns.length);
-            int orderByListSize = orderByLists[0].size();
+            int orderByListSize = obl.size();
             int intermediateOrderByIdx = 0;
             for( int i = 0; i < orderByListSize; i++)
             {
                 if( colsOrdered.get(i))
                     continue;
                 OrderByColumn orderByColumn =
-                    orderByLists[0].getOrderByColumn(i);
+                    obl.getOrderByColumn(i);
                 intermediateOrderByDirection[intermediateOrderByIdx] = orderByColumn.isAscending() ? 1 : -1;
                 intermediateOrderByNullsLow[intermediateOrderByIdx] = orderByColumn.isNullsOrderedLow();
                 int columnIdx = orderByColumn.getResultColumn().getColumnPosition() - 1;
@@ -179,7 +173,7 @@ public class IntersectOrExceptNode extends SetOperatorNode
                     intermediateOrderByIdx++;
                 }
             }
-            orderByLists[0] = null; // It will be pushed down.
+            qec.setOrderByList(0, null); // It will be pushed down.
         }
         else // The output of the intersect/except does not have to be ordered
         {
@@ -202,29 +196,37 @@ public class IntersectOrExceptNode extends SetOperatorNode
         throws StandardException
     {
         ContextManager cm = getContextManager();
-        NodeFactory nf = getNodeFactory();
-        OrderByList orderByList = (OrderByList) nf.getNode( C_NodeTypes.ORDER_BY_LIST, cm);
+        OrderByList orderByList = new OrderByList(null, cm);
+
         for( int i = 0; i < intermediateOrderByColumns.length; i++)
         {
-            OrderByColumn orderByColumn = (OrderByColumn)
-              nf.getNode( C_NodeTypes.ORDER_BY_COLUMN,
-			  nf.getNode(C_NodeTypes.INT_CONSTANT_NODE,
-				     ReuseFactory.getInteger( intermediateOrderByColumns[i] + 1),
-				     cm),
-                          cm);
-            if( intermediateOrderByDirection[i] < 0)
+            OrderByColumn orderByColumn =
+                new OrderByColumn(
+                    new NumericConstantNode(
+                        TypeId.getBuiltInTypeId(Types.INTEGER),
+                        intermediateOrderByColumns[i] + 1,
+                        cm),
+                    cm);
+
+            if( intermediateOrderByDirection[i] < 0) {
                 orderByColumn.setDescending();
-            if( intermediateOrderByNullsLow[i])
+            }
+
+            if( intermediateOrderByNullsLow[i]) {
                 orderByColumn.setNullsOrderedLow();
+            }
+
             orderByList.addOrderByColumn( orderByColumn);
         }
         orderByList.bindOrderByColumns( rsn);
+        rsn.pushQueryExpressionSuffix();
         rsn.pushOrderByList( orderByList);
     } // end of pushOrderingDown
                                                             
     /**
      * @see org.apache.derby.iapi.sql.compile.Optimizable#estimateCost
      */
+    @Override
     public CostEstimate estimateCost( OptimizablePredicateList predList,
                                       ConglomerateDescriptor cd,
                                       CostEstimate outerCost,
@@ -244,17 +246,23 @@ public class IntersectOrExceptNode extends SetOperatorNode
 							(PredicateList) null,
 							outerCost);
 
-		CostEstimate costEstimate = getCostEstimate(optimizer);
+        CostEstimate costEst = getCostEstimate(optimizer);
         CostEstimate leftCostEstimate = leftResultSet.getCostEstimate();
         CostEstimate rightCostEstimate = rightResultSet.getCostEstimate();
-        // The cost is the sum of the two child costs plus the cost of sorting the union.
-        costEstimate.setCost( leftCostEstimate.getEstimatedCost() + rightCostEstimate.getEstimatedCost(),
-                              getRowCountEstimate( leftCostEstimate.rowCount(),
-                                                   rightCostEstimate.rowCount()),
-                              getSingleScanRowCountEstimate( leftCostEstimate.singleScanRowCount(),
-                                                             rightCostEstimate.singleScanRowCount()));
 
-        return costEstimate;
+        // The cost is the sum of the two child costs plus the cost of
+        // sorting the union.
+        costEst.setCost(
+            leftCostEstimate.getEstimatedCost() +
+                rightCostEstimate.getEstimatedCost(),
+            getRowCountEstimate(
+                leftCostEstimate.rowCount(),
+                rightCostEstimate.rowCount()),
+            getSingleScanRowCountEstimate(
+                leftCostEstimate.singleScanRowCount(),
+                rightCostEstimate.singleScanRowCount()));
+
+        return costEst;
     } // End of estimateCost
 
 	/**
@@ -262,6 +270,7 @@ public class IntersectOrExceptNode extends SetOperatorNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	public Optimizable modifyAccessPath(JBitSet outerTables) throws StandardException
 	{
 		Optimizable retOptimizable;
@@ -280,7 +289,8 @@ public class IntersectOrExceptNode extends SetOperatorNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ResultSetNode modifyAccessPaths() throws StandardException
+    @Override
+    ResultSetNode modifyAccessPaths() throws StandardException
 	{
 		ResultSetNode retRSN;
 		retRSN = super.modifyAccessPaths();
@@ -315,30 +325,37 @@ public class IntersectOrExceptNode extends SetOperatorNode
 
         ResultSetNode treeTop = this;
 
-        if( orderByLists[0] != null) {
-            // Generate an order by node on top of the intersect/except
-            treeTop = (ResultSetNode) getNodeFactory().getNode(
-                C_NodeTypes.ORDER_BY_NODE,
-                treeTop,
-                orderByLists[0],
-                tableProperties,
-                getContextManager());
+        for (int i = 0; i < qec.size(); i++) {
+            final OrderByList obl = qec.getOrderByList(i);
+
+            if(obl != null) {
+                // Generate an order by node on top of the intersect/except
+                treeTop = new OrderByNode(
+                        treeTop,
+                        obl,
+                        tableProperties,
+                        getContextManager());
+            }
+
+            final ValueNode offset = qec.getOffset(i);
+            final ValueNode fetchFirst = qec.getFetchFirst(i);
+
+            if (offset != null || fetchFirst != null) {
+                ResultColumnList newRcl =
+                    treeTop.getResultColumns().copyListAndObjects();
+                newRcl.genVirtualColumnNodes(
+                    treeTop, treeTop.getResultColumns());
+
+                treeTop = new RowCountNode(
+                        treeTop,
+                        newRcl,
+                        offset,
+                        fetchFirst,
+                        qec.getHasJDBCLimitClause()[i].booleanValue(),
+                        getContextManager());
+            }
         }
 
-        if (offset != null || fetchFirst != null) {
-            ResultColumnList newRcl =
-                treeTop.getResultColumns().copyListAndObjects();
-            newRcl.genVirtualColumnNodes(treeTop, treeTop.getResultColumns());
-
-            treeTop = (ResultSetNode)getNodeFactory().getNode(
-                C_NodeTypes.ROW_COUNT_NODE,
-                treeTop,
-                newRcl,
-                offset,
-                fetchFirst,
-                Boolean.valueOf( hasJDBClimitClause ),
-                getContextManager());
-        }
         return treeTop;
 
     } // end of addNewNodes
@@ -348,8 +365,8 @@ public class IntersectOrExceptNode extends SetOperatorNode
 	 *
 	 * @exception StandardException		Thrown on error
      */
-	public void generate( ActivationClassBuilder acb,
-                          MethodBuilder mb)
+    @Override
+    void generate( ActivationClassBuilder acb, MethodBuilder mb)
         throws StandardException
 	{
 
@@ -359,7 +376,7 @@ public class IntersectOrExceptNode extends SetOperatorNode
 		assignResultSetNumber();
 
 		// Get our final cost estimate based on the child estimates.
-		costEstimate = getFinalCostEstimate();
+		setCostEstimate( getFinalCostEstimate() );
 
 		// build up the tree.
 
@@ -383,9 +400,9 @@ public class IntersectOrExceptNode extends SetOperatorNode
 		getRightResultSet().generate( acb, mb);
 
 		acb.pushThisAsActivation(mb);
-		mb.push(resultSetNumber);
-        mb.push( costEstimate.getEstimatedRowCount());
-        mb.push( costEstimate.getEstimatedCost());
+		mb.push(getResultSetNumber());
+        mb.push( getCostEstimate().getEstimatedRowCount());
+        mb.push( getCostEstimate().getEstimatedCost());
         mb.push( getOpType());
         mb.push( all);
         mb.push( getCompilerContext().addSavedObject( intermediateOrderByColumns));
@@ -408,23 +425,26 @@ public class IntersectOrExceptNode extends SetOperatorNode
 	 *  rows depends on whether this is an INTERSECT or EXCEPT (see
 	 *  getRowCountEstimate() in this class for more).
 	 */
-	public CostEstimate getFinalCostEstimate()
+    @Override
+    CostEstimate getFinalCostEstimate()
 		throws StandardException
 	{
-		if (finalCostEstimate != null)
-			return finalCostEstimate;
+		if (getCandidateFinalCostEstimate() != null)
+        {
+			return getCandidateFinalCostEstimate();
+        }
 
 		CostEstimate leftCE = leftResultSet.getFinalCostEstimate();
 		CostEstimate rightCE = rightResultSet.getFinalCostEstimate();
 
-		finalCostEstimate = getNewCostEstimate();
-		finalCostEstimate.setCost(
+		setCandidateFinalCostEstimate( getNewCostEstimate() );
+		getCandidateFinalCostEstimate().setCost(
 			leftCE.getEstimatedCost() + rightCE.getEstimatedCost(),
 			getRowCountEstimate(leftCE.rowCount(), rightCE.rowCount()),
 			getSingleScanRowCountEstimate(leftCE.singleScanRowCount(),
 				rightCE.singleScanRowCount()));
 
-		return finalCostEstimate;
+		return getCandidateFinalCostEstimate();
 	}
 
     String getOperatorName()
