@@ -21,41 +21,46 @@
 
 package org.apache.derby.impl.sql.execute;
 
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.sql.Connection;
 import java.sql.SQLException;
 import java.sql.SQLWarning;
 import java.util.ArrayList;
+import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.Hashtable;
+import java.util.Stack;
 import java.util.Vector;
-
-import	org.apache.derby.catalog.Dependable;
-import	org.apache.derby.catalog.DependableFinder;
+import org.apache.derby.catalog.Dependable;
+import org.apache.derby.catalog.DependableFinder;
 import org.apache.derby.catalog.UUID;
-import org.apache.derby.iapi.services.uuid.UUIDFactory;
-import org.apache.derby.iapi.services.monitor.Monitor;
-import org.apache.derby.iapi.sql.depend.Provider;
-import org.apache.derby.iapi.sql.Row;
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.jdbc.ConnectionContext;
 import org.apache.derby.iapi.reference.Property;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.context.Context;
 import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.iapi.services.io.ArrayUtil;
 import org.apache.derby.iapi.services.io.FormatableBitSet;
 import org.apache.derby.iapi.services.loader.GeneratedByteCode;
 import org.apache.derby.iapi.services.loader.GeneratedClass;
 import org.apache.derby.iapi.services.loader.GeneratedMethod;
+import org.apache.derby.iapi.services.monitor.ModuleFactory;
+import org.apache.derby.iapi.services.monitor.Monitor;
 import org.apache.derby.iapi.services.property.PropertyUtil;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.iapi.services.uuid.UUIDFactory;
 import org.apache.derby.iapi.sql.Activation;
 import org.apache.derby.iapi.sql.ParameterValueSet;
+import org.apache.derby.iapi.sql.PreparedStatement;
 import org.apache.derby.iapi.sql.ResultDescription;
 import org.apache.derby.iapi.sql.ResultSet;
+import org.apache.derby.iapi.sql.Row;
 import org.apache.derby.iapi.sql.compile.Optimizer;
 import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
 import org.apache.derby.iapi.sql.conn.SQLSessionContext;
 import org.apache.derby.iapi.sql.depend.DependencyManager;
+import org.apache.derby.iapi.sql.depend.Provider;
 import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
 import org.apache.derby.iapi.sql.execute.ConstantAction;
 import org.apache.derby.iapi.sql.execute.CursorActivation;
@@ -76,7 +81,7 @@ import org.apache.derby.iapi.types.DataValueFactory;
 import org.apache.derby.iapi.types.NumberDataValue;
 import org.apache.derby.iapi.types.RowLocation;
 import org.apache.derby.iapi.types.StringDataValue;
-import org.apache.derby.iapi.util.ReuseFactory;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 /**
  * BaseActivation
@@ -110,7 +115,7 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 	private GeneratedClass gc;	// my Generated class object.
 
 	private boolean checkRowCounts;
-	private HashSet rowCountsCheckedThisExecution = new HashSet(4, 0.9f);
+	private HashSet<Integer> rowCountsCheckedThisExecution = new HashSet<Integer>(4, 0.9f);
 
 	private static final long MAX_SQRT = (long) Math.sqrt(Long.MAX_VALUE);
 
@@ -179,16 +184,16 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 	 *
 	 * A non-null 'parentActivation' represents the activation of the calling
 	 * statement (if we are in a nested connection of a stored routine), or the
-	 * activation of the parent statement (if we are executing a substatement)
+     * activation of the parent statement (if we are executing a sub-statement)
 	 *
 	 * 'parentActivation' is set when this activation is created (@see
-	 * GenericPreparedStatement#getActivation) based on the top of the
+     * PreparedStatement#getActivation) based on the top of the
 	 * dynamic call stack of execution, which is tracked by
 	 * StatementContext. The nested SQL session context is initialized
 	 * by code generated for the call, after parameters are evaluated
-	 * or just substatement execution starts.
-	 * @see org.apache.derby.impl.sql.compile.StaticMethodCallNode#generateSetupNestedSessionContext
-	 * @see org.apache.derby.impl.sql.GenericPreparedStatement#executeSubStatement
+     * or just sub-statement execution starts.
+     * @see org.apache.derby.impl.sql.compile.StaticMethodCallNode#generatePushNestedSessionContext
+     * @see PreparedStatement#executeSubStatement
 	 *
 	 */
 	private Activation parentActivation;
@@ -200,6 +205,11 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 	 */
 	private SQLSessionContext sqlSessionContextForChildren;
 
+    /**
+     * Stack of ConstantActions.
+     */
+    private Stack<ConstantAction>   constantActionStack = new Stack<ConstantAction>();
+
 	//Following is the position of the session table names list in savedObjects in compiler context
 	//This is updated to be the correct value at cursor generate time if the cursor references any session table names.
 	//If the cursor does not reference any session table names, this will stay negative
@@ -246,7 +256,7 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 
 		/* Get the UUID for this activation */
 		UUIDFactory uuidFactory =
-			Monitor.getMonitor().getUUIDFactory();
+			getMonitor().getUUIDFactory();
 
 		UUIDValue = uuidFactory.createUUID();
 		UUIDString = UUIDValue.toString();
@@ -293,6 +303,7 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
     /**
      * Create the ResultSet tree for this statement.
      * @return the root of the ResultSet tree for this statement
+     * @throws StandardException standard error policy
      */
     protected abstract ResultSet createResultSet() throws StandardException;
 
@@ -301,6 +312,7 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
      * execution of the statement. The default implementation does nothing.
      * Sub-classes should override this method if they need to perform
      * operations before each execution.
+     * @throws org.apache.derby.iapi.error.StandardException
      */
     protected void reinit() throws StandardException {
         // Do nothing by default. Overridden by sub-classes that need it.
@@ -310,8 +322,23 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 		return preStmt;
 	}
 
-	public ConstantAction getConstantAction() {
-		return preStmt.getConstantAction();
+    public  ConstantAction    pushConstantAction( ConstantAction newConstantAction )
+    {
+        return constantActionStack.push( newConstantAction );
+    }
+
+    public  ConstantAction    popConstantAction()
+    {
+        return constantActionStack.pop();
+    }
+
+	public ConstantAction getConstantAction()
+    {
+        if ( constantActionStack.size() > 0 )
+        {
+            return constantActionStack.peek();
+        }
+        else { return preStmt.getConstantAction(); }
 	}
 
 
@@ -326,11 +353,13 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 	}
 
 	/**
-		Link this activation with its PreparedStatement.
-		It can be called with null to break the link with the
-		PreparedStatement.
-
-	*/
+     * Link this activation with its PreparedStatement.
+     * It can be called with null to break the link with the
+     * PreparedStatement.
+     * @param ps prepared statement
+     * @param scrollable activation for a scrollable result set
+     * @throws StandardException standard error policy
+     */
 	public final void setupActivation(ExecPreparedStatement ps, boolean scrollable) 
 	throws StandardException {
 		preStmt = ps;
@@ -462,8 +491,6 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 
 			closed = true;
 
-			LanguageConnectionContext lcc = getLanguageConnectionContext();
-
             // Remove all the dependencies this activation has. It won't need
             // them after it's closed, so let's free up the memory in the
             // dependency manager. (DERBY-4571)
@@ -488,9 +515,10 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 	}
 
 	/**
-		A generated class can create its own closeActivationAction
-		method to invoke special logic when the activation is closed.
-	*/
+     * A generated class can create its own closeActivationAction
+     * method to invoke special logic when the activation is closed.
+     * @throws java.lang.Exception error
+     */
 	protected void closeActivationAction() throws Exception {
 		// no code to be added here as generated code
 		// will not call super.closeActivationAction()
@@ -643,8 +671,8 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 	public void setAutoGeneratedKeysResultsetInfo(int[] columnIndexes, String[] columnNames)
 	{
 		autoGeneratedKeysResultSetMode = true;
-		autoGeneratedKeysColumnIndexes = columnIndexes;
-		autoGeneratedKeysColumnNames = columnNames;
+		autoGeneratedKeysColumnIndexes = ArrayUtil.copy( columnIndexes );
+		autoGeneratedKeysColumnNames = ArrayUtil.copy( columnNames );
 	}
 
 	/** @see Activation#getAutoGeneratedKeysResultsetMode */
@@ -656,13 +684,13 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 	/** @see Activation#getAutoGeneratedKeysColumnIndexes */
 	public int[] getAutoGeneratedKeysColumnIndexes()
 	{
-		return autoGeneratedKeysColumnIndexes;
+		return ArrayUtil.copy( autoGeneratedKeysColumnIndexes );
 	}
 
 	/** @see Activation#getAutoGeneratedKeysColumnNames */
 	public String[] getAutoGeneratedKeysColumnNames()
 	{
-		return autoGeneratedKeysColumnNames;
+		return ArrayUtil.copy( autoGeneratedKeysColumnNames );
 	}
 
 	//
@@ -713,7 +741,7 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 	       throws StandardException
 	{
 		DataValueDescriptor l =
-			((InsertResultSet)resultSet).getSetAutoincrementValue(columnPosition, increment);
+			((DMLWriteGeneratedColumnsResultSet)resultSet).getSetAutoincrementValue(columnPosition, increment);
 		return l;
 
 	}
@@ -728,7 +756,7 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
      *
 	 * @return The next number in the sequence
 	 */
-	protected NumberDataValue getCurrentValueAndAdvance
+	public NumberDataValue getCurrentValueAndAdvance
         ( String sequenceUUIDstring, int typeFormatID )
 	       throws StandardException
 	{
@@ -933,10 +961,9 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 			boolean significantChange = false;
 
 			int resultSetNumber = resultSet.resultSetNumber();
-			Integer rsn = ReuseFactory.getInteger(resultSetNumber);
 
 			/* Check each result set only once per execution */
-			if (rowCountsCheckedThisExecution.add(rsn))
+			if (rowCountsCheckedThisExecution.add(resultSetNumber))
 			{
                 long n1 = getPreparedStatement()
                         .getInitialRowCount(resultSetNumber, currentRowCount);
@@ -1391,13 +1418,6 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 	 * @see org.apache.derby.iapi.sql.Activation#getSQLSessionContextForChildren
 	 */
 	public SQLSessionContext getSQLSessionContextForChildren() {
-
-		if (SanityManager.DEBUG) {
-			SanityManager.ASSERT
-				(sqlSessionContextForChildren != null,
-				 "Expected sqlSessionContextForChildren to be non-null");
-		}
-
 		return sqlSessionContextForChildren;
 	}
 
@@ -1490,8 +1510,6 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 		if (ps == null)
 			return;
 			
-		LanguageConnectionContext lcc = getLanguageConnectionContext();
-
 		CursorActivation cursorActivation = lcc.lookupCursorActivation(cursorName);
 
 		if (cursorActivation != null)
@@ -1529,11 +1547,12 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 	 *	@param	rs	input result set
 	 *	@return	materialized resultset, or original rs if it can't be materialized
 	 */
+    @SuppressWarnings("UseOfObsoleteCollectionType")
 	public NoPutResultSet materializeResultSetIfPossible(NoPutResultSet rs)
 		throws StandardException
 	{
 		rs.openCore();
-		Vector rowCache = new Vector();
+		Vector<ExecRow> rowCache = new Vector<ExecRow>();
 		ExecRow aRow;
 		int cacheSize = 0;
 		FormatableBitSet toClone = null;
@@ -1579,7 +1598,7 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 			{
 				rrs[i] = new RowResultSet(
 										this,
-										(ExecRow) rowCache.elementAt(i),
+                                        rowCache.elementAt(i),
 										true,
 										rsNum,
 										1,
@@ -1614,16 +1633,20 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 
 	// maintain hash table of parent result set vector
 	// a table can have more than one parent source.
-	protected Hashtable parentResultSets;
-	public void setParentResultSet(TemporaryRowHolder rs, String resultSetId)
+    @SuppressWarnings("UseOfObsoleteCollectionType")
+	protected Hashtable<String,Vector<TemporaryRowHolder>> parentResultSets;
+
+    @SuppressWarnings("UseOfObsoleteCollectionType")
+    public void setParentResultSet(TemporaryRowHolder rs, String resultSetId)
 	{
-		Vector  rsVector;
+		Vector<TemporaryRowHolder>  rsVector;
 		if(parentResultSets == null)
-			parentResultSets = new Hashtable();
-		rsVector = (Vector) parentResultSets.get(resultSetId);
+			parentResultSets = new Hashtable<String,Vector<TemporaryRowHolder>>();
+        rsVector = parentResultSets.get(resultSetId);
+
 		if(rsVector == null)
 		{
-			rsVector = new Vector();
+			rsVector = new Vector<TemporaryRowHolder>();
 			rsVector.addElement(rs);
 		}else
 		{
@@ -1636,14 +1659,15 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 	 * get the reference to parent table ResultSets, that will be needed by the 
 	 * referential action dependent table scans.
 	 */
-	public Vector getParentResultSet(String resultSetId)
+    @SuppressWarnings("UseOfObsoleteCollectionType")
+	public Vector<TemporaryRowHolder> getParentResultSet(String resultSetId)
 	{
-		return (Vector) parentResultSets.get(resultSetId);
+        return parentResultSets.get(resultSetId);
 	}
 
-	public Hashtable getParentResultSets()
+	public Enumeration<String> getParentResultSetKeys()
 	{
-		return parentResultSets;
+		return parentResultSets.keys();
 	}
 
 	/**
@@ -1878,4 +1902,23 @@ public abstract class BaseActivation implements CursorActivation, GeneratedByteC
 							 LanguageConnectionContext lcc)
 			throws StandardException {
 	}
+    
+    /**
+     * Privileged Monitor lookup. Must be package private so that user code
+     * can't call this entry point.
+     */
+    static  ModuleFactory  getMonitor()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ModuleFactory>()
+             {
+                 public ModuleFactory run()
+                 {
+                     return Monitor.getMonitor();
+                 }
+             }
+             );
+    }
+
 }
