@@ -23,6 +23,7 @@ package org.apache.derby.impl.sql.execute;
 
 import org.apache.derby.iapi.error.StandardException;
 
+import org.apache.derby.iapi.services.context.Context;
 import org.apache.derby.iapi.services.context.ContextService;
 
 import org.apache.derby.iapi.sql.conn.Authorizer;
@@ -52,6 +53,8 @@ import org.apache.derby.impl.sql.compile.TableName;
 import java.util.List;
 import java.util.Properties;
 
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.sql.Timestamp;
 
 /**
@@ -85,27 +88,14 @@ public class GenericConstantActionFactory
 	/**
 	 * Get ConstantAction for SET CONSTRAINTS statement.
 	 *
-	 *  @param cdl			the constraints to set, if null,
-	 *						we'll go ahead and set them all
-	 *  @param enable		if true, turn them on, if false
-	 *						disable them
-	 *  @param unconditionallyEnforce	Replication sets this to true at
-	 *									the end of REFRESH. This forces us
-	 *									to run the included foreign key constraints even
-	 *									if they're already marked ENABLED.
-	 *	@param ddlList		Replication list of actions to propagate,
-	 *						null unless a replication source
+     *  @param constraints  The constraints to set, if null,
+     *                      set them ALL.
+     *  @param initiallyDeferred   ncodes IMMEDIATE (false), DEFERRED (true)
 	 */
-	public	ConstantAction getSetConstraintsConstantAction
-	(
-		ConstraintDescriptorList	cdl,
-		boolean						enable,
-		boolean						unconditionallyEnforce,
-		Object[]					ddlList
-    )
-	{
-		// ignore rep arg
-		return new SetConstraintsConstantAction(cdl, enable, unconditionallyEnforce);
+    public  ConstantAction getSetConstraintsConstantAction(
+            List<TableName> constraints,
+            boolean         initiallyDeferred) {
+        return new SetConstraintsConstantAction(constraints, initiallyDeferred);
 	}
 
 
@@ -183,6 +173,9 @@ public class GenericConstantActionFactory
 	 *
 	 *  @param constraintName	Constraint name.
 	 *  @param constraintType	Constraint type.
+     *  @param constraintCharacteristics
+     *                          Constraint characteristics, see {@link
+     *  org.apache.derby.impl.sql.compile.ConstraintDefinitionNode#characteristics}
      *  @param forCreateTable   True if for a CREATE TABLE
 	 *  @param tableName		Table name.
 	 *	@param tableId			UUID of table.
@@ -190,8 +183,6 @@ public class GenericConstantActionFactory
 	 *  @param columnNames		String[] for column names
 	 *  @param indexAction		IndexConstantAction for constraint (if necessary)
 	 *  @param constraintText	Text for check constraint
-	 *	@param enabled			Should the constraint be created as enabled 
-	 *							(enabled == true), or disabled (enabled == false).
 	 *	@param otherConstraint	The referenced constraint, if a foreign key constraint
 	 *  @param providerInfo Information on all the Providers
 	 */
@@ -199,6 +190,7 @@ public class GenericConstantActionFactory
 	(
 		String				constraintName,
 		int					constraintType,
+        boolean[]           constraintCharacteristics,
         boolean             forCreateTable,
 		String				tableName,
 		UUID				tableId,
@@ -206,15 +198,23 @@ public class GenericConstantActionFactory
 		String[]			columnNames,
 		IndexConstantAction indexAction,
 		String				constraintText,
-		boolean				enabled,
 		ConstraintInfo		otherConstraint,
 		ProviderInfo[]		providerInfo
 	)
 	{
-		return new CreateConstraintConstantAction
-			( constraintName, constraintType, forCreateTable, tableName, 
-			  tableId, schemaName, columnNames, indexAction, constraintText, 
-			  enabled, otherConstraint, providerInfo );
+        return new CreateConstraintConstantAction(
+                constraintName,
+                constraintType,
+                constraintCharacteristics,
+                forCreateTable,
+                tableName,
+                tableId,
+                schemaName,
+                columnNames,
+                indexAction,
+                constraintText,
+                otherConstraint,
+                providerInfo );
 	}
 
 
@@ -229,6 +229,11 @@ public class GenericConstantActionFactory
      *                                  column in the key has a null value,
      *                                  no checking is done and insert will
      *                                  always succeed.
+     * @param hasDeferrableChecking True if the index is used to back a
+     *                              deferrable constraint
+     * @param initiallyDeferred  True means the deferrable constraint has
+     *                           deferred mode initially.
+     * @param constraintType  The constraint type
      * @param indexType	The type of index (BTREE, for example)
      * @param schemaName			the schema that table (and index) lives in.
      * @param indexName	Name of the index
@@ -240,11 +245,14 @@ public class GenericConstantActionFactory
      * @param conglomerateUUID	ID of conglomerate
      * @param properties	The optional properties list associated with the index.
      */
-	public	CreateIndexConstantAction	getCreateIndexConstantAction
+    public IndexConstantAction getCreateIndexConstantAction
 	(
         boolean forCreateTable,
 		boolean			unique,
 		boolean			uniqueWithDuplicateNulls,
+        boolean         hasDeferrableChecking,
+        boolean         initiallyDeferred,
+        int             constraintType,
 		String			indexType,
 		String			schemaName,
 		String			indexName,
@@ -257,11 +265,23 @@ public class GenericConstantActionFactory
 		Properties		properties
     )
 	{
-		return	new CreateIndexConstantAction
-			( forCreateTable, unique, uniqueWithDuplicateNulls, indexType, 
-				schemaName, indexName, tableName, tableId,
-			  columnNames, isAscending, isConstraint,
-			  conglomerateUUID, properties );
+        return new CreateIndexConstantAction(
+            forCreateTable,
+            unique,
+            uniqueWithDuplicateNulls,
+            hasDeferrableChecking,
+            initiallyDeferred,
+            constraintType,
+            indexType,
+            schemaName,
+            indexName,
+            tableName,
+            tableId,
+            columnNames,
+            isAscending,
+            isConstraint,
+            conglomerateUUID,
+            properties);
 	}
 
 
@@ -464,9 +484,10 @@ public class GenericConstantActionFactory
 	 *  @param numColumns			Number of columns to read
 	 *  @param dependencyId			UUID for dependency system
 	 *  @param baseRowReadList      Map of columns read in.  1 based.
-	 *	@param baseRowReadMap		BaseRowReadMap[heapColId]->ReadRowColumnId.
+	 *	@param baseRowReadMap		BaseRowReadMap[heapColId]-&gt;ReadRowColumnId.
      *  @param streamStorableHeapColIds Null for non rep. (0 based)
 	 *  @param singleRowSource		Whether or not source is a single row source
+	 *  @param underMerge   True if this is an action of a MERGE statement.
 	 *
 	 *  @exception StandardException		Thrown on failure
 	 */
@@ -497,7 +518,8 @@ public class GenericConstantActionFactory
 								int					numColumns,
 								UUID				dependencyId,
 								boolean				singleRowSource,
-								ConstantAction[]	dependentConstantActions
+								ConstantAction[]	dependentConstantActions,
+								boolean				underMerge
 	)
 			throws StandardException
 	{
@@ -519,7 +541,8 @@ public class GenericConstantActionFactory
 										numColumns,
 										singleRowSource,
 										resultDescription,
-										dependentConstantActions
+										dependentConstantActions,
+										underMerge
 										);
 	}
 
@@ -554,6 +577,39 @@ public class GenericConstantActionFactory
 
 
 	/**
+     *  Make ConstantAction to drop a constraint.
+     *
+     *  @param constraintName   Constraint name.
+     *  @param constraintSchemaName     Constraint Schema Name
+     *  @param characteristics  The presumably altered characteristics
+     *  @param tableName        Table name.
+     *  @param tableId          UUID of table.
+     *  @param tableSchemaName  The schema that table lives in.
+     *  @param indexAction      IndexConstantAction for constraint (if necessary)
+     */
+    public  ConstraintConstantAction    getAlterConstraintConstantAction
+    (
+        String                  constraintName,
+        String                  constraintSchemaName,
+        boolean[]               characteristics,
+        String                  tableName,
+        UUID                    tableId,
+        String                  tableSchemaName,
+        IndexConstantAction     indexAction
+    )
+    {
+        return  new AlterConstraintConstantAction(
+                constraintName,
+                constraintSchemaName,
+                characteristics,
+                tableName,
+                tableId,
+                tableSchemaName,
+                indexAction);
+    }
+
+
+    /**
 	 *	Make the ConstantAction for a DROP INDEX statement.
 	 *
 	 *
@@ -565,7 +621,7 @@ public class GenericConstantActionFactory
 	 *  @param  tableConglomerateId	heap conglomerate ID for table
 	 *
 	 */
-	public	DropIndexConstantAction	getDropIndexConstantAction
+    public IndexConstantAction getDropIndexConstantAction
 	(
 		String				fullIndexName,
 		String				indexName,
@@ -704,7 +760,7 @@ public class GenericConstantActionFactory
 	}
 
 	/**
-	 *	Make the ConstantAction for a Replicated INSERT statement.
+     *  Make the ConstantAction for a INSERT statement.
 	 *
 	 *  @param conglomId		Conglomerate ID.
 	 *  @param heapSCOCI		StaticCompiledOpenConglomInfo for target heap.
@@ -716,7 +772,10 @@ public class GenericConstantActionFactory
 	 *	@param deferred			True means deferred insert
 	 *  @param tableIsPublished	true if table is published, false otherwise
 	 *  @param tableID			table id
-	 *  @param targetProperties	Properties on the target table
+     *  @param hasDeferrableChecks
+     *                          The target table has deferrable CHECK
+     *                          constraints
+     *  @param targetProperties Properties on the target table
 	 *	@param fkInfo			Array of structures containing foreign key info, 
 	 *							if any (may be null)
 	 *	@param triggerInfo		Array of structures containing trigger info, 
@@ -729,6 +788,8 @@ public class GenericConstantActionFactory
 	 *  @param singleRowSource	Whether or not source is a single row source
 	 *  @param autoincRowLocation array of row locations into syscolumns for
 	                              autoincrement columns
+	 *  @param underMerge   True if this is an INSERT action of a MERGE statement.
+	 *  @param identitySequenceUUIDString   For 10.11 and higher, the handle on the sequence for the identity column
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
@@ -741,7 +802,8 @@ public class GenericConstantActionFactory
 								StaticCompiledOpenConglomInfo[] indexSCOCIs,
 								String[]			indexNames,
 								boolean				deferred,
-								boolean				tableIsPublished,
+                                boolean             tableIsPublished,
+                                boolean             hasDeferrableChecks,
 								UUID				tableID,
 								int					lockMode,
 								Object         		insertToken,
@@ -755,7 +817,9 @@ public class GenericConstantActionFactory
 								Object[]			stageControl,
 								Object[]			ddlList,
 								boolean				singleRowSource,
-								RowLocation[]		autoincRowLocation
+								RowLocation[]		autoincRowLocation,
+								boolean		underMerge,
+								String		identitySequenceUUIDString
 							)
 			throws StandardException
 	{
@@ -767,6 +831,7 @@ public class GenericConstantActionFactory
 										indexSCOCIs,
 										indexNames,
 										deferred,
+                                        hasDeferrableChecks,
 										targetProperties,
 										tableID,
 										lockMode,
@@ -775,14 +840,18 @@ public class GenericConstantActionFactory
 										streamStorableHeapColIds,
 										indexedCols,
 										singleRowSource,
-										autoincRowLocation
+										autoincRowLocation,
+										underMerge,
+										identitySequenceUUIDString
 										);
 	}
 
 	/**
 	 *	Make the ConstantAction for an updatable VTI statement.
 	 *
-	 * @param deferred					Deferred mode?
+     * @param statementType             Statement type, cf.
+     * {@link org.apache.derby.vti.DeferModification#INSERT_STATEMENT} etc.
+     * @param deferred                  Deferred processing mode?
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
@@ -795,7 +864,9 @@ public class GenericConstantActionFactory
 	/**
 	 *	Make the ConstantAction for an updatable VTI statement.
 	 *
-	 * @param deferred					Deferred mode?
+     * @param statementType    Statement type, cf.
+     * {@link org.apache.derby.vti.DeferModification#INSERT_STATEMENT} etc.
+     * @param deferred         Deferred processing mode?
      * @param changedColumnIds Array of ids of changed columns
 	 *
 	 * @exception StandardException		Thrown on failure
@@ -847,14 +918,14 @@ public class GenericConstantActionFactory
 
 
 	/**
-	 *	Make the ConstantAction for a Replicated DELETE statement.
+     *  Make the ConstantAction for an UPDATE statement.
 	 *
-	 *  @param conglomId			Conglomerate ID.
-	 *  @param tableType			type of this table
+     *  @param targetTableDesc      Descriptor for the updated table
 	 *	@param heapSCOCI			StaticCompiledOpenConglomInfo for heap.
 	 *  @param irgs					Index descriptors
 	 *  @param indexCIDS			Conglomerate IDs of indices
-	 *	@param indexSCOCIs	StaticCompiledOpenConglomInfos for indexes.
+     *  @param indexSCOCIs          StaticCompiledOpenConglomInfos for indexes.
+     *  @param indexNames
 	 *	@param deferred				True means deferred update
 	 *	@param targetUUID			UUID of target table
 	 *	@param lockMode				The lock mode to use
@@ -862,6 +933,7 @@ public class GenericConstantActionFactory
 	 *  @param tableIsPublished		true if table is published, false otherwise
 	 *	@param changedColumnIds		Array of ids of changes columns
 	 *  @param keyPositions     	positions of primary key columns in base row
+     *  @param updateToken
 	 *	@param fkInfo				Array of structures containing foreign key info, 
 	 *								if any (may be null)
 	 *	@param triggerInfo			Array of structures containing trigger info, 
@@ -872,12 +944,16 @@ public class GenericConstantActionFactory
 	 *  @param numColumns			The number of columns being read.
 	 *	@param positionedUpdate		is this a positioned update
 	 *  @param singleRowSource		Whether or not source is a single row source
+	 *  @param autoincRowLocation array of row locations into syscolumns for
+	                              autoincrement columns
+	 *  @param underMerge   True if this is an action of a MERGE statement.
+	 *  @param identitySequenceUUIDString   For 10.11 and higher, the handle on the sequence for the identity column
+     *  @return                     The constant action constructed
 	 *
 	 *  @exception StandardException Thrown on failure
 	 */
 	public	UpdateConstantAction	getUpdateConstantAction(
-								long				conglomId,
-								int					tableType,
+                                TableDescriptor     targetTableDesc,
 								StaticCompiledOpenConglomInfo heapSCOCI,
 								IndexRowGenerator[]	irgs,
 								long[]				indexCIDS,
@@ -892,17 +968,20 @@ public class GenericConstantActionFactory
 								Object         		updateToken,
 								FKInfo[]			fkInfo,
 								TriggerInfo			triggerInfo,
-								FormatableBitSet				baseRowReadList,
+                                FormatableBitSet    baseRowReadList,
 								int[]				baseRowReadMap,
 								int[]				streamStorableHeapColIds,
 								int					numColumns,
 								boolean				positionedUpdate,
-								boolean				singleRowSource
+								boolean				singleRowSource,
+								RowLocation[]		autoincRowLocation,
+								boolean				underMerge,
+								String		identitySequenceUUIDString
 							)
 			throws StandardException
 	{
 		return new UpdateConstantAction(
-										conglomId,
+                                        targetTableDesc,
 										heapSCOCI,
 										irgs,
 										indexCIDS,
@@ -919,14 +998,17 @@ public class GenericConstantActionFactory
 										streamStorableHeapColIds,
 										numColumns,
 										positionedUpdate,
-										singleRowSource
+										singleRowSource,
+										autoincRowLocation,
+										underMerge,
+										identitySequenceUUIDString
 										);
 	}
 
 	static protected Authorizer getAuthorizer()
 	{
 		LanguageConnectionContext lcc = (LanguageConnectionContext)
-			ContextService.getContext(LanguageConnectionContext.CONTEXT_ID);
+			getContext(LanguageConnectionContext.CONTEXT_ID);
 		return lcc.getAuthorizer();
 	}
 
@@ -951,11 +1033,13 @@ public class GenericConstantActionFactory
 	 * @param referencedColsInTriggerAction	what columns does the trigger 
 	 *						action reference through old/new transition variables
 	 *						(may be null)
+     * @param originalWhenText The original user text of the WHEN clause (may be null)
 	 * @param originalActionText The original user text of the trigger action
 	 * @param referencingOld whether or not OLD appears in REFERENCING clause
 	 * @param referencingNew whether or not NEW appears in REFERENCING clause
 	 * @param oldReferencingName old referencing table name, if any, that appears in REFERCING clause
 	 * @param newReferencingName new referencing table name, if any, that appears in REFERCING clause
+     * @param providerInfo array of providers that the trigger depends on
 	 */
 	public ConstantAction getCreateTriggerConstantAction
 	(
@@ -973,18 +1057,22 @@ public class GenericConstantActionFactory
 		UUID				spsCompSchemaId,
 		int[]				referencedCols,
 		int[]				referencedColsInTriggerAction,
+        String              originalWhenText,
 		String				originalActionText,
 		boolean				referencingOld,
 		boolean				referencingNew,
 		String				oldReferencingName,
-		String				newReferencingName
+        String              newReferencingName,
+        ProviderInfo[]      providerInfo
 	)
 	{
 		return new CreateTriggerConstantAction(triggerSchemaName, triggerName, 
 				eventMask, isBefore, isRow, isEnabled, triggerTable, whenSPSId,
                 whenText, actionSPSId, actionText, spsCompSchemaId,
-				referencedCols, referencedColsInTriggerAction, originalActionText,
-				referencingOld, referencingNew, oldReferencingName, newReferencingName);
+                referencedCols, referencedColsInTriggerAction,
+                originalWhenText, originalActionText,
+                referencingOld, referencingNew,
+                oldReferencingName, newReferencingName, providerInfo);
 	}
 
 	/**
@@ -1073,4 +1161,67 @@ public class GenericConstantActionFactory
 	{
 		return new RevokeRoleConstantAction(roleNames, grantees);
 	}
+
+	/**
+	 * Make the ConstantAction for a WHEN [ NOT ] MATCHED clause.
+	 */
+	public	ConstantAction	getMatchingClauseConstantAction
+	(
+         int    clauseType,
+         String matchRefinementName,
+         ResultDescription  thenColumnSignature,
+         String rowMakingMethodName,
+         String resultSetFieldName,
+         String actionMethodName,
+         ConstantAction thenAction
+     )
+	{
+		return new MatchingClauseConstantAction
+            (
+             clauseType,
+             matchRefinementName,
+             thenColumnSignature,
+             rowMakingMethodName,
+             resultSetFieldName,
+             actionMethodName,
+             thenAction
+             );
+	}
+
+	/**
+	 * Make the ConstantAction for a MERGE statement.
+	 */
+	public	MergeConstantAction	getMergeConstantAction
+        (
+         ConstantAction[] matchingClauses
+         )
+	{
+		return new MergeConstantAction( matchingClauses );
+	}
+
+    /**
+     * Privileged lookup of a Context. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  Context    getContext( final String contextID )
+    {
+        if ( System.getSecurityManager() == null )
+        {
+            return ContextService.getContext( contextID );
+        }
+        else
+        {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedAction<Context>()
+                 {
+                     public Context run()
+                     {
+                         return ContextService.getContext( contextID );
+                     }
+                 }
+                 );
+        }
+    }
+
 }
