@@ -22,25 +22,26 @@
 package org.apache.derby.impl.sql.execute;
 
 import java.util.Properties;
-
 import org.apache.derby.catalog.UUID;
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.i18n.MessageService;
 import org.apache.derby.iapi.services.io.FormatableBitSet;
 import org.apache.derby.iapi.services.monitor.Monitor;
-import org.apache.derby.iapi.services.sanity.SanityManager;
 import org.apache.derby.iapi.sql.Activation;
 import org.apache.derby.iapi.sql.ResultDescription;
 import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
 import org.apache.derby.iapi.sql.dictionary.ConglomerateDescriptor;
 import org.apache.derby.iapi.sql.dictionary.ConstraintDescriptor;
+import org.apache.derby.iapi.sql.dictionary.ConstraintDescriptorList;
 import org.apache.derby.iapi.sql.dictionary.DataDictionary;
 import org.apache.derby.iapi.sql.dictionary.IndexRowGenerator;
+import org.apache.derby.iapi.sql.dictionary.ReferencedKeyConstraintDescriptor;
 import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
 import org.apache.derby.iapi.sql.execute.CursorResultSet;
 import org.apache.derby.iapi.sql.execute.ExecIndexRow;
 import org.apache.derby.iapi.sql.execute.ExecRow;
+import org.apache.derby.iapi.store.access.BackingStoreHashtable;
 import org.apache.derby.iapi.store.access.ConglomerateController;
 import org.apache.derby.iapi.store.access.DynamicCompiledOpenConglomInfo;
 import org.apache.derby.iapi.store.access.ScanController;
@@ -48,22 +49,23 @@ import org.apache.derby.iapi.store.access.StaticCompiledOpenConglomInfo;
 import org.apache.derby.iapi.store.access.TransactionController;
 import org.apache.derby.iapi.types.DataValueDescriptor;
 import org.apache.derby.iapi.types.RowLocation;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 /**
-  Perform Index maintenace associated with DML operations for a single index.
+  Perform Index maintenance associated with DML operations for a single index.
   */
-public class IndexChanger
+class IndexChanger
 {
-	private IndexRowGenerator irg;
+    final private IndexRowGenerator irg;
 	//Index Conglomerate ID
-	private long indexCID;
-	private DynamicCompiledOpenConglomInfo indexDCOCI;
-	private StaticCompiledOpenConglomInfo indexSCOCI;
-	private String indexName;
+    final private long indexCID;
+    final private DynamicCompiledOpenConglomInfo indexDCOCI;
+    final private StaticCompiledOpenConglomInfo indexSCOCI;
+    final private String indexName;
 	private ConglomerateController baseCC;
-	private TransactionController tc;
-	private int lockMode;
-	private FormatableBitSet baseRowReadMap;
+    final private TransactionController tc;
+    final private int lockMode;
+    final private FormatableBitSet baseRowReadMap;
 
 	private ConglomerateController indexCC = null;
 	private ScanController indexSC = null;
@@ -79,13 +81,17 @@ public class IndexChanger
 	private final Activation				activation;
 	private boolean					ownIndexSC = true;
 
-	/**
+    private final boolean deferrable; // supports a deferrable constraint
+    private final LanguageConnectionContext lcc;
+    private BackingStoreHashtable deferredDuplicates; // cached for speed
+
+    /**
 	  Create an IndexChanger
 
 	  @param irg the IndexRowGenerator for the index.
 	  @param indexCID the conglomerate id for the index.
-	  @param indexSCOCI the SCOCI for the idexes. 
-	  @param indexDCOCI the DCOCI for the idexes. 
+	  @param indexSCOCI the SCOCI for the indexes.
+	  @param indexDCOCI the DCOCI for the indexes.
 	  @param baseCC the ConglomerateController for the base table.
 	  @param tc			The TransactionController
 	  @param lockMode	The lock mode (granularity) to use
@@ -95,7 +101,7 @@ public class IndexChanger
 
 	  @exception StandardException		Thrown on error
 	  */
-	public IndexChanger
+    IndexChanger
 	(
 		IndexRowGenerator 		irg,
 		long 					indexCID,
@@ -112,6 +118,8 @@ public class IndexChanger
 		 throws StandardException
 	{
 		this.irg = irg;
+        this.deferrable = irg.hasDeferrableChecking(); // cache value
+                                                       // for speed..
 		this.indexCID = indexCID;
 		this.indexSCOCI = indexSCOCI;
 		this.indexDCOCI = indexDCOCI;
@@ -124,8 +132,11 @@ public class IndexChanger
 		this.activation = activation;
 		this.indexName = indexName;
 
-		// activation will be null when called from DataDictionary
-		if (activation != null && activation.getIndexConglomerateNumber() == indexCID)
+        this.lcc = (activation != null) ?
+                activation.getLanguageConnectionContext() : null;
+        // activation will be null when called from DataDictionary
+        if (activation != null &&
+                activation.getIndexConglomerateNumber() == indexCID)
 		{
 			ownIndexSC = false;
 		}
@@ -134,10 +145,9 @@ public class IndexChanger
 		{
 			SanityManager.ASSERT(tc != null, 
 				"TransactionController argument to constructor is null");
-			SanityManager.ASSERT(irg != null, 
-				"IndexRowGenerator argument to constructor is null");
 		}
-	}
+
+    }
 
 	/**
 	 * Set the row holder for this changer to use.
@@ -148,7 +158,7 @@ public class IndexChanger
 	 *
 	 * @param rowHolder	the row holder
 	 */
-	public void setRowHolder(TemporaryRowHolderImpl rowHolder)
+    void setRowHolder(TemporaryRowHolderImpl rowHolder)
 	{
 		this.rowHolder = rowHolder;
 		rowHolderPassedIn = (rowHolder != null);
@@ -160,7 +170,7 @@ public class IndexChanger
 	 *
 	 * @param baseCC	The heap's ConglomerateController.
 	 */
-	public void setBaseCC(ConglomerateController baseCC)
+    void setBaseCC(ConglomerateController baseCC)
 	{
 		this.baseCC = baseCC;
 	}
@@ -355,9 +365,9 @@ public class IndexChanger
 
                 Object[] args = new Object[2];
                 args[0] = ourIndexRow.getRowArray()[ourIndexRow.getRowArray().length - 1];
-                args[1] = new Long(indexCID);
+                args[1] = indexCID;
 
-                Monitor.getStream().println(MessageService.getCompleteMessage(
+                Monitor.getStream().println(MessageService.getTextMessage(
                     SQLState.LANG_IGNORE_MISSING_INDEX_ROW_DURING_DELETE, 
                     args));
 
@@ -424,22 +434,138 @@ public class IndexChanger
 		}
 	}
 
+    private UUID uniqueConstraintId; // cached copy
+
+    // Return the id of the corresponding unique or primary key
+    // constraint. Note: this only works because deferrable constraints
+    // do not share an index with other constraints and explicit indexes, so the
+    // mapping back from index conglomerate to constraint is one-to-one.
+    private UUID getUniqueConstraintId() throws StandardException {
+        if (uniqueConstraintId == null) {
+            DataDictionary dd = lcc.getDataDictionary();
+            ConglomerateDescriptor cd = dd.getConglomerateDescriptor(indexCID);
+            uniqueConstraintId =
+                    dd.getConstraintDescriptor(
+                    dd.getTableDescriptor(cd.getTableID()),
+                    cd.getUUID()).getUUID();
+        }
+
+        return uniqueConstraintId;
+    }
+
 	/**
 	 * Insert the given row into the given conglomerate and check for duplicate
 	 * key error.
 	 *
 	 * @param row	The row to insert
 	 *
-	 * @exception StandardException		Thrown on duplicate key error
+     * @exception StandardException     Thrown on duplicate key error unless
+     *                                  we have a deferred constraint. In that
+     *                                  index rows are saved for checking
+     *                                  on commit.
 	 */
 	private void insertAndCheckDups(ExecIndexRow row)
 				throws StandardException
 	{
 		openIndexCC();
 
-		int insertStatus = indexCC.insert(row.getRowArray());
+        int insertStatus;
 
-		if (insertStatus == ConglomerateController.ROWISDUPLICATE)
+        final DataValueDescriptor[] rowArray = row.getRowArray();
+
+        if (deferrable) {
+            insertStatus = indexCC.insert(row.getRowArray());
+
+            if (SanityManager.DEBUG) { // deferrable: we use a non-unique index
+                SanityManager.ASSERT(
+                        insertStatus != ConglomerateController.ROWISDUPLICATE);
+            }
+
+            final DataValueDescriptor [] key =
+                    new DataValueDescriptor[rowArray.length - 1];
+            System.arraycopy(rowArray, 0, key, 0, key.length);
+
+            // If the constraint mode is deferred, perform the check without
+            // waiting for any locks; we will just presume any lock conflicts
+            // constitute duplicates (not always the case), and check those keys
+            // again at commit time.
+            final boolean deferred = lcc.isEffectivelyDeferred(
+                    lcc.getCurrentSQLSessionContext(activation),
+                    getUniqueConstraintId());
+            // TODO add assert getUniqueConstraintId() != null
+
+            ScanController idxScan = tc.openScan(
+                    indexCID,
+                    false,
+                    (deferred ?
+                     TransactionController.OPENMODE_LOCK_ROW_NOWAIT :
+                     0),
+                    TransactionController.MODE_RECORD,
+                    TransactionController.ISOLATION_READ_COMMITTED_NOHOLDLOCK,
+                    (FormatableBitSet)null, // retrieve all fields
+                    key,
+                    ScanController.GE, // startSearchOp
+                    null,
+                    key,
+                    ScanController.GT);
+
+            boolean duplicate = false;
+
+            try {
+                final boolean foundOne = idxScan.next();
+
+                if (SanityManager.DEBUG) {
+                    SanityManager.ASSERT(
+                            foundOne, "IndexChanger: inserted row gone?");
+                }
+
+                duplicate = foundOne && idxScan.next();
+
+            } catch (StandardException e) {
+                if ((e.getSQLState().equals(SQLState.LOCK_TIMEOUT) ||
+                     e.getSQLState().equals(SQLState.DEADLOCK)) &&
+                     deferred)  {
+                    // Assume there is a duplicate, so we'll check again at
+                    // commit time.
+                    duplicate = true;
+                } else {
+                    throw e;
+                }
+            }
+
+            if (duplicate && irg.isUniqueWithDuplicateNulls()) {
+                int keyParts = rowArray.length - 1;
+
+                for (int i = 0; i < keyParts; i++) {
+                    // Keys with null in it are always unique
+                    if (rowArray[i].isNull()) {
+                        duplicate = false;
+                        break;
+                    }
+                }
+            }
+
+            if (duplicate) {
+                if (deferred) {
+                    // Save duplicate row so we can check at commit time there is
+                    // no longer any duplicate.
+
+                    deferredDuplicates =
+                        DeferredConstraintsMemory.rememberDuplicate(
+                            lcc,
+                            deferredDuplicates,
+                            getUniqueConstraintId(),
+                            row.getRowArray());
+                } else { // the constraint is not deferred, so throw
+                    insertStatus = ConglomerateController.ROWISDUPLICATE;
+                }
+            }
+
+        } else { // not a deferred constraint
+            insertStatus = indexCC.insert(row.getRowArray());
+        }
+
+        if (insertStatus == ConglomerateController.ROWISDUPLICATE)
 		{
 			/*
 			** We have a duplicate key error. 
@@ -467,16 +593,14 @@ public class IndexChanger
 				StandardException.newException(
 				SQLState.LANG_DUPLICATE_KEY_CONSTRAINT, indexOrConstraintName, tableName);
 			throw se;
-		}
-		if (SanityManager.DEBUG)
-		{
-			if (insertStatus != 0)
-			{
-				SanityManager.THROWASSERT("Unknown insert status " + insertStatus);
-			}
-		}
-	}
-
+        } else {
+            if (SanityManager.DEBUG) {
+                if (insertStatus != 0) {
+                    SanityManager.THROWASSERT("Unknown insert status " + insertStatus);
+                }
+            }
+        }
+    }
 
 	/**
 	 * Open the ConglomerateController for this index if it isn't open yet.
@@ -524,7 +648,7 @@ public class IndexChanger
 
 	  @exception StandardException		Thrown on error
 	  */
-	public void open()
+    void open()
 		 throws StandardException
 	{
 	}
@@ -536,8 +660,7 @@ public class IndexChanger
 	  @param baseRowLocation the base table row's location.
 	  @exception StandardException		Thrown on error
 	  */
-	public void delete(ExecRow baseRow,
-					   RowLocation baseRowLocation)
+    void delete(ExecRow baseRow, RowLocation baseRowLocation)
 		 throws StandardException
 	{
 		setOurIndexRow(baseRow, baseRowLocation);
@@ -554,7 +677,7 @@ public class IndexChanger
 
 	  @exception StandardException		Thrown on error
 	  */
-	public void update(ExecRow oldBaseRow,
+    void update(ExecRow oldBaseRow,
 					   ExecRow newBaseRow,
 					   RowLocation baseRowLocation
 					   )
@@ -583,7 +706,7 @@ public class IndexChanger
 
 	  @exception StandardException		Thrown on error
 	  */
-	public void insert(ExecRow newRow, RowLocation baseRowLocation)
+    void insert(ExecRow newRow, RowLocation baseRowLocation)
 		 throws StandardException
 	{
 		setOurIndexRow(newRow, baseRowLocation);
@@ -608,7 +731,9 @@ public class IndexChanger
 	{
 		setOurIndexRow(newRow, baseRowLocation);
 		//defer inserts if its on unique or UniqueWhereNotNull index
-		if (irg.isUnique() || irg.isUniqueWithDuplicateNulls())
+        if (irg.isUnique() ||
+            irg.isUniqueWithDuplicateNulls() ||
+            irg.hasDeferrableChecking())
 		{
 			doDeferredInsert();
 		}
@@ -625,8 +750,7 @@ public class IndexChanger
 
 	  @exception StandardException		Thrown on error
 	 */
-	public void finish()
-		throws StandardException
+    void finish() throws StandardException
 	{
 		ExecRow			deferredRow;
 
@@ -672,8 +796,7 @@ public class IndexChanger
 
 	  @exception StandardException		Thrown on error
 	  */
-	public void close()
-		throws StandardException
+    void close() throws StandardException
 	{
 		closeIndexCC();
 		closeIndexSC();
