@@ -21,8 +21,8 @@
 
 package org.apache.derby.impl.sql.execute;
 
+import java.sql.ResultSet;
 import java.sql.SQLException;
-import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Map;
 import java.util.Vector;
@@ -35,7 +35,6 @@ import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.jdbc.ConnectionContext;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.i18n.MessageService;
-import org.apache.derby.iapi.services.sanity.SanityManager;
 import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
 import org.apache.derby.iapi.sql.dictionary.TriggerDescriptor;
 import org.apache.derby.iapi.sql.execute.ConstantAction;
@@ -57,13 +56,12 @@ import org.apache.derby.iapi.types.DataValueDescriptor;
  * were holding from when the trigger fired.
  *
  */
-public class InternalTriggerExecutionContext implements TriggerExecutionContext, ExecutionStmtValidator
+class InternalTriggerExecutionContext
+        implements TriggerExecutionContext, ExecutionStmtValidator
 {
 	/*
 	** Immutable
 	*/
-	protected int[]					changedColIds;
-	protected String[]				changedColNames;
 	protected int					dmlType;
 	protected String				statementText;
 	protected ConnectionContext		cc;
@@ -94,7 +92,8 @@ public class InternalTriggerExecutionContext implements TriggerExecutionContext,
 	** space because they can no longer provide meaningful
 	** results.
 	*/
-	private Vector	resultSetVector;
+    @SuppressWarnings("UseOfObsoleteCollectionType")
+    private Vector<ResultSet>   resultSetVector;
 
 	/**
 	 * aiCounters is a vector of AutoincrementCounters used to keep state which
@@ -104,17 +103,18 @@ public class InternalTriggerExecutionContext implements TriggerExecutionContext,
 	 * @see AutoincrementCounter
 	 * 
 	 */
-	private Vector aiCounters;
+    @SuppressWarnings("UseOfObsoleteCollectionType")
+    private Vector<AutoincrementCounter> aiCounters;
 	
 	/**
-	 * aiHT is a hashtable of autincrement <key, value> pairs. This is used for
-	 * ai values generated by the trigger.
+     * aiHT is a hash table of auto increment (key, value) pairs. This is used
+     * for auto increment values generated by the trigger.
 	 */
-	private Hashtable aiHT;
+    @SuppressWarnings("UseOfObsoleteCollectionType")
+	private Hashtable<String,Long> aiHT;
 
 	/**
-	 * Build me a big old nasty trigger execution context.
-	 * Damnit.
+     * Build a trigger execution context.
 	 * <p>
 	 * About the only thing of real interest to outside observers
 	 * is that it pushes itself as the trigger execution context
@@ -125,9 +125,6 @@ public class InternalTriggerExecutionContext implements TriggerExecutionContext,
 	 * @param lcc	the lcc
 	 * @param statementText	the text of the statement that caused the
 	 *		trigger to fire.  may be null if we are replicating
-	 * @param changedColIds	the list of columns that changed.  Null
-	 *		for all columns or INSERT/DELETE.
-	 * @param changedColNames	the names that correspond to changedColIds
 	 * @param targetTableId	the UUID of the table upon which the trigger
 	 *		fired
 	 * @param targetTableName	the name of the table upon which the trigger
@@ -137,45 +134,27 @@ public class InternalTriggerExecutionContext implements TriggerExecutionContext,
 	 *
 	 * @exception StandardException on error
 	 */
-	public InternalTriggerExecutionContext
+    @SuppressWarnings({"UseOfObsoleteCollectionType", "LeakingThisInConstructor"})
+    InternalTriggerExecutionContext
 	(
 		LanguageConnectionContext	lcc,
 		ConnectionContext			cc,
 		String 						statementText,
 		int 						dmlType,
-		int[]						changedColIds,
-		String[]					changedColNames,
 		UUID						targetTableId,
 		String						targetTableName,
-		Vector						aiCounters
+        Vector<AutoincrementCounter> aiCounters
 	) throws StandardException
 	{
 		this.dmlType = dmlType;
-		this.changedColIds = changedColIds;
-		this.changedColNames = changedColNames;
 		this.statementText = statementText;
 		this.cc = cc;
 		this.lcc = lcc;
 		this.targetTableId = targetTableId;
 		this.targetTableName = targetTableName;
-		this.resultSetVector = new Vector();
+		this.resultSetVector = new Vector<java.sql.ResultSet>();
 		this.aiCounters = aiCounters;
 
-		if (SanityManager.DEBUG)
-		{	
-			if ((changedColIds == null) != (changedColNames == null))
-			{
-				SanityManager.THROWASSERT("bad changed cols, "+
-					"(changedColsIds == null) = "+(changedColIds == null)+ 
-					"  (changedColsNames == null) = "+(changedColNames == null));
-			}
-			if (changedColIds != null)
-			{
-				SanityManager.ASSERT(changedColIds.length == changedColNames.length, 
-					"different number of changed col ids vs names");
-			}
-		}
-
 		lcc.pushTriggerExecutionContext(this);
 	}
 
@@ -253,20 +232,21 @@ public class InternalTriggerExecutionContext implements TriggerExecutionContext,
 	protected void cleanup()
 		throws StandardException
 	{
-		lcc.popTriggerExecutionContext(this);
+        if (lcc != null) {
+            lcc.popTriggerExecutionContext(this);
+        }
 
 		/*
 		** Explicitly close all result sets that we have
 		** given out to the user.  
 	 	*/
-		for (Enumeration e = resultSetVector.elements();
-			 e.hasMoreElements(); )
-		{
-			java.sql.ResultSet rs = (java.sql.ResultSet)e.nextElement();
-			try
-			{
-				rs.close();
-			} catch (SQLException se) {}
+        if (resultSetVector != null) {
+            for (ResultSet rs : resultSetVector) {
+                try {
+                    rs.close();
+                } catch (SQLException se) {
+                }
+            }
 		}
 		resultSetVector = null;
 	
@@ -394,71 +374,6 @@ public class InternalTriggerExecutionContext implements TriggerExecutionContext,
 		return statementText;
 	}
 
-	/**
-	 * Get the columns that have been modified by the statement
-	 * that caused this trigger to fire.  If all columns are
-	 * modified, will return null (e.g. for INSERT or DELETE will
-	 * return null).
-	 *
-	 * @return an array of Strings
-	 */
-	public String[] getModifiedColumns()
-	{
-		return changedColNames;
-	}
-
-	/**
-	 * Find out of a column was changed, by column name
-	 *
-	 * @param columnName the column to check
- 	 *
-	 * @return true if the column was modified by this statement.
-	 * Note that this will always return true for INSERT
-	 * and DELETE regardless of the column name passed in.
-	 */
-	public boolean wasColumnModified(String columnName)
-	{
-		if (changedColNames == null)
-		{
-			return true;
-		}
-
-		for (int i = 0; i < changedColNames.length; i++)
-		{
-			if (changedColNames[i].equals(columnName))
-			{
-				return true;
-			}
-		}
-		return false;
-	}
-
-	/**
-	 * Find out of a column was changed, by column number
-	 *
-	 * @param columnNumber the column to check
- 	 *
-	 * @return true if the column was modified by this statement.
-	 * Note that this will always return true for INSERT
-	 * and DELETE regardless of the column name passed in.
-	 */
-	public boolean wasColumnModified(int columnNumber)
-	{
-		if (changedColIds == null)
-		{
-			return true;
-		}
-
-		for (int i = 0; i < changedColNames.length; i++)
-		{
-			if (changedColIds[i] == columnNumber)
-			{
-				return true;
-			}
-		}
-		return false;
-	}
-
 	/**
 	 * Returns a result set row the old images of the changed rows.
 	 * For a row trigger, the result set will have a single row.  For
@@ -590,7 +505,7 @@ public class InternalTriggerExecutionContext implements TriggerExecutionContext,
 		// by this trigger.
 		if (aiHT != null)
 			{
-				Long value = (Long)aiHT.get(identity);
+                Long value = aiHT.get(identity);
 				if (value != null)
 					return value;
 			}
@@ -600,15 +515,10 @@ public class InternalTriggerExecutionContext implements TriggerExecutionContext,
 		// represent values inherited by trigger from insert statements.
 		if (aiCounters != null)
 		{
-			for (int i = 0; i < aiCounters.size(); i++)
+            for (AutoincrementCounter aic : aiCounters)
 			{
-				AutoincrementCounter aic = 
-					(AutoincrementCounter)aiCounters.elementAt(i);
-
-				//				System.out.println("in itec:getaivalue " + aic);
 				if (identity.equals(aic.getIdentity()))
 				{
-					//					System.out.println("in itec:getvalue--returning " +  aic.getCurrentValue());
 					return aic.getCurrentValue();
 				}
 			}
@@ -618,15 +528,16 @@ public class InternalTriggerExecutionContext implements TriggerExecutionContext,
 		return null;
 	}
 	/**
-	 * Copy a map of autoincrement values into the trigger 
-	 * execution context hashtable of autoincrement values.
+     * Copy a map of auto increment values into the trigger
+     * execution context hash table of auto increment values.
 	 */
-	public void copyHashtableToAIHT(Map from)
+    @SuppressWarnings("UseOfObsoleteCollectionType")
+	public void copyHashtableToAIHT(Map<String,Long> from)
 	{
 		if (from == null)
 			return;
 		if (aiHT == null)
-			aiHT = new Hashtable();
+			aiHT = new Hashtable<String,Long>();
 
 		aiHT.putAll(from);
 	}
@@ -647,17 +558,14 @@ public class InternalTriggerExecutionContext implements TriggerExecutionContext,
 
 		afterRow = null;
 
-		int size = aiCounters.size();
-		for (int i = 0; i < size; i++)
+        for (AutoincrementCounter aic : aiCounters)
 		{
-			AutoincrementCounter aic = 
-				(AutoincrementCounter)aiCounters.elementAt(i);
 			aic.reset(begin);
 		}
 	}	
 	
 	/**
-	 * Update Autoincrement Counters from the last row inserted.
+     * Update auto increment counters from the last row inserted.
 	 *
 	 */
 	public void updateAICounters() throws StandardException
@@ -665,17 +573,15 @@ public class InternalTriggerExecutionContext implements TriggerExecutionContext,
 		if (aiCounters == null)
 			return;
 
-		int size = aiCounters.size();
-		for (int i = 0; i < size; i++)
+        for (AutoincrementCounter aic : aiCounters)
 		{
-			AutoincrementCounter aic = 
-				(AutoincrementCounter)aiCounters.elementAt(i);
 			DataValueDescriptor dvd = afterRow.getColumn(aic.getColumnPosition());
 			aic.update(dvd.getLong());
 		}
 	}
 
 
+    @Override
 	public String toString() {
 		return triggerd.getName();
 	}
