@@ -27,7 +27,7 @@ import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.i18n.MessageService;
 import org.apache.derby.iapi.services.loader.GeneratedMethod;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.Activation;
 import org.apache.derby.iapi.sql.execute.CursorResultSet;
 import org.apache.derby.iapi.sql.execute.ExecIndexRow;
@@ -48,7 +48,7 @@ import org.apache.derby.iapi.types.RowLocation;
  * the table's rows satisfying the filter as a result set.
  *
  * There are several things we could do during object
- * construction that are done in the open & next calls, to
+ * construction that are done in the open and next calls, to
  * improve performance.
  *
  */
@@ -72,15 +72,13 @@ class TableScanResultSet extends ScanResultSet
 	protected GeneratedMethod stopKeyGetter;
 	protected int stopSearchOperator;
 	public    Qualifier[][] qualifiers;
-	public String tableName;
 	public String userSuppliedOptimizerOverrides;
-	public String indexName;
 	protected boolean runTimeStatisticsOn;
     protected int[] indexCols;		//index keys base column position array
 	public int rowsPerRead;
 	public boolean forUpdate;
 	final boolean sameStartStopPosition;
-	private boolean nextDone;
+    protected boolean nextDone;
 	private RowLocation rlTemplate;
 
 	// Run time statistics
@@ -107,17 +105,17 @@ class TableScanResultSet extends ScanResultSet
 	// For Scrollable insensitive updatable result sets, only qualify a row the 
 	// first time it's been read, since an update can change a row so that it 
 	// no longer qualifies
-	private boolean qualify;
+    protected boolean qualify;
 
 	// currentRowIsValid is set to the result of positioning at a rowLocation.
 	// It will be true if the positioning was successful and false if the row 
 	// was deleted under our feet. Whenenver currentRowIsValid is false it means 
 	// that the row has been deleted.
-	private boolean currentRowIsValid;
+    protected boolean currentRowIsValid;
 	
 	// Indicates whether the scan has been positioned back to a previously read
 	// row, or it is accessing a row for the first time.
-	private boolean scanRepositioned;
+    protected boolean scanRepositioned;
 
     //
     // class interface
@@ -448,6 +446,11 @@ class TableScanResultSet extends ScanResultSet
 		openTime += getElapsedMillis(beginTime);
 	}
 
+
+    boolean loopControl(boolean moreRows) throws StandardException {
+        return scanController.fetchNext(candidate.getRowArray());
+    }
+
 	/**
      * Return the next row (if any) from the scan (if open).
 	 *
@@ -479,11 +482,15 @@ class TableScanResultSet extends ScanResultSet
 
 			if (scanControllerOpened)
 			{
-				boolean moreRows;
+                boolean moreRows = true;
 
-				while (moreRows =
-							scanController.fetchNext(candidate.getRowArray()))
+                while (true)
 				{
+                    // loop control overriden by subclass
+                    // ValidateCheckConstraintResultSet..
+                    if (! (moreRows = loopControl(moreRows))) {
+                        break;
+                    }
 					rowsSeen++;
 					rowsThisScan++;
 
