@@ -31,7 +31,7 @@ import org.apache.derby.iapi.services.cache.CacheManager;
 
 import org.apache.derby.iapi.services.monitor.Monitor;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.services.io.FormatIdUtil;
 
@@ -697,7 +697,7 @@ public abstract class CachedPage extends BasePage implements Cacheable
 					StandardException se = 
 						StandardException.newException(
 								   SQLState.FILE_READ_PAGE_EXCEPTION, 
-								   ioe, newIdentity, new Integer(pagesize));
+								   ioe, newIdentity, pagesize);
 
 						
 				    if (dataFactory.getLogFactory().inRFR())
