@@ -25,12 +25,11 @@ import org.apache.derby.iapi.reference.SQLState;
 
 import org.apache.derby.iapi.services.context.ContextService;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.services.io.Storable;
 import org.apache.derby.iapi.services.io.StreamStorable;
 import org.apache.derby.iapi.services.io.FormatIdInputStream;
 import org.apache.derby.iapi.services.io.FormatIdOutputStream;
-import org.apache.derby.iapi.services.io.FormatIdUtil;
 import org.apache.derby.iapi.services.io.StoredFormatIds;
 import org.apache.derby.iapi.services.io.TypedFormat;
 import org.apache.derby.iapi.services.monitor.Monitor;
@@ -38,27 +37,18 @@ import org.apache.derby.iapi.services.monitor.Monitor;
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.store.access.AccessFactory;
 import org.apache.derby.iapi.store.access.RowSource;
-import org.apache.derby.iapi.store.access.RowUtil;
 import org.apache.derby.iapi.store.access.TransactionController;
 import org.apache.derby.iapi.store.raw.ContainerKey;
 import org.apache.derby.iapi.store.raw.RawStoreFactory;
 import org.apache.derby.iapi.store.raw.StreamContainerHandle;
 
-import org.apache.derby.io.StorageFactory;
-import org.apache.derby.io.WritableStorageFactory;
 import org.apache.derby.io.StorageFile;
 
-import org.apache.derby.impl.store.raw.data.DecryptInputStream;
-import org.apache.derby.impl.store.raw.data.StoredFieldHeader;
-import org.apache.derby.impl.store.raw.data.StoredRecordHeader;
-
-import org.apache.derby.iapi.services.io.ArrayInputStream;
 import org.apache.derby.iapi.services.io.FormatableBitSet;
 import org.apache.derby.iapi.services.io.CompressedNumber;
 import org.apache.derby.iapi.services.io.DynamicByteArrayOutputStream;
 import org.apache.derby.iapi.services.io.LimitInputStream;
 import org.apache.derby.iapi.services.property.PropertyUtil;
-import org.apache.derby.iapi.util.ReuseFactory;
 
 import java.util.Properties;
 import java.io.InputStream;
@@ -68,6 +58,7 @@ import java.io.IOException;
 import java.io.EOFException;
 import java.io.InvalidClassException;
 import java.io.Externalizable;
+import java.security.PrivilegedAction;
 import java.security.AccessController;
 import java.security.PrivilegedExceptionAction;
 import java.security.PrivilegedActionException;
@@ -90,7 +81,7 @@ import java.io.FileNotFoundException;
 **/
 
 
-public class StreamFileContainer implements TypedFormat, PrivilegedExceptionAction
+class StreamFileContainer implements TypedFormat, PrivilegedExceptionAction<Object>
 {
 
     /**************************************************************************
@@ -403,13 +394,13 @@ public class StreamFileContainer implements TypedFormat, PrivilegedExceptionActi
     {
 
 		AccessFactory af = (AccessFactory)
-			Monitor.getServiceModule(dataFactory, AccessFactory.MODULE);
+			getServiceModule(dataFactory, AccessFactory.MODULE);
 
 		TransactionController tc = 
             (af == null) ? 
                 null : 
                 af.getTransaction(
-                    ContextService.getFactory().getCurrentContextManager());
+                    getContextService().getCurrentContextManager());
 
 		bufferSize = 
 			PropertyUtil.getServiceInt(tc, prop,
@@ -1042,14 +1033,22 @@ public class StreamFileContainer implements TypedFormat, PrivilegedExceptionActi
                     {
 						if (!privExists(directory)) 
                         {
-							if (!privMkdirs(directory)) 
+                            boolean created = false;
+                            IOException ex = null;
+                            try {
+                                created = privMkdirs(directory);
+                            } catch (IOException ioe) {
+                                ex = ioe;
+                            }
+
+                            if (!created)
                             {
 								if (errorOK)
 									return null;
 								else
 									throw StandardException.newException(
                                             SQLState.FILE_CANNOT_CREATE_SEGMENT,
-                                            directory);
+                                            ex, directory);
 							}
 						}
 					}
@@ -1084,6 +1083,7 @@ public class StreamFileContainer implements TypedFormat, PrivilegedExceptionActi
     }
 
     private synchronized boolean privMkdirs(StorageFile file)
+            throws IOException
     {
         actionCode = STORAGE_FILE_MKDIRS_ACTION;
         actionStorageFile = file;
@@ -1095,8 +1095,8 @@ public class StreamFileContainer implements TypedFormat, PrivilegedExceptionActi
         }catch( PrivilegedActionException pae) 
         {
             // method executed under this priveleged block 
-            // does not throw an exception
-            return false;
+            // could throw IOException
+            throw (IOException) pae.getCause();
         } 
         finally
         {
@@ -1167,18 +1167,18 @@ public class StreamFileContainer implements TypedFormat, PrivilegedExceptionActi
 
 
     // PrivilegedAction method
-    public Object run() throws FileNotFoundException
+    public Object run() throws IOException
     {
         switch(actionCode)
         {
         case STORAGE_FILE_EXISTS_ACTION:
-            return ReuseFactory.getBoolean(actionStorageFile.exists());
+            return actionStorageFile.exists();
         case STORAGE_FILE_DELETE_ACTION:
-            return ReuseFactory.getBoolean(actionStorageFile.delete());
+            return actionStorageFile.delete();
         case STORAGE_FILE_MKDIRS_ACTION:
             boolean created = actionStorageFile.mkdirs();
             actionStorageFile.limitAccessToOwner();
-            return ReuseFactory.getBoolean(created);
+            return created;
         case STORAGE_FILE_GET_OUTPUT_STREAM_ACTION:
             return actionStorageFile.getOutputStream();
         case STORAGE_FILE_GET_INPUT_STREAM_ACTION:
@@ -1188,4 +1188,40 @@ public class StreamFileContainer implements TypedFormat, PrivilegedExceptionActi
         return null;
     }
 
+    /**
+     * Privileged lookup of the ContextService. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ContextService    getContextService()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ContextService>()
+             {
+                 public ContextService run()
+                 {
+                     return ContextService.getFactory();
+                 }
+             }
+             );
+    }
+
+    /**
+     * Privileged module lookup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private static  Object getServiceModule( final Object serviceModule, final String factoryInterface )
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<Object>()
+             {
+                 public Object run()
+                 {
+                     return Monitor.getServiceModule( serviceModule, factoryInterface );
+                 }
+             }
+             );
+    }
+
 }
