@@ -67,7 +67,7 @@ import java.util.Dictionary;
 
 import org.apache.derby.iapi.error.StandardException;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.services.io.DynamicByteArrayOutputStream;
 import org.apache.derby.iapi.util.ByteArray;
 import org.apache.derby.iapi.services.locks.CompatibilitySpace;
@@ -179,6 +179,9 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 	// id that is valid locally in this raw store.
 	private volatile TransactionId	myId;
 
+    // id of parent transaction if this is a nested user transaction
+    private volatile TransactionId parentTransactionId;
+
 	protected Logger		logger;		// the object we use to access the log.
 
 
@@ -216,12 +219,14 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 	private LogInstant		logLast;  // the last log record written by this
 									  // transaction 
 
-	private Stack			savePoints;	// stack of SavePoint objects.
+	private Stack<SavePoint>			savePoints;	// stack of SavePoint objects.
 
-	protected List   		postCommitWorks; // a list of post commit work
-	protected List		    postTerminationWorks; // work to be done after
-												  // transaction terminates,
-												  // commit or abort
+	protected List<Serviceable> postCommitWorks; // a list of post commit work
+	protected List<Serviceable> postAbortWorks;  // a list of post abort work
+	protected List<Serviceable> postTerminationWorks; // work to be done after
+												      // transaction terminates,
+												      // commit or abort
+                                                      
 	private boolean			recoveryTransaction;  // this transaction is being
 												  // used by recovery
 
@@ -272,8 +277,9 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 	** Constructor
 	*/
 
-	protected Xact(
+    protected Xact(
     XactFactory         xactFactory, 
+    Xact          parentTransaction, 
     LogFactory          logFactory, 
     DataFactory         dataFactory,
     DataValueFactory    dataValueFactory,
@@ -285,6 +291,10 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 		super();
 
 		this.xactFactory            = xactFactory;
+        if ( parentTransaction != null )
+        {
+            parentTransactionId = parentTransaction.getId();
+        }
 		this.logFactory             = logFactory;
 		this.dataFactory            = dataFactory;
 		this.dataValueFactory       = dataValueFactory;
@@ -479,7 +489,7 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 	/**
 		Set my transaction identifier.
 	*/
-	public void setTransactionId(GlobalTransactionId extid, TransactionId localid) {
+    void setTransactionId(GlobalTransactionId extid, TransactionId localid) {
 
 		if (SanityManager.DEBUG) {
 
@@ -1015,6 +1025,7 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 		{
 			postCommitWorks.clear();
 		}
+       
 
 		// Now do post termination work - must do this after the rollback is
 		// complete because the rollback itself may generate postTermination
@@ -1128,8 +1139,6 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 		@exception StandardException Thrown if the transaction is not idle, the
 		transaction remains open.
 		@see Transaction#close
-
-		@exception StandardException	Standard Derby policy
 	*/
 	public void close() throws StandardException {
 
@@ -1245,7 +1254,7 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 					    // set the top savepoint to rollback to this record if
                         // it doesn't yet have a point saved
 
-						SavePoint sp = (SavePoint) savePoints.elementAt(i);
+						SavePoint sp = savePoints.elementAt(i);
 						if (sp.getSavePoint() == null) {
 							sp.setSavePoint(instant);
 						} else
@@ -1268,18 +1277,35 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 		if (recoveryTransaction)
 			return;
 
+
 		if (postCommitWorks == null)
-			postCommitWorks = new ArrayList(1);
+			postCommitWorks = new ArrayList<Serviceable>(1);
 		postCommitWorks.add(work);
 	}
 
+	/**
+		Add to the list of post abort work that may be processed after this
+		transaction aborts.  
+
+		@param work the post commit work that is added
+	*/
+	public void addPostAbortWork(Serviceable work)
+	{
+		if (recoveryTransaction)
+			return;
+
+		if (postAbortWorks == null)
+			postAbortWorks = new ArrayList<Serviceable>(1);
+		postAbortWorks.add(work);
+	}
+
 	public void addPostTerminationWork(Serviceable work)
 	{
 		if (recoveryTransaction)
 			return;
 
 		if (postTerminationWorks == null)
-			postTerminationWorks = new ArrayList(2);
+			postTerminationWorks = new ArrayList<Serviceable>(2);
 		postTerminationWorks.add(work);
 	}
 
@@ -1475,7 +1501,7 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
         }
 
 		if (savePoints == null)
-			savePoints = new Stack();
+			savePoints = new Stack<SavePoint>();
 
 		savePoints.push(new SavePoint(name, kindOfSavepoint));
 
@@ -1506,7 +1532,7 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 
 		if ((savePoints != null) && !savePoints.empty()) {
 			for (int i = savePoints.size() - 1; i >= 0; i--) {
-				SavePoint sp = (SavePoint) savePoints.elementAt(i);
+				SavePoint sp = savePoints.elementAt(i);
 				if (sp.isThisUserDefinedsavepoint()) 
 				{
                     //found a user defined savepoint
@@ -2039,26 +2065,16 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 
 	}
 
-
-	private final void postTermination() throws StandardException
-	{
-		// move all the postTermination work to the postCommit queue
-		int count = (postTerminationWorks == null) ? 
-			0 : postTerminationWorks.size(); 
-
-		for (int i = 0; i < count; i++)
-			addPostCommitWork((Serviceable)postTerminationWorks.get(i));
-
-		if (count > 0)
-			postTerminationWorks.clear();
-
+    private void transferPostCommitorAbortWork(List <Serviceable> work_list)
+        throws StandardException
+    {
 
 		// if there are post commit work to be done, transfer them to the
 		// daemon.  The log is flushed, all locks released and the
 		// transaction has ended at this point.
-		if (postCommitWorks != null && !postCommitWorks.isEmpty())
+		if (work_list != null && !work_list.isEmpty())
 		{
-			int pcsize = postCommitWorks.size();
+			int pcsize = work_list.size();
 			
 			// do we want to do post commit work with this transaction object?
 			if (doPostCommitWorkInTran())
@@ -2067,36 +2083,43 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 				{
 					inPostCommitProcessing = true;
 
-					// to avoid confusion, copy the post commit work to an array if this
-					// is going to do some work now
+					// to avoid confusion, copy the post commit or abort work 
+                    // to an array if this is going to do some work now
 					Serviceable[] work = new Serviceable[pcsize];
-					work = (Serviceable[])postCommitWorks.toArray(work);
+					work = (Serviceable[])work_list.toArray(work);
 
-					// clear this for post commit processing to queue its own post
-					// commit works - when it commits, it will send all its post
-					// commit request to the daemon instead of dealing with it here.
-					postCommitWorks.clear();
+					// clear this for post commit or abort processing to queue 
+                    // its own post commit works - when it commits, it will 
+                    // send all its post commit request to the daemon instead 
+                    // of dealing with it here.
+					work_list.clear();
 
-					//All the post commit work that is part  of the database creation
-					//should be done on this thread immediately.
-					boolean doWorkInThisThread = xactFactory.inDatabaseCreation();
+					//All the post commit or abort work that is part of the 
+                    //database creation should be done on this thread 
+                    //immediately.
+					boolean doWorkInThisThread = 
+                        xactFactory.inDatabaseCreation();
 
 					for (int i = 0; i < pcsize; i++)
 					{
 
 						//process work that should be done immediately or
 						//when we  are in still in database creattion.
-						//All the other work should be submitted 
-						//to the post commit thread to be processed asynchronously
+						//All the other work should be submitted to the post 
+                        //commit thread to be processed asynchronously
+
 						if (doWorkInThisThread || work[i].serviceImmediately())
 						{
 							try
 							{
 								// this may cause other post commit work to be
-								// added.  when that transaction commits, those
+								// added.  when that transaction commits, that
 								// work will be transfered to the daemon
-								if (work[i].performWork(xc.getContextManager()) == Serviceable.DONE)
+								if (work[i].performWork(xc.getContextManager())
+                                        == Serviceable.DONE)
+                                {
 									work[i] = null;
+                                }
 
 								// if REQUEUE, leave it on for the postcommit
 								// daemon to handle
@@ -2106,7 +2129,9 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 								// don't try to service this again
 								work[i] = null;
 
-								// try to handle it here.  If we fail, then let the error percolate.
+								// try to handle it here.  
+                                // If we fail, then let the error percolate.
+
 								xc.cleanupOnError(se);
 							}
 						}
@@ -2115,11 +2140,15 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 						// requeueing, send it off.   Note that this is one case
 						// where a REQUEUE ends up in the high priority queue.
 						// Unfortunately, there is no easy way to tell.  If the
-						// Servicable is well mannered, it can change itself from
-						// serviceASAP to not serviceASAP if it returns REQUEUE.
+						// Servicable is well mannered, it can change itself 
+                        // from serviceASAP to not serviceASAP if it returns 
+                        // REQUEUE.
+
 						if (work[i] != null)
 						{
-							boolean needHelp = xactFactory.submitPostCommitWork(work[i]);
+							boolean needHelp = 
+                                xactFactory.submitPostCommitWork(work[i]);
+
 							work[i] = null;
 							if (needHelp)
 								doWorkInThisThread = true;
@@ -2131,8 +2160,8 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 					inPostCommitProcessing = false;
 
 					// if something untoward happends, clear the queue.
-					if (postCommitWorks != null)
-						postCommitWorks.clear();
+					if (work_list != null)
+						work_list.clear();
 				}
 
 			}
@@ -2143,15 +2172,37 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 				// work submitting other post commit work)
 				for (int i = 0; i < pcsize; i++)
 				{
-					// SanityManager.DEBUG_PRINT("PostTermination",postCommitWorks.elementAt((i)).toString());
-					xactFactory.submitPostCommitWork((Serviceable)postCommitWorks.get((i)));
+					// SanityManager.DEBUG_PRINT("PostTermination",work_list.elementAt((i)).toString());
+					xactFactory.submitPostCommitWork(
+                        (Serviceable)work_list.get((i)));
 				}
 			}
 
-			postCommitWorks.clear();
+			work_list.clear();
 
 		}
 
+    }
+
+
+	private final void postTermination() throws StandardException
+	{
+		// move all the postTermination work to the postCommit queue
+		int count = (postTerminationWorks == null) ? 
+			0 : postTerminationWorks.size(); 
+
+		for (int i = 0; i < count; i++)
+			addPostCommitWork(postTerminationWorks.get(i));
+
+		if (count > 0)
+			postTerminationWorks.clear();
+
+        // transfer post commit work
+        transferPostCommitorAbortWork(postCommitWorks);
+
+        // transfer post abort work
+        transferPostCommitorAbortWork(postAbortWorks);
+
         // any backup blocking operations (like unlogged ops) in this 
         // transaction are done with post commit/abort work that needs be
         // done in the same trasaction,  unblock the backup.
@@ -2172,7 +2223,7 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 
         for (int i = savePoints.size() - 1; i >= 0; i--)
         {
-            SavePoint savepoint = (SavePoint)savePoints.elementAt(i);
+            SavePoint savepoint = savePoints.elementAt(i);
 
             if (savepoint.getName().equals(name))
             {
@@ -2212,7 +2263,7 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 
 		int size = savePoints.size();
 		for (int i = position; i < size; i++) {
-			SavePoint rollbackSavePoint = (SavePoint) savePoints.elementAt(i);
+			SavePoint rollbackSavePoint = savePoints.elementAt(i);
 
 			LogInstant li = rollbackSavePoint.getSavePoint();
 			if (li != null) {
@@ -2435,7 +2486,7 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 		throws StandardException {
 
 		// Count row locks by table
-		Dictionary containers = new java.util.Hashtable();
+		Dictionary<ContainerKey,LockCount> containers = new java.util.Hashtable<ContainerKey,LockCount>();
 
 		for (; lockList.hasMoreElements(); ) {
 
@@ -2465,10 +2516,10 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 		// this threshold
 
 		boolean didEscalate = false;
-		for (Enumeration e = containers.keys(); e.hasMoreElements(); ) {
-			ContainerKey ckey = (ContainerKey) e.nextElement();
+		for (Enumeration<ContainerKey> e = containers.keys(); e.hasMoreElements(); ) {
+			ContainerKey ckey = e.nextElement();
 
-			LockCount lc = (LockCount) containers.get(ckey);
+			LockCount lc = containers.get(ckey);
 
 			if (lc.count < threshold) {
 				continue;
@@ -2836,6 +2887,19 @@ public class Xact extends RawTransaction implements Limit, LockOwner {
 		logFactory.checkpointInRFR(cinstant, redoLWM, undoLWM, dataFactory);
 	}
 
+    public boolean isNestedOwner()
+    {
+        return (parentTransactionId != null );
+    }
+
+    public boolean nestsUnder( LockOwner other )
+    {
+        if ( parentTransactionId == null ) { return false; }
+        else if ( !(other instanceof Xact) ) { return false; }
+        {
+            return parentTransactionId.equals( ((Xact) other).getId() );
+        }
+    }
 }
 
 class LockCount {
