@@ -21,41 +21,34 @@
 
 package org.apache.derbyTesting.functionTests.tests.derbynet;
 
-import org.apache.derby.drda.NetworkServerControl;
-import org.apache.derbyTesting.functionTests.tests.lang.SecurityPolicyReloadingTest;
-import org.apache.derbyTesting.functionTests.tests.lang.SimpleTest;
-import org.apache.derbyTesting.functionTests.util.TestUtil;
-import org.apache.derbyTesting.junit.BaseJDBCTestCase;
-import org.apache.derbyTesting.junit.Derby;
-import org.apache.derbyTesting.junit.NetworkServerTestSetup;
-import org.apache.derbyTesting.junit.SecurityManagerSetup;
-import org.apache.derbyTesting.junit.SupportFilesSetup;
-import org.apache.derbyTesting.junit.SystemPropertyTestSetup;
-import org.apache.derbyTesting.junit.TestConfiguration;
-
-
 import java.io.File;
 import java.net.InetAddress;
 import java.net.UnknownHostException;
 import java.security.AccessController;
-import java.security.Policy;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 import java.sql.Connection;
 import java.sql.SQLException;
-import java.util.Enumeration;
 import java.util.Properties;
-
 import junit.framework.Test;
-import junit.framework.TestSuite;
+import org.apache.derby.drda.NetworkServerControl;
+import org.apache.derbyTesting.functionTests.util.PrivilegedFileOpsForTests;
+import org.apache.derbyTesting.functionTests.util.TestUtil;
+import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
+import org.apache.derbyTesting.junit.Derby;
+import org.apache.derbyTesting.junit.NetworkServerTestSetup;
+import org.apache.derbyTesting.junit.SecurityManagerSetup;
+import org.apache.derbyTesting.junit.SystemPropertyTestSetup;
+import org.apache.derbyTesting.junit.TestConfiguration;
 
 public class NetworkServerControlApiTest extends BaseJDBCTestCase {
 
     private static final String NON_ASCII_USER = "bj\u00F8rn";
     private static final String NON_ASCII_PASSWORD = "l\u00F8yndom";
 
-    private static String POLICY_FILE_NAME="functionTests/tests/derbynet/NetworkServerControlApiTest.policy";
-    private static String TARGET_POLICY_FILE_NAME="server.policy";
+    private static final String POLICY_FILE_NAME =
+            "org/apache/derbyTesting/functionTests/tests/derbynet/NetworkServerControlApiTest.policy";
     
     public NetworkServerControlApiTest(String name) {
         super(name);
@@ -213,73 +206,29 @@ public class NetworkServerControlApiTest extends BaseJDBCTestCase {
         }
     }
     
-        /**
-         * Wraps InitAddress.getByName in privilege block.
-         * 
-         * @param host  host to resolve
-         * @return InetAddress of host
-         * @throws UnknownHostException
-         */
-        private InetAddress privInetAddressGetByName(final String host) throws UnknownHostException
-        {
-            InetAddress inetAddr = null;
-            try {
-                inetAddr = (InetAddress) AccessController
-                    .doPrivileged(new PrivilegedExceptionAction() {
-                        public Object run() throws UnknownHostException {
-                            return InetAddress.getByName(host);
-                        }
-                    });
-            } catch (PrivilegedActionException pe) {
-                Exception e = pe.getException();
-                if (e instanceof UnknownHostException)
-                    throw (UnknownHostException) e;
-                else
-                    throw (SecurityException) e;
-            }
-        return inetAddr;
-            
-        }
-        
-        
-    
-    
-    private boolean fileExists(String filename) {
-        final File file = new File(filename);
-        try {
-            return ((Boolean)AccessController.doPrivileged(
-                new PrivilegedExceptionAction() {
-                    public Object run() throws SecurityException {
-                        return new Boolean(file.exists());
-                    }
-                })).booleanValue();
-        } catch (PrivilegedActionException pae) {
-            throw (SecurityException)pae.getException();
-        }
-        
-    }
-    
     /**
-     * Construct the name of the server policy file.
+     * Wraps InitAddress.getByName in privilege block.
+     *
+     * @param host host to resolve
+     * @return InetAddress of host
      */
-    private String makeServerPolicyName()
-    {
+    private InetAddress privInetAddressGetByName(final String host)
+            throws UnknownHostException {
         try {
-            String  userDir = getSystemProperty( "user.dir" );
-            String  fileName = userDir + File.separator + SupportFilesSetup.EXTINOUT + File.separator + TARGET_POLICY_FILE_NAME;
-            File      file = new File( fileName );
-            String  urlString = file.toURL().toExternalForm();
-
-            return urlString;
-        }
-        catch (Exception e)
-        {
-            System.out.println( "Unexpected exception caught by makeServerPolicyName(): " + e );
-
-            return null;
+            return AccessController.doPrivileged(
+                    new PrivilegedExceptionAction<InetAddress>() {
+                public InetAddress run() throws UnknownHostException {
+                    return InetAddress.getByName(host);
+                }
+            });
+        } catch (PrivilegedActionException pae) {
+            throw (UnknownHostException) pae.getCause();
         }
     }
-    
+
+    private boolean fileExists(String filename) {
+        return PrivilegedFileOpsForTests.exists(new File(filename));
+    }
     
     /**
      * Add decorators to a test run. Context is established in the reverse order
@@ -294,44 +243,19 @@ public class NetworkServerControlApiTest extends BaseJDBCTestCase {
      */
     private static Test decorateTest()
     {
-        
-        String serverPolicyName = new NetworkServerControlApiTest("test").makeServerPolicyName();
-
-        // Use a fixed ordering of the test cases. Some of the test cases set
-        // properties that will be seen by subsequent test cases, but only if
-        // they run in one particular order. Also, some test cases depend on
-        // the exact connection number, which can only be guaranteed if we
-        // know exactly how many connections have been established so far, and
-        // then we need a stable ordering of the test cases.
         Test test = TestConfiguration.clientServerDecorator(
-                TestConfiguration.orderedSuite(
-                        NetworkServerControlApiTest.class));
+                new BaseTestSuite(NetworkServerControlApiTest.class));
         //
         // Install a security manager using the initial policy file.
         //
-        test = new SecurityManagerSetup( test,serverPolicyName );
-        
-        
-        //
-        // Copy over the policy file we want to use.
-        //
-        test = new SupportFilesSetup
-            (
-             test,
-             null,
-             new String[] { POLICY_FILE_NAME },
-             null,
-             new String[] { TARGET_POLICY_FILE_NAME}
-             );
-
-       
-        return test;
+        return new SecurityManagerSetup(test, POLICY_FILE_NAME);
     }
     
     public static Test suite()
     {
         
-        TestSuite suite = new TestSuite("NetworkServerControlApiTest");
+        BaseTestSuite suite =
+            new BaseTestSuite("NetworkServerControlApiTest");
         
         // Need derbynet.jar in the classpath!
         if (!Derby.hasServer())
@@ -347,7 +271,9 @@ public class NetworkServerControlApiTest extends BaseJDBCTestCase {
         return suite;
     }
 
-    private static TestSuite decorateSystemPropertyTests(TestSuite suite) {
+    private static BaseTestSuite decorateSystemPropertyTests(
+        BaseTestSuite suite) {
+
         Properties traceProps = new Properties();
         traceProps.put("derby.drda.traceDirectory","/");
         traceProps.put("derby.drda.traceAll","true");
