@@ -20,23 +20,28 @@
 
 package org.apache.derbyTesting.functionTests.tests.jdbc4;
 
-import org.apache.derby.vti.VTITemplate;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
+import java.sql.CallableStatement;
+import java.sql.Connection;
+import java.sql.PreparedStatement;
+import java.sql.ResultSet;
+import java.sql.SQLException;
+import java.sql.SQLTimeoutException;
+import java.sql.Statement;
+import junit.framework.Test;
+import org.apache.derby.client.am.ClientStatement;
 import org.apache.derby.impl.jdbc.EmbedResultSet;
 import org.apache.derby.impl.sql.execute.RowUtil;
-
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derbyTesting.functionTests.tests.jdbcapi.BatchUpdateTest;
 import org.apache.derbyTesting.functionTests.tests.jdbcapi.SetQueryTimeoutTest;
 import org.apache.derbyTesting.functionTests.tests.jdbcapi.Wrapper41Statement;
 import org.apache.derbyTesting.functionTests.util.SQLStateConstants;
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.TestConfiguration;
 
-import junit.framework.*;
-
-import java.lang.reflect.InvocationTargetException;
-import java.lang.reflect.Method;
-import java.sql.*;
-
 /**
  * Tests for new methods added for Statement in JDBC4.
  */
@@ -367,6 +372,13 @@ public class StatementTest
     public  static  void    largeUpdate_jdbc4_2( Connection conn )
         throws Exception
     {
+        //
+        // This test makes use of a debug entry point which is a NOP
+        // in an insane production build.
+        //
+        if (!SanityManager.DEBUG)    { return; }
+        else { println( "Running largeUpdate_jdbc4_2() on debug code." ); }
+
         conn.prepareStatement
             (
              "create procedure setRowCountBase( newBase bigint )\n" +
@@ -393,6 +405,13 @@ public class StatementTest
     private static  void    largeUpdateTest( StatementWrapper sw, long rowCountBase )
         throws Exception
     {
+        //
+        // This test makes use of a debug entry point which is a NOP
+        // in an insane production build.
+        //
+        if (!SanityManager.DEBUG)    { return; }
+        else { println( "Running largeUpdateTest() on debug code." ); }
+
         // poke the rowCountBase into the engine. all returned row counts will be
         // increased by this amount
         setRowCountBase( sw.getWrappedStatement(), false, rowCountBase );
@@ -445,6 +464,12 @@ public class StatementTest
     private static  void    largeBatchTest( StatementWrapper sw, long rowCountBase )
         throws Exception
     {
+        //
+        // This test makes use of a debug entry point which is a NOP
+        // in an insane production build.
+        //
+        if (!SanityManager.DEBUG)    { return; }
+
         println( "Large batch test with rowCountBase = " + rowCountBase );
         
         // poke the rowCountBase into the engine. all returned row counts will be
@@ -472,6 +497,12 @@ public class StatementTest
     private static  void    largeMaxRowsTest( StatementWrapper sw, long maxRows )
         throws Exception
     {
+        //
+        // This test makes use of a debug entry point which is a NOP
+        // in an insane production build.
+        //
+        if (!SanityManager.DEBUG)    { return; }
+
         println( "Large max rows test with maxRows = " + maxRows );
 
         long    expectedRowCount = 3L;
@@ -496,6 +527,12 @@ public class StatementTest
     private static  void    largeBatchUpdateExceptionTest( StatementWrapper sw, long rowCountBase )
         throws Exception
     {
+        //
+        // This test makes use of a debug entry point which is a NOP
+        // in an insane production build.
+        //
+        if (!SanityManager.DEBUG)    { return; }
+
         println( "Large batch update exception test with rowCountBase = " + rowCountBase );
         
         sw.getWrappedStatement().clearBatch();
@@ -539,7 +576,7 @@ public class StatementTest
     {
         if ( onClient )
         {
-            org.apache.derby.client.am.Statement.fetchedRowBase = rowCountBase;
+            ClientStatement.setFetchedRowBase( rowCountBase );
         }
         else
         {
@@ -564,12 +601,12 @@ public class StatementTest
      * Create test suite for StatementTest.
      */
     public static Test suite() {
-        TestSuite suite = new TestSuite("StatementTest suite");
+        BaseTestSuite suite = new BaseTestSuite("StatementTest suite");
         // Decorate test suite with a TestSetup class.
         suite.addTest(new StatementTestSetup(
-                        new TestSuite(StatementTest.class)));
+                        new BaseTestSuite(StatementTest.class)));
         suite.addTest(TestConfiguration.clientServerDecorator(
-            new StatementTestSetup(new TestSuite(StatementTest.class))));
+            new StatementTestSetup(new BaseTestSuite(StatementTest.class))));
         return suite;
     }
 
@@ -621,7 +658,7 @@ public class StatementTest
                 (
                  "executeLargeUpdate",
                  new Class[] { String.class, Integer.TYPE },
-                 new Object[] { sql, new Integer( autoGeneratedKeys ) }
+                 new Object[] { sql, autoGeneratedKeys }
                  )).longValue();
         }
         public  long executeLargeUpdate( String sql, int[] columnIndexes ) throws SQLException
@@ -666,7 +703,7 @@ public class StatementTest
                 (
                  "setLargeMaxRows",
                  new Class[] { Long.TYPE },
-                 new Object[] { new Long( max ) }
+                 new Object[] { max }
                  );
         }
 
@@ -698,8 +735,8 @@ public class StatementTest
     /** Set the base which is used for returned row counts and fetched row counters */
     public  static  void    setRowCountBase( long newBase )
     {
-        EmbedResultSet.fetchedRowBase = newBase;
-        RowUtil.rowCountBase = newBase;
+        EmbedResultSet.setFetchedRowBase( newBase );
+        RowUtil.setRowCountBase( newBase );
     }
 
 } // End class StatementTest
