@@ -23,11 +23,8 @@ import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
-import java.util.Properties;
-
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.DatabasePropertyTestSetup;
 import org.apache.derbyTesting.junit.JDBC;
 import org.apache.derbyTesting.junit.TestConfiguration;
@@ -421,11 +418,12 @@ public class ConcurrencyTest extends SURBaseTest {
             ps2.executeUpdate();
             con2.commit();
             
-            ps2 = con2.prepareStatement("insert into t1 values(?,?,?,?)");
+            ps2 = con2.prepareStatement("insert into t1 values(?,?,?,?,?)");
             ps2.setInt(1, firstKey);
             ps2.setInt(2, -1);
             ps2.setInt(3, -1);
             ps2.setString(4, "UPDATED TUPLE");
+            ps2.setString(5, "UPDATED CLOB");
             assertEquals("Expected one record to be inserted", 1, 
                          ps2.executeUpdate());
             println("T4: Inserted record (" + firstKey + ",-1,-1)" );
@@ -505,11 +503,12 @@ public class ConcurrencyTest extends SURBaseTest {
             con2.commit();
             println("T3: commit");
             
-            ps2 = con2.prepareStatement("insert into t1 values(?,?,?,?)");
+            ps2 = con2.prepareStatement("insert into t1 values(?,?,?,?,?)");
             ps2.setInt(1, firstKey);
             ps2.setInt(2, valA);
             ps2.setInt(3, valB);
             ps2.setString(4, "UPDATE TUPLE " + firstKey);
+            ps2.setString(5, "UPDATED CLOB " + firstKey);
             assertEquals("Expected one record to be inserted", 1, 
                          ps2.executeUpdate());
             println("T4: Inserted record (" + firstKey + "," + valA + "," + 
@@ -849,7 +848,7 @@ public class ConcurrencyTest extends SURBaseTest {
      */
     public static Test suite()
     {
-        final TestSuite suite = new TestSuite("ConcurrencyTest");
+        final BaseTestSuite suite = new BaseTestSuite("ConcurrencyTest");
         suite.addTest(baseSuite("ConcurrencyTest:embedded", true));
         
         suite.addTest(
@@ -862,7 +861,7 @@ public class ConcurrencyTest extends SURBaseTest {
     }
     
     private static Test baseSuite(String name, boolean embedded) {
-        final TestSuite suite = new TestSuite(name);
+        final BaseTestSuite suite = new BaseTestSuite(name);
         
         // This testcase does not require JDBC3/JSR169, since it does not
         // specify result set concurrency) in Connection.createStatement().
