@@ -22,6 +22,7 @@
 package org.apache.derbyTesting.functionTests.tests.lang;
 
 import java.io.File;
+import java.net.URL;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
@@ -29,14 +30,12 @@ import java.sql.SQLException;
 import java.sql.SQLWarning;
 import java.util.Properties;
 import javax.sql.DataSource;
-
 import junit.framework.Test;
-import junit.framework.TestSuite;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.ClasspathSetup;
 import org.apache.derbyTesting.junit.DatabaseChangeSetup;
 import org.apache.derbyTesting.junit.JDBC;
 import org.apache.derbyTesting.junit.JDBCDataSource;
-import org.apache.derbyTesting.junit.SecurityManagerSetup;
 import org.apache.derbyTesting.junit.SupportFilesSetup;
 import org.apache.derbyTesting.junit.SystemPropertyTestSetup;
 import org.apache.derbyTesting.junit.TestConfiguration;
@@ -72,9 +71,6 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
     private static  final   boolean DISABLE_AUTHORIZATION = true;
     private static  final   boolean DONT_DISABLE_AUTH = false;
     
-    private static  final   boolean DISABLE_JAVA_SECURITY = true;
-    private static  final   boolean ENABLE_JAVA_SECURITY = false;
-    
     // fruits are legal users. nuts are not
     private static  final   String  DBO = "KIWI";   
     private static  final   String  APPLE_USER = "APPLE";   
@@ -157,7 +153,6 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
     private final   boolean _nativeAuthentication;
     private final   boolean _localAuthentication;
     private final   boolean _turnOffAuthenticationAndAuthorization;
-    private final   boolean _disableSecurityManager;
 
     private String  _credentialsDBPhysicalName;
 
@@ -183,8 +178,7 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
          int            credentialsDBLocation,
          boolean    nativeAuthentication,
          boolean    localAuthentication,
-         boolean    turnOffAuthenticationAndAuthorization,
-         boolean    disableSecurityManager
+         boolean    turnOffAuthenticationAndAuthorization
          )
     {
         super( "testAll" );
@@ -193,7 +187,6 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
         _nativeAuthentication = nativeAuthentication;
         _localAuthentication = localAuthentication;
         _turnOffAuthenticationAndAuthorization = turnOffAuthenticationAndAuthorization;
-        _disableSecurityManager = disableSecurityManager;
     }
 
     ///////////////////////////////////////////////////////////////////////////////////
@@ -328,14 +321,11 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
         String  authOverrides = _turnOffAuthenticationAndAuthorization ?
             "Authentication/Authorization turned OFF, " :
             "Authentication/Authorization DEFAULT, ";
-        String  securityManager = _disableSecurityManager ?
-            "SecurityManager OFF, " :
-            "SecurityManager ON, ";
         String  embedded = isEmbedded() ?
             "Embedded" :
             "Client/Server";
 
-        return "[ " + dbLocation + authType + local + authOverrides + securityManager + embedded + " ]";
+        return "[ " + dbLocation + authType + local + authOverrides + embedded + " ]";
     }
 
     /** Return true if the test is running embedded */
@@ -351,23 +341,22 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
     /**
      * Construct top level suite in this JUnit test
      */
-    public static Test suite()
+    public static Test suite() throws Exception
     {
-        TestSuite suite = new TestSuite();
+        BaseTestSuite suite = new BaseTestSuite();
 
         //
         // Special version of the test which uses an encrypted database for credentials.
-        // Not run on windows until DERBY-5618 is addressed.
-        // Also not run on small devices because encryption is not supported there
-        // by default.
+        // Only run on platforms where we can expect cleanup of support files
+        // to succeed (DERBY-5618).
         //
-        if ( !onWindows() && !JDBC.vmSupportsJSR169() )
+        if (enableSubprotocolTests())
         {
             suite.addTest
                 (
                  (
                   new NativeAuthenticationServiceTest
-                  ( JAR_ENCRYPTED, NATIVE, LOCAL, DONT_DISABLE_AUTH, ENABLE_JAVA_SECURITY )
+                  ( JAR_ENCRYPTED, NATIVE, LOCAL, DONT_DISABLE_AUTH )
                   ).decorate( false )
                  );
         }
@@ -380,9 +369,21 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
         // should be OK to run in the local time zone.
         return new TimeZoneTestSetup(suite, "GMT");
     }
-    private static  boolean onWindows()
-    {
-        return getSystemProperty("os.name").startsWith("Windows");
+
+    /**
+     * Check whether or not tests for subprotocols should be enabled
+     * on this platform.
+     */
+    private static boolean enableSubprotocolTests() {
+        // DERBY-5618: On Windows, we are not able to delete the jar files
+        // used by the tests for the classpath subsubprotocol if we don't
+        // close the URLClassLoader when we're done. Closing the class loader
+        // can only be done on Java 7 and higher.
+        //
+        // Run the tests if the Java version supports URLClassLoader.close(),
+        // or if we're on a non-Windows platform (where the jar files can be
+        // deleted even if the class loader hasn't been closed).
+        return ClasspathSetup.supportsClose() || !isWindowsPlatform();
     }
 
     /**
@@ -390,9 +391,9 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
      * Create a suite of all test configurations.
      * </p>
      */
-    private static  Test   allConfigurations( boolean clientServer )
+    private static Test allConfigurations(boolean clientServer) throws Exception
     {
-        TestSuite suite = new TestSuite();
+        BaseTestSuite suite = new BaseTestSuite();
 
         //
         // No authentication. 
@@ -401,7 +402,7 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
             (
              (
               new NativeAuthenticationServiceTest
-              ( NONE, NO_AUTH, SYSTEM_WIDE, DONT_DISABLE_AUTH, ENABLE_JAVA_SECURITY )
+              ( NONE, NO_AUTH, SYSTEM_WIDE, DONT_DISABLE_AUTH )
               ).decorate( clientServer )
              );
 
@@ -412,14 +413,14 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
             (
              (
               new NativeAuthenticationServiceTest
-              ( FILE, NATIVE, LOCAL, DISABLE_AUTHORIZATION, ENABLE_JAVA_SECURITY )
+              ( FILE, NATIVE, LOCAL, DISABLE_AUTHORIZATION )
               ).decorate( clientServer )
              );
         suite.addTest
             (
              (
               new NativeAuthenticationServiceTest
-              ( FILE, NATIVE, LOCAL, DONT_DISABLE_AUTH, ENABLE_JAVA_SECURITY )
+              ( FILE, NATIVE, LOCAL, DONT_DISABLE_AUTH )
               ).decorate( clientServer )
              );
 
@@ -430,21 +431,22 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
             (
              (
               new NativeAuthenticationServiceTest
-              ( FILE, NATIVE, SYSTEM_WIDE, DISABLE_AUTHORIZATION, ENABLE_JAVA_SECURITY )
+              ( FILE, NATIVE, SYSTEM_WIDE, DISABLE_AUTHORIZATION )
               ).decorate( clientServer )
              );
         suite.addTest
             (
              (
               new NativeAuthenticationServiceTest
-              ( FILE, NATIVE, SYSTEM_WIDE, DONT_DISABLE_AUTH, ENABLE_JAVA_SECURITY )
+              ( FILE, NATIVE, SYSTEM_WIDE, DONT_DISABLE_AUTH )
               ).decorate( clientServer )
              );
         
         //
-        // For testing subprotocols. Cleanup of support files is blocked by DERBY-5618.
+        // For testing subprotocols. Only run on platforms where we can
+        // expect cleanup of support files to succeed (DERBY-5618).
         //
-        if ( !onWindows() )
+        if (enableSubprotocolTests())
         {
             //
             // NATIVE authentication with credentials in read-only databases accessed via jar subprotocol
@@ -453,14 +455,14 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
                 (
                  (
                   new NativeAuthenticationServiceTest
-                  ( JAR, NATIVE, SYSTEM_WIDE, DONT_DISABLE_AUTH, ENABLE_JAVA_SECURITY )
+                  ( JAR, NATIVE, SYSTEM_WIDE, DONT_DISABLE_AUTH )
                   ).decorate( clientServer )
                  );
             suite.addTest
                 (
                  (
                   new NativeAuthenticationServiceTest
-                  ( JAR, NATIVE, LOCAL, DONT_DISABLE_AUTH, ENABLE_JAVA_SECURITY )
+                  ( JAR, NATIVE, LOCAL, DONT_DISABLE_AUTH )
                   ).decorate( clientServer )
                  );
 
@@ -471,17 +473,17 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
                 (
                  (
                   new NativeAuthenticationServiceTest
-                  ( CLASSPATH, NATIVE, SYSTEM_WIDE, DONT_DISABLE_AUTH, DISABLE_JAVA_SECURITY )
+                  ( CLASSPATH, NATIVE, SYSTEM_WIDE, DONT_DISABLE_AUTH )
                   ).decorate( clientServer )
                  );
             suite.addTest
                 (
                  (
                   new NativeAuthenticationServiceTest
-                  ( CLASSPATH, NATIVE, LOCAL, DONT_DISABLE_AUTH, DISABLE_JAVA_SECURITY )
+                  ( CLASSPATH, NATIVE, LOCAL, DONT_DISABLE_AUTH )
                   ).decorate( clientServer )
                  );
-        }   // end if !onWindows()
+        }
 
         return suite;
     }
@@ -493,14 +495,12 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
      * stored properties that can't be removed at tearDown time.
      * </p>
      */
-    private Test    decorate( boolean clientServer )
+    private Test decorate(boolean clientServer) throws Exception
     {
         String      credentialsDBPhysicalName = TestConfiguration.generateUniqueDatabaseName();
         
         Test        result = this;
 
-        if ( _disableSecurityManager ) { result = SecurityManagerSetup.noSecurityManager( result ); }
-
         //
         // Putting the clientServer decorator on the inside allows the server-side
         // embedded driver to be re-registered after engine shutdown. If you put
@@ -534,24 +534,17 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
         //    };
         
         //
-        // For testing subprotocols. Cleanup of support files is blocked by DERBY-5618.
+        // For testing subprotocols. Only run on platforms where we can
+        // expect cleanup of support files to succeed (DERBY-5618).
         //
-        if ( !onWindows() )
+        if (enableSubprotocolTests())
         {
+            // Add a jar file to the classpath so that we can test the classpath subprotocol.
+            URL nast2Jar = SupportFilesSetup.getReadOnlyURL(NAST2_JAR_FILE);
+            result = new ClasspathSetup(result, nast2Jar);
+
             // Add the jar files needed for testing jar and classpath subprotocols.
             result = new SupportFilesSetup( result, SUPPORT_FILES_SOURCE, null, SUPPORT_FILES_TARGET, null );
-
-            //
-            // Add a jar file to the classpath so that we can test the classpath subprotocol.
-            //
-            File        currentDirectory = currentDirectory();
-            File        readOnlyDirectory = new File( currentDirectory, SupportFilesSetup.EXTIN );
-            File        nast2Jar = new File( readOnlyDirectory, NAST2_JAR_FILE );
-
-            try {
-                result = new ClasspathSetup( result, nast2Jar.toURL() );
-            }
-            catch (Exception e) { printStackTrace( e ); }
         }
         
         //
@@ -866,21 +859,17 @@ public class NativeAuthenticationServiceTest extends GeneratedColumnsHelper
 
         ///////////////////////////////////////////////////////////////////////////////////
         //
-        // Vet databases accessed via subprotocols. Not run on Windows machines
-        // because of DERBY-5618.
+        // Vet databases accessed via subprotocols.
         //
         ///////////////////////////////////////////////////////////////////////////////////
 
-        if ( !onWindows() )
+        if (enableSubprotocolTests())
         {
             // database accessed via jar subprotocol
             vetProtocol( jarDBName( _credentialsDBLocation ) );
         
-            //
-            // We only use the classpath subprotocol if we are not running under a security manager.
-            // We may be able to remove that restriction after DERBY-5615 is fixed.
-            //
-            if ( _disableSecurityManager ) { vetProtocol( classpathDBName() ); }
+            // database accessed via classpath subprotocol
+            vetProtocol( classpathDBName() );
         }
         
         ///////////////////////////////////////////////////////////////////////////////////
