@@ -22,14 +22,14 @@ limitations under the License.
 package org.apache.derbyTesting.functionTests.tests.largedata;
 
 import java.io.File;
-import java.io.FileOutputStream;
 import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.FileReader;
 import java.io.FileWriter;
+import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.Reader;
-import java.io.FileReader;
-import java.io.IOException;
 import java.sql.Blob;
 import java.sql.Clob;
 import java.sql.PreparedStatement;
@@ -37,16 +37,15 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.util.Properties;
-
 import junit.framework.Test;
-
-import org.apache.derbyTesting.junit.BaseJDBCTestCase;
-import org.apache.derbyTesting.junit.CleanDatabaseTestSetup;
-import org.apache.derbyTesting.junit.SupportFilesSetup;
-import org.apache.derbyTesting.junit.TestConfiguration;
 import org.apache.derbyTesting.functionTests.util.PrivilegedFileOpsForTests;
+import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
+import org.apache.derbyTesting.junit.CleanDatabaseTestSetup;
 import org.apache.derbyTesting.junit.JDBC;
+import org.apache.derbyTesting.junit.SupportFilesSetup;
 import org.apache.derbyTesting.junit.SystemPropertyTestSetup;
+import org.apache.derbyTesting.junit.TestConfiguration;
 
 /**
  * This test is part of the "largedata" suite because this test tests data for
@@ -141,8 +140,10 @@ public class LobLimitsTest extends BaseJDBCTestCase {
         sysprops.setProperty("derby.locks.monitor", "true");
         // Some of the test cases depend on certain other test cases to run
         // first, so force the test cases to run in lexicographical order.
-        Test suite = new CleanDatabaseTestSetup(
-                TestConfiguration.orderedSuite(LobLimitsTest.class)) {
+        Test suite =
+                new CleanDatabaseTestSetup(
+                        new BaseTestSuite(LobLimitsTest.class)) {
+            @Override
             protected void decorateSQL(Statement s)
                            throws SQLException {
                 setupTables(s, biggestSize, bigSize);
@@ -1082,18 +1083,22 @@ public class LobLimitsTest extends BaseJDBCTestCase {
         long dlen = rs.getLong(2);
         assertEquals("FAIL - MISMATCH LENGTHS GOT " + l + " expected "
                 + dlen + " for row in CLOBTBL with ID=" + id, dlen, l);
-        // DERBY-5317 cannot use setCharacterStream with value from
-        // Clob.getCharacterStream because server will try to stream
-        // lob to and from server at the same time. setClob can be
-        // used as a work around.
-        if (!usingDerbyNetClient()) {
-            PreparedStatement psUpd =
-                    prepareStatement("update CLOBTBL set content=?, " +
-                            "dlen =? where id = ?");
-            psUpd.setCharacterStream(1, value.getCharacterStream(), (int) l);
-            psUpd.setLong(2, l);
-            psUpd.setInt(3, updateId);
-
+                
+        PreparedStatement psUpd =
+                prepareStatement("update CLOBTBL set content=?, "
+                + "dlen =? where id = ?");
+        psUpd.setCharacterStream(1, value.getCharacterStream(), (int) l);
+        psUpd.setLong(2, l);
+        psUpd.setInt(3, updateId);
+        if (usingDerbyNetClient()) {
+            // DERBY-5317 cannot use setCharacterStream with value from
+            // Clob.getCharacterStream because server will try to stream
+            // lob to and from server at the same time. setClob can be
+            // used as a work around.
+            // Verify that new error is thrown 
+            assertPreparedStatementError("XN023", psUpd);
+            return;
+        } else {
             assertUpdateCount(psUpd, 1);
         }
         commit();
@@ -1612,4 +1617,12 @@ class BlobImplT implements java.sql.Blob {
         throw new SQLException("Not implemented");
     }
 
+    public void free() throws SQLException {
+        throw new SQLException("Not implemented");
+    }
+
+    public InputStream getBinaryStream(long pos, long length) throws SQLException {
+        throw new SQLException("Not implemented");
+    }
+
 }
