@@ -25,25 +25,26 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.io.BufferedInputStream;
 import java.io.BufferedReader;
-import java.io.File;
-import java.io.FileNotFoundException;
+import java.io.PrintStream;
 import java.io.Reader;
 import java.io.UnsupportedEncodingException;
 import java.lang.reflect.Method;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.security.PrivilegedActionException;
 import java.net.URL;
 import java.sql.*;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Iterator;
 import java.util.List;
 
 import junit.framework.AssertionFailedError;
+import junit.framework.Test;
 
+import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
 import org.apache.derby.iapi.sql.execute.RunTimeStatistics;
 import org.apache.derby.impl.jdbc.EmbedConnection;
 import org.apache.derby.tools.ij;
-import org.apache.derbyTesting.functionTests.util.PrivilegedFileOpsForTests;
 import org.apache.derbyTesting.functionTests.util.TestNullOutputStream;
 
 
@@ -71,14 +72,14 @@ public abstract class BaseJDBCTestCase
      * were returned by utility methods and close
      * them at teardown.
      */
-    private List statements;
+    private List<Statement> statements;
 
     /**
      * Maintain a list of connection objects that
      * were returned by utility methods and close
      * them at teardown.
      */
-    private List connections;
+    private List<Connection> connections;
     
     /**
      * Create a test case with the given name.
@@ -158,7 +159,7 @@ public abstract class BaseJDBCTestCase
     private void addStatement(Statement s)
     {
         if (statements == null)
-            statements = new ArrayList();
+            statements = new ArrayList<Statement>();
         statements.add(s);
     }
     
@@ -169,7 +170,7 @@ public abstract class BaseJDBCTestCase
     private void addConnection(Connection c)
     {
         if (connections == null)
-            connections = new ArrayList();
+            connections = new ArrayList<Connection>();
         connections.add(c);     
     }
     
@@ -466,7 +467,7 @@ public abstract class BaseJDBCTestCase
 
     /**
      * Tear down this fixture, sub-classes should call
-     * super.tearDown(). This cleanups & closes the connection
+     * super.tearDown(). This cleans up and closes the connection
      * if it is open and any statement objects returned through
      * the utility methods.
      */
@@ -474,18 +475,14 @@ public abstract class BaseJDBCTestCase
     throws java.lang.Exception
     {
         if (statements != null) {
-            for (Iterator i = statements.iterator(); i.hasNext(); )
-            {
-                Statement s = (Statement) i.next();
+            for (Statement s : statements) {
                 s.close();
             }
             // Allow gc'ing of all those statements.
             statements = null;
         }
         if (connections != null) {
-            for (Iterator i = connections.iterator(); i.hasNext(); )
-            {
-                Connection c = (Connection) i.next();
+            for (Connection c : connections) {
                 JDBC.cleanup(c);
             }
             // Allow gc'ing of all those connections.
@@ -544,7 +541,7 @@ public abstract class BaseJDBCTestCase
      * specified user name and password.
      * <BR>
      * This connection is not
-     * automaticaly closed on tearDown, the test fixture must
+     * automatically closed on tearDown, the test fixture must
      * ensure the connection is closed.
      * 
      * The connection will be initialized by calling initializeConnection.
@@ -1186,9 +1183,9 @@ public abstract class BaseJDBCTestCase
     private static void assertStatementErrorMinion(
             String[] sqlStates, boolean orderedStates,
             Statement st, String query) {
-        ArrayList statesBag = null;
+        ArrayList<String> statesBag = null;
         if (!orderedStates) {
-            statesBag = new ArrayList(Arrays.asList(sqlStates));
+            statesBag = new ArrayList<String>(Arrays.asList(sqlStates));
         }
         try {
             boolean haveRS = st.execute(query);
@@ -1234,7 +1231,19 @@ public abstract class BaseJDBCTestCase
         assertStatementError(new String[] {sqlState},st,query);
     }
         
-   
+    /**
+     * Assert that that a commit fails with the given error
+     * @param sqlState state
+     * @param c        the connection
+     */
+    public static void assertCommitError(String sqlState, Connection c) {
+        try {
+            c.commit();
+            fail();
+        } catch (SQLException e) {
+            assertSQLState(sqlState, e);
+        }
+    }
    
     /**
      * Assert that the query fails (either in compilation,
@@ -1558,13 +1567,14 @@ public abstract class BaseJDBCTestCase
      * @throws SQLException
      */
     public static void checkEstimatedRowCount(Connection conn, double expectedCount) throws SQLException {
-	if (! (conn instanceof EmbedConnection))
-	    return;
+        if (! (conn instanceof EmbedConnection))
+	    { return; }
 	
-	EmbedConnection econn = (EmbedConnection) conn;
-	RunTimeStatistics rts = econn.getLanguageConnection().getRunTimeStatisticsObject();
-	assertNotNull(" RuntimeStatistics is null. Did you call SYSCS_UTIL.SYSCS_SET_RUNTIMESTATISTICS(1)?",rts);
-	assertEquals((long) expectedCount, (long) rts.getEstimatedRowCount());
+        EmbedConnection econn = (EmbedConnection) conn;
+        LanguageConnectionContext   lcc = (LanguageConnectionContext) getLanguageConnectionContext( econn );
+        RunTimeStatistics rts = lcc.getRunTimeStatisticsObject();
+        assertNotNull(" RuntimeStatistics is null. Did you call SYSCS_UTIL.SYSCS_SET_RUNTIMESTATISTICS(1)?",rts);
+        assertEquals((long) expectedCount, (long) rts.getEstimatedRowCount());
 	}
 
     /**
@@ -1605,6 +1615,250 @@ public abstract class BaseJDBCTestCase
 
         conn.commit();
     }
+
+    protected static void dumpRs(ResultSet s, PrintStream out)
+            throws SQLException
+    {
+        if (s == null) {
+            out.println("<NULL>");
+            return;
+        }
+
+        ResultSetMetaData rsmd = s.getMetaData();
+
+        // Get the number of columns in the result set
+        int numCols = rsmd.getColumnCount();
+
+        if (numCols <= 0) {
+            out.println("(no columns!)");
+            return;
+        }
+
+        StringBuilder heading = new StringBuilder("\t ");
+        StringBuilder underline = new StringBuilder("\t ");
+
+        int len;
+        // Display column headings
+        for (int i=1; i<=numCols; i++) {
+            if (i > 1) {
+                heading.append(",");
+                underline.append(" ");
+            }
+
+            len = heading.length();
+            heading.append(rsmd.getColumnLabel(i));
+            len = heading.length() - len;
+
+            for (int j = len; j > 0; j--) {
+                underline.append("-");
+            }
+        }
+
+        out.println(heading.toString());
+        out.println(underline.toString());
+
+
+        StringBuilder row = new StringBuilder();
+        // Display data, fetching until end of the result set
+        while (s.next()) {
+            row.append("\t{");
+            // Loop through each column, getting the
+            // column data and displaying
+            for (int i=1; i<=numCols; i++) {
+                if (i > 1) row.append(",");
+                row.append(s.getString(i));
+            }
+
+            row.append("}\n");
+        }
+
+        out.println(row.toString());
+        s.close();
+    }
+
+    public static void dumpRs(ResultSet s) throws SQLException {
+        dumpRs(s, System.out);
+    }
+
+    // helper methods moved from GeneratedColumnsHelper
+
+    /**
+     * Run good DDL.
+     * @throws SQLException 
+     */
+    protected void    goodStatement( Connection conn, String command ) throws SQLException
+    {
+        PreparedStatement    ps = chattyPrepare( conn, command );
+
+        ps.execute();
+        ps.close();
+    }
+    
+    /**
+     * Run a good update statement with an expected row count.
+     * @throws SQLException 
+     */
+    protected void    goodUpdate( Connection conn, String update, int expectedRowCount ) throws SQLException
+    {
+        PreparedStatement    ps = chattyPrepare( conn, update );
+
+        int actualRowCount = ps.executeUpdate();
+        ps.close();
+
+        println( "Expecting to touch " + expectedRowCount + " rows." );
+        assertEquals( expectedRowCount, actualRowCount );
+    }
+    
+    /**
+     * Assert that the statement returns the correct results.
+     */
+    protected void assertResults( Connection conn, String query, String[][] rows, boolean trimResults )
+        throws SQLException
+    {
+        PreparedStatement   ps = chattyPrepare( conn, query );
+        ResultSet                   rs = ps.executeQuery();
+
+        assertResults( rs, rows, trimResults );
+
+        rs.close();
+        ps.close();
+    }
+        
+    /**
+     * Assert that the ResultSet returns the desired rows.
+     */
+    protected void assertResults( ResultSet rs, String[][] rows, boolean trimResults )
+        throws SQLException
+    {
+        int     rowCount = rows.length;
+
+        for ( int i = 0; i < rowCount; i++ )
+        {
+            String[]    row = rows[ i ];
+            int             columnCount = row.length;
+
+            assertTrue( rs.next() );
+
+            for ( int j = 0; j < columnCount; j++ )
+            {
+                String  expectedValue =  row[ j ];
+                //println( "(row, column ) ( " + i + ", " +  j + " ) should be " + expectedValue );
+                String  actualValue = null;
+                int         column = j+1;
+
+                actualValue = rs.getString( column );
+                if ( rs.wasNull() ) { actualValue = null; }
+
+                if ( (actualValue != null) && trimResults ) { actualValue = actualValue.trim(); }
+                
+                assertEquals( (expectedValue == null), rs.wasNull() );
+                
+                if ( expectedValue == null )    { assertNull( actualValue ); }
+                else { assertEquals(expectedValue, actualValue); }
+            }
+        }
+
+        assertFalse( rs.next() );
+    }
+
+	protected	static	ResultSet	executeQuery( Statement stmt, String text )
+		throws SQLException
+	{
+		println( "Executing '" + text + "'" );
+
+        return stmt.executeQuery( text );
+	}
+
+    /**
+     * Prepare a statement and report its sql text.
+     */
+    protected PreparedStatement   chattyPrepare( Connection conn, String text )
+        throws SQLException
+    {
+        println( "Preparing statement:\n\t" + text );
+        
+        return conn.prepareStatement( text );
+    }
+
+    /**
+     * Prepare a callable statement and report its sql text.
+     */
+    protected CallableStatement   chattyPrepareCall( Connection conn, String text )
+        throws SQLException
+    {
+        println( "Preparing callable statement:\n\t" + text );
+        
+        return conn.prepareCall( text );
+    }
+
+    /**
+     * Assert that the statement text, when compiled, raises an exception
+     */
+    protected void    expectCompilationError( String sqlState, String query )
+    {
+        println( "\nExpecting " + sqlState + " when preparing:\n\t" + query );
+
+        assertCompileError( sqlState, query );
+    }
+
+    /**
+     * Assert that the statement text, when compiled, raises an exception
+     */
+    protected void    expectCompilationError( Connection conn, String sqlState, String query )
+    {
+        println( "\nExpecting " + sqlState + " when preparing:\n\t" + query );
+
+        PreparedStatement ps = null;
+
+        try {
+            ps = conn.prepareStatement( query );
+        } catch (SQLException se )
+        {
+            assertSQLState( sqlState, se );
+
+            return;
+        }
+
+        fail( "Expected SQL state: " + sqlState );
+    }
+
+    /**
+     * Assert that the statement text, when executed, raises an error.
+     */
+    protected void    expectExecutionError( Connection conn, String sqlState, String query )
+        throws Exception
+    {
+        println( "\nExpecting " + sqlState + " when executing:\n\t"  );
+        PreparedStatement   ps = chattyPrepare( conn, query );
+
+        assertStatementError( sqlState, ps );
+        ps.close();
+    }
+    
+	/**
+	  * Gets the LanguageConnectionContext for this connection. You might think that
+      * this method could take an EmbedConnection as its argument and return a
+      * LanguageConnectionContext. That, however, makes the compatibility tests blow up.
+      * With those stronger types, the test lookup machinery in junit.framework.TestSuite
+      * can't resolve the signature of this private method. That is because the engine jar is
+      * not on the client-only classpath used by the compatibility tests. Now you know.
+	  */
+	private static Object	getLanguageConnectionContext( Connection conn )
+	{
+        final EmbedConnection   econn = (EmbedConnection) conn;
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<LanguageConnectionContext>()
+             {
+                 public LanguageConnectionContext run()
+                 {
+                     return econn.getLanguageConnection();
+                 }
+             }
+             );
+	}
+
+
 } // End class BaseJDBCTestCase
 
 
