@@ -46,10 +46,13 @@ import org.apache.derby.iapi.types.SQLInteger;
 
 import java.security.AccessController;
 import java.security.PrivilegedAction;
+import java.security.PrivilegedActionException;
+import java.security.PrivilegedExceptionAction;
 import java.util.Properties;
 import java.util.Vector;
 import java.util.StringTokenizer;
 import java.io.File;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 /**
 
@@ -70,8 +73,8 @@ public class T_SortController extends T_Generic
 	}
 
 	private void setSortBufferSize(final String buf_length) {
-    	AccessController.doPrivileged(new PrivilegedAction() {
-		    public Object run()  {
+    	AccessController.doPrivileged(new PrivilegedAction<Void>() {
+		    public Void run()  {
 		    	System.setProperty("derby.storage.sortBufferMax", buf_length);
 		    	return null;
 		    }
@@ -133,7 +136,7 @@ public class T_SortController extends T_Generic
 		int failcount = 0;
 
 		try {
-			store = (AccessFactory) Monitor.createPersistentService(getModuleToTestProtocolName(), 
+			store = (AccessFactory) createPersistentService(getModuleToTestProtocolName(), 
 				testService + tail, startParams);
 		} catch (StandardException mse) {
 			throw T_Fail.exceptionFail(mse);
@@ -143,7 +146,7 @@ public class T_SortController extends T_Generic
 		}
 
 		tc = store.getTransaction(
-                ContextService.getFactory().getCurrentContextManager());
+                getContextService().getCurrentContextManager());
 
 		if (!sortExample(tc))
 			failcount++;
@@ -728,7 +731,7 @@ public class T_SortController extends T_Generic
         {
             int numMergeRuns = Integer.parseInt(sortprop.getProperty(
 			 MessageService.getTextMessage(SQLState.STORE_RTS_NUM_MERGE_RUNS)));
-            Vector mergeRuns = new Vector();
+            Vector<Integer> mergeRuns = new Vector<Integer>();
             StringTokenizer st = new StringTokenizer(sortprop.getProperty(
 			 MessageService.getTextMessage(SQLState.STORE_RTS_MERGE_RUNS_SIZE)),
 			 "[],",false);
@@ -742,7 +745,7 @@ public class T_SortController extends T_Generic
 
             int totRunSize = 0;
             for (int i = 0; i < mergeRuns.size(); i++)
-                totRunSize += ((Integer) mergeRuns.elementAt(i)).intValue();
+                totRunSize += mergeRuns.elementAt(i);
             if (totRunSize != numRowsInput)
                FAIL("(testSort) the sum of the elements of the vector SortInfo.mergeRunsSize (value: " +
                 totRunSize + " ) is not equal to SortInfo.numRowsInput (value: " +
@@ -837,18 +840,69 @@ public class T_SortController extends T_Generic
 
 		return (!mismatch && !toofew && !toomany);
 	}
+    
+    /**
+     * Privileged lookup of the ContextService. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ContextService    getContextService()
+    {
+        if ( System.getSecurityManager() == null )
+        {
+            return ContextService.getFactory();
+        }
+        else
+        {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedAction<ContextService>()
+                 {
+                     public ContextService run()
+                     {
+                         return ContextService.getFactory();
+                     }
+                 }
+                 );
+        }
+    }
+
+    /**
+     * Privileged startup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  Object createPersistentService( final String factoryInterface, final String serviceName, final Properties properties ) 
+        throws StandardException
+    {
+        try {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedExceptionAction<Object>()
+                 {
+                     public Object run()
+                         throws StandardException
+                     {
+                         return Monitor.createPersistentService( factoryInterface, serviceName, properties );
+                     }
+                 }
+                 );
+        } catch (PrivilegedActionException pae)
+        {
+            throw StandardException.plainWrapException( pae );
+        }
+    }
+
 }
 
 
 class T_DummySortObserver implements SortObserver
 {
 	T_AccessRow  template;
-	Vector vector;
+	Vector<DataValueDescriptor[]> vector;
 
 	T_DummySortObserver(T_AccessRow template)
 	{
 		this.template = template;
-		vector = new Vector();
+		vector = new Vector<DataValueDescriptor[]>();
 	}
 
 	/*
@@ -884,13 +938,25 @@ class T_DummySortObserver implements SortObserver
 
 		if (lastElement > 0)
 		{
-			DataValueDescriptor[] retval = 
-                (DataValueDescriptor[]) vector.elementAt(lastElement - 1);
-			vector.removeElementAt(lastElement - 1);
-			return retval;
+            return vector.remove(lastElement - 1);
 		}
 		return template.getRowArrayClone();
 	}
+
+    public boolean deferred() {
+        return false;
+    }
+
+    public boolean deferrable() {
+        return false;
+    }
+
+    public void rememberDuplicate(DataValueDescriptor[] row)
+            throws StandardException {
+        if (SanityManager.DEBUG) {
+            SanityManager.NOTREACHED();
+        }
+    }
 }
 
 class T_DuplicateEliminator extends T_DummySortObserver
@@ -965,5 +1031,20 @@ class T_SumForIntCol implements SortObserver
 	{
 		return null;
 	}
+
+    public boolean deferred() {
+        return false;
+    }
+
+    public boolean deferrable() {
+        return false;
+    }
+
+    public void rememberDuplicate(DataValueDescriptor[] row)
+            throws StandardException {
+        if (SanityManager.DEBUG) {
+            SanityManager.NOTREACHED();
+        }
+    }
 }
 
