@@ -106,7 +106,7 @@ public final class Main {
      * <code>main()</code>.
      */
     private final static LocalizedResource LOCALIZED_RESOURCE =
-        new LocalizedResource(null, null, MESSAGE_FILE);
+        new LocalizedResource(MESSAGE_FILE);
 
   /**
     Application entry point for SysInfo.   This will print out
@@ -232,9 +232,9 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
 	  String classpath = null;
 
       try {
-          classpath = (String) AccessController.doPrivileged( new PrivilegedAction()
+          classpath = AccessController.doPrivileged( new PrivilegedAction<String>()
               {
-                  public Object run()
+                  public String run()
                   {
                       return System.getProperty("java.class.path");
                   }
@@ -357,9 +357,9 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
     final   String unavailable = nullUnavailable ? null : Main.getTextMessage ("SIF01.H");
 
     try {
-        String  property = (String) AccessController.doPrivileged( new PrivilegedAction()
+        String  property = AccessController.doPrivileged( new PrivilegedAction<String>()
             {
-                public  Object  run()
+                public  String  run()
                 {
                     return System.getProperty (whichProperty, unavailable);
                 }
@@ -386,9 +386,9 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
     private static String getCanonicalPath(final File f) throws IOException {
 
         try {
-            return (String) AccessController
-                    .doPrivileged(new PrivilegedExceptionAction() {
-                        public Object run() throws IOException {
+            return AccessController
+                    .doPrivileged(new PrivilegedExceptionAction<String>() {
+                        public String run() throws IOException {
                             return f.getCanonicalPath();
                         }
                     });
@@ -457,9 +457,9 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
       final String finalLocaleResource = localeResource;
      
       try {     
-        	InputStream is = (InputStream) AccessController.doPrivileged
-            (new PrivilegedAction() {
-                  public Object run() {
+        	InputStream is = AccessController.doPrivileged
+            (new PrivilegedAction<InputStream>() {
+                  public InputStream run() {
   		            InputStream locis =
   		            	finalp.getClass().getResourceAsStream (finalLocaleResource);
   					return locis;
@@ -545,9 +545,9 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
       final Properties finalp = p;
       final String finalTstingResource = tstingResource;
       try {
-          InputStream is = (InputStream) AccessController.doPrivileged
-                  (new PrivilegedAction() {
-                      public Object run() {
+          InputStream is = AccessController.doPrivileged
+                  (new PrivilegedAction<InputStream>() {
+                      public InputStream run() {
                           InputStream is =
                                   finalp.getClass().getResourceAsStream (finalTstingResource);
                           return is;
@@ -568,7 +568,7 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
                   tryTstingClasspath(successes, failures);
                   String successString = successes.toString();
 
-                  if (successString == null || successString.length()<=2)
+                  if (successString.isEmpty() || successString.length()<=2)
                   {
                       // if we don't have the BaseTestCase class, assume we don't have any of the
                       // testing classes, and just print nothing
@@ -753,7 +753,8 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
 	}
 	
 	private static void tryTstingClasspath(StringBuffer successes, StringBuffer failures) {
-        tryMyClasspath("org.apache.derbyTesting.junit.BaseTestCase", "", successes, failures);
+        // use a class that is not dependent on junit
+        tryMyClasspath("org.apache.derbyTesting.junit.XATestUtil", "", successes, failures);
     }
 
 	private static void tryMyClasspath(String cn, String library, StringBuffer successes, StringBuffer failures) {
@@ -881,7 +882,11 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
 
                                    "org/apache/derby/info/" +
                                     org.apache.derby.iapi.services.info.ProductGenusNames.DNC +
-                                    ".properties"
+                                    ".properties",
+
+                                   "org/apache/derby/optional/" +
+                                    org.apache.derby.iapi.services.info.ProductGenusNames.OPTIONALTOOLS +
+                                    ".properties",
                                 };
 
     /**
@@ -915,7 +920,7 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
         {
 			if (classpath != null) {
 				String cp [] = parseClasspath(classpath);
-				Vector v = new Vector();
+				Vector<ZipInfoProperties> v = new Vector<ZipInfoProperties>();
 				for (int i = 0; i < cp.length; i++)
 				{
 					ZipInfoProperties zip = null;
@@ -964,15 +969,15 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
      */
     private static ZipInfoProperties [] loadZipFromResource()
     {
-		java.util.ArrayList al = new java.util.ArrayList();
+		java.util.ArrayList<ZipInfoProperties> al = new java.util.ArrayList<ZipInfoProperties>();
 
         for (int i = 0; i < infoNames.length; i++)
         {
             final String resource = "/".concat(infoNames[i]);
 
-            InputStream is = (InputStream) AccessController.doPrivileged
-            (new PrivilegedAction() {
-                public Object run() {
+            InputStream is = AccessController.doPrivileged
+            (new PrivilegedAction<InputStream>() {
+                public InputStream run() {
 			        InputStream locis =
                         new Main().getClass().getResourceAsStream(resource);
                             return locis;
@@ -986,9 +991,9 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
 			ZipInfoProperties ze = new ZipInfoProperties(ProductVersionHolder.getProductVersionHolderFromMyEnv(is));
  
                         // get the real location of the info file
-                        URL locUrl = (URL) AccessController.doPrivileged
-                        (new PrivilegedAction() {
-                            public Object run() {
+                        URL locUrl = AccessController.doPrivileged
+                        (new PrivilegedAction<URL>() {
+                            public URL run() {
                                 URL realUrl = new Main().getClass().getResource(resource);
                                 return realUrl;
                             }
@@ -1044,9 +1049,9 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
      */
     private static ZipInfoProperties checkForInfo(final String cpEntry)
     {
-        return (ZipInfoProperties) AccessController.doPrivileged( new PrivilegedAction()
+        return AccessController.doPrivileged( new PrivilegedAction<ZipInfoProperties>()
             {
-                public Object run()
+                public ZipInfoProperties run()
                 {
                     File f = new File(cpEntry);
                     if ( ! f.exists())
@@ -1169,26 +1174,7 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
 	** Message handling
 	*/
 
-	public static String getTextMessage(String msgId) {
-		return getCompleteMessage(msgId, new Object[0]);
-	}
-	public static String getTextMessage(String msgId, Object a1) {
-
-		return getCompleteMessage(msgId, new Object[] {a1});
-	}
-	public static String getTextMessage(String msgId, Object a1, Object a2) {
-		return getCompleteMessage(msgId, new Object[] {a1, a2});
-	}
-	public static String getTextMessage(String msgId, Object a1, Object a2, Object a3) {
-		return getCompleteMessage(msgId, new Object[] {a1, a2, a3});
-	}
-	public static String getTextMessage(String msgId, Object a1, Object a2, Object a3, Object a4) {
-		return getCompleteMessage(msgId, new Object[] {a1, a2, a3, a4});
-	}
-
-	/**
-	 */
-	public static String getCompleteMessage(String msgId, Object[] arguments) {
+    public static String getTextMessage(String msgId, Object... arguments) {
         return LOCALIZED_RESOURCE.getTextMessage(msgId, arguments);
 	}
 
@@ -1201,9 +1187,9 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
      */
     private static String getFileWhichLoadedClass(final Class cls)
     {
-         return (String)AccessController.doPrivileged( new PrivilegedAction()
+         return AccessController.doPrivileged( new PrivilegedAction<String>()
         {
-            public Object run()
+            public String run()
             {
                 CodeSource cs = null;
                 try {
@@ -1253,7 +1239,7 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
     private static ZipInfoProperties[] mergeZips(ZipInfoProperties[] zip1,
                                                  ZipInfoProperties[] zip2)
     {
-        Vector v = new Vector();
+        Vector<ZipInfoProperties> v = new Vector<ZipInfoProperties>();
         boolean foundDup = false;
   
         // remove duplicates from first array
@@ -1279,7 +1265,7 @@ public static void getMainInfo (java.io.PrintWriter aw, boolean pause) {
           {
             for (int k = 0; k < v.size(); k++)
             {
-                ZipInfoProperties z = (ZipInfoProperties)v.get(k);
+                ZipInfoProperties z = v.get(k);
                 if (zip2[j].getLocation().equals(z.getLocation()))
                   foundDup = true;
             }
